@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,8 @@ static ClipboardUtils *global_instance = nullptr;
  */
 static QString m_clipboard_parent_uri = nullptr;
 
+static QString m_last_target_directory_uri = nullptr;
+
 ClipboardUtils *ClipboardUtils::getInstance()
 {
     if (!global_instance) {
@@ -56,7 +58,7 @@ ClipboardUtils *ClipboardUtils::getInstance()
 ClipboardUtils::ClipboardUtils(QObject *parent) : QObject(parent)
 {
     connect(QApplication::clipboard(), &QClipboard::dataChanged, this, &ClipboardUtils::clipboardChanged);
-    connect(QApplication::clipboard(), &QClipboard::dataChanged, [=](){
+    connect(QApplication::clipboard(), &QClipboard::dataChanged, [=]() {
         auto data = QApplication::clipboard()->mimeData();
         if (!data->hasFormat("peony-qt/is-cut")) {
             m_clipboard_parent_uri = nullptr;
@@ -79,22 +81,38 @@ const QString ClipboardUtils::getClipedFilesParentUri()
     return m_clipboard_parent_uri;
 }
 
+const QString ClipboardUtils::getLastTargetDirectoryUri()
+{
+    return m_last_target_directory_uri;
+}
+
 void ClipboardUtils::setClipboardFiles(const QStringList &uris, bool isCut)
 {
     if (!global_instance) {
         global_instance = new ClipboardUtils;
     }
 
+    if (uris.isEmpty()) {
+        clearClipboard();
+        return;
+    }
+
     m_clipboard_parent_uri = FileUtils::getParentUri(uris.first());
 
+    // we should remain the encoded uri for file operation
     auto data = new QMimeData;
     QVariant isCutData = QVariant(isCut);
     data->setData("peony-qt/is-cut", isCutData.toByteArray());
     QList<QUrl> urls;
+    QStringList encodedUris;
     for (auto uri : uris) {
         urls<<uri;
+        encodedUris<<uri;
     }
     data->setUrls(urls);
+    QString string = encodedUris.join(" ");
+    data->setData("peony-qt/encoded-uris", string.toUtf8());
+    data->setText(string);
     QApplication::clipboard()->setMimeData(data);
 }
 
@@ -122,10 +140,23 @@ QStringList ClipboardUtils::getClipboardFilesUris()
     if (!isClipboardHasFiles()) {
         return l;
     }
-    auto urls = QApplication::clipboard()->mimeData()->urls();
-    for (auto url : urls) {
-        l<<url.toString();
+
+    auto mimeData = QApplication::clipboard()->mimeData();
+    //auto text = mimeData->text();
+    auto peonyText = mimeData->data("peony-qt/encoded-uris");
+
+    if (!peonyText.isEmpty()) {
+        auto byteArrays = peonyText.split(' ');
+        for (auto byteArray : byteArrays) {
+            l<<byteArray;
+        }
+    } else {
+        auto urls = mimeData->urls();
+        for (auto url : urls) {
+            l<<url.toString();
+        }
     }
+
     return l;
 }
 
@@ -156,6 +187,8 @@ void ClipboardUtils::pasteClipboardFiles(const QString &targetDirUri)
         auto copyOp = new FileCopyOperation(uris, targetDirUri);
         fileOpMgr->startOperation(copyOp, true);
     }
+
+    m_last_target_directory_uri = targetDirUri;
 }
 
 void ClipboardUtils::clearClipboard()
