@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -50,9 +50,23 @@ FileOperationUtils::FileOperationUtils()
 
 void FileOperationUtils::move(const QStringList &srcUris, const QString &destUri, bool addHistory, bool copyMove)
 {
+    FileOperation *op;
+    QString destDir = nullptr;
     auto fileOpMgr = FileOperationManager::getInstance();
     if (destUri != "trash:///") {
-        auto moveOp = new FileMoveOperation(srcUris, destUri);
+        if (true == destUri.startsWith("computer:///")) {
+            destDir = FileUtils::getTargetUri(destUri);
+            if (nullptr == destDir){
+                qWarning()<<"get target uri failed, from uri:"
+                          <<destUri;
+                destDir = destUri;
+            }
+        }
+        else {
+            destDir = destUri;
+        }
+
+        auto moveOp = new FileMoveOperation(srcUris, destDir);
         moveOp->setCopyMove(copyMove);
         fileOpMgr->startOperation(moveOp, addHistory);
     } else {
@@ -98,8 +112,8 @@ void FileOperationUtils::trash(const QStringList &uris, bool addHistory)
 
     if (canNotTrash) {
         auto result = QMessageBox::question(nullptr, QObject::tr("Can not trash"), QObject::tr("Can not trash these files. "
-                                                                                               "You can delete them permanently. "
-                                                                                               "Are you sure doing that?"));
+                                            "You can delete them permanently. "
+                                            "Are you sure doing that?"));
 
         if (result == QMessageBox::Yes) {
             FileOperationUtils::remove(uris);
@@ -173,13 +187,14 @@ void FileOperationUtils::executeRemoveActionWithDialog(const QStringList &uris)
 
     int result = 0;
     if (uris.count() == 1) {
+        QUrl url = uris.first();
         result = QMessageBox::question(nullptr, QObject::tr("Delete Permanently"), QObject::tr("Are you sure that you want to delete %1? "
-                                                                                               "Once you start a deletion, the files deleting will never be "
-                                                                                               "restored again.").arg(uris.first().split("/").last()));
+                                       "Once you start a deletion, the files deleting will never be "
+                                       "restored again.").arg(url.fileName()));
     } else {
         result = QMessageBox::question(nullptr, QObject::tr("Delete Permanently"), QObject::tr("Are you sure that you want to delete these %1 files? "
-                                                                                               "Once you start a deletion, the files deleting will never be "
-                                                                                               "restored again.").arg(uris.count()));
+                                       "Once you start a deletion, the files deleting will never be "
+                                       "restored again.").arg(uris.count()));
     }
 
     if (result == QMessageBox::Yes) {
