@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 #include "file-meta-info.h"
 
 #include "file-info-manager.h"
+#include "file-label-model.h"
 
 #include <gio/gdesktopappinfo.h>
 
@@ -33,6 +34,7 @@
 #include <QDateTime>
 #include <QIcon>
 #include <QUrl>
+#include <QLocale>
 
 using namespace Peony;
 
@@ -125,8 +127,8 @@ GAsyncReadyCallback FileInfoJob::query_info_async_callback(GFile *file, GAsyncRe
     GError *err = nullptr;
 
     GFileInfo *_info = g_file_query_info_finish(file,
-                                                res,
-                                                &err);
+                       res,
+                       &err);
 
     if (_info != nullptr) {
         thisJob->refreshInfoContents(_info);
@@ -191,7 +193,12 @@ void FileInfoJob::refreshInfoContents(GFileInfo *new_info)
     }
 
     info->m_is_symbol_link = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK);
-    info->m_can_read = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ);
+    if (g_file_info_has_attribute(new_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ)) {
+        info->m_can_read = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ);
+    } else {
+        // we assume an unknow access file is readable.
+        info->m_can_read = true;
+    }
     info->m_can_write = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
     info->m_can_excute = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE);
     info->m_can_delete = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE);
@@ -201,6 +208,8 @@ void FileInfoJob::refreshInfoContents(GFileInfo *new_info)
     info->m_can_mount = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_MOUNT);
     info->m_can_unmount = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_UNMOUNT);
     info->m_can_eject = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_EJECT);
+    info->m_can_start = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_START);
+    info->m_can_stop = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_STOP);
 
     info->m_is_virtual = g_file_info_get_attribute_boolean(new_info, G_FILE_ATTRIBUTE_STANDARD_IS_VIRTUAL);
 
@@ -232,17 +241,27 @@ void FileInfoJob::refreshInfoContents(GFileInfo *new_info)
         //g_object_unref(g_symbolic_icon);
     }
 
+    char* name = g_file_get_path(info->m_file);
+    info->m_path = name;
+    if (NULL != name) {
+        g_free(name);
+    }
     info->m_file_id = g_file_info_get_attribute_string(new_info, G_FILE_ATTRIBUTE_ID_FILE);
 
     info->m_content_type = g_file_info_get_content_type (new_info);
+    if (info->m_content_type == nullptr) {
+        if (g_file_info_has_attribute(new_info, "standard::fast-content-type")) {
+            info->m_content_type = g_file_info_get_attribute_string(new_info, "standard::fast-content-type");
+        }
+    }
+
     info->m_size = g_file_info_get_attribute_uint64(new_info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
     info->m_modified_time = g_file_info_get_attribute_uint64(new_info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
     info->m_access_time = g_file_info_get_attribute_uint64(new_info, G_FILE_ATTRIBUTE_TIME_ACCESS);
 
-    const char *content_type_str = g_file_info_get_content_type (new_info);
-    info->m_mime_type_string = content_type_str;
-    if (content_type_str) {
-        char *content_type = g_content_type_get_description (content_type_str);
+    info->m_mime_type_string = info->m_content_type;
+    if (!info->m_mime_type_string.isEmpty()) {
+        char *content_type = g_content_type_get_description (info->m_mime_type_string.toUtf8().constData());
         info->m_file_type = content_type;
         g_free (content_type);
         content_type = nullptr;
@@ -259,6 +278,8 @@ void FileInfoJob::refreshInfoContents(GFileInfo *new_info)
     info->m_access_date = date.toString(Qt::SystemLocaleShortDate);
 
     m_info->m_meta_info = FileMetaInfo::fromGFileInfo(m_info->uri(), new_info);
+    // update peony qt color list after meta info updated.
+    m_info->m_colors = FileLabelModel::getGlobalModel()->getFileColors(m_info->uri());
 
     if (info->isDesktopFile()) {
         QUrl url = info->uri();
@@ -268,15 +289,31 @@ void FileInfoJob::refreshInfoContents(GFileInfo *new_info)
             info->updated();
             return;
         }
+#if GLIB_CHECK_VERSION(2, 56, 0)
         auto string = g_desktop_app_info_get_locale_string(desktop_info, "Name");
+#else
+        //FIXME: should handle locale?
+        //change "Name" to QLocale::system().name(),
+        //try to fix Qt5.6 untranslated desktop file issue
+        auto key = "Name[" +  QLocale::system().name() + "]";
+        auto string = g_desktop_app_info_get_string(desktop_info, key.toUtf8().constData());
+#endif
+        qDebug() << "get name string:"<<string <<info->uri()<<info->displayName();
         if (string) {
             info->m_display_name = string;
             g_free(string);
         } else {
-            string = g_desktop_app_info_get_string(desktop_info, "Name");
-            if (string) {
-                info->m_display_name = string;
-                g_free(string);
+            QString path = "/usr/share/applications/" + info->displayName();
+            auto name = getAppName(path);
+            if (name.length() > 0)
+                info->m_display_name = name;
+            else
+            {
+                string = g_desktop_app_info_get_string(desktop_info, "Name");
+                if (string) {
+                    info->m_display_name = string;
+                    g_free(string);
+                }
             }
         }
         g_object_unref(desktop_info);
@@ -285,3 +322,20 @@ void FileInfoJob::refreshInfoContents(GFileInfo *new_info)
     Q_EMIT info->updated();
     m_info->m_mutex.unlock();
 }
+
+QString FileInfoJob::getAppName(QString desktopfp)
+{
+    GError** error=nullptr;
+    GKeyFileFlags flags=G_KEY_FILE_NONE;
+    GKeyFile* keyfile=g_key_file_new ();
+
+    QByteArray fpbyte=desktopfp.toLocal8Bit();
+    char* filepath=fpbyte.data();
+    g_key_file_load_from_file(keyfile,filepath,flags,error);
+
+    char* name=g_key_file_get_locale_string(keyfile,"Desktop Entry","Name", nullptr, nullptr);
+    QString namestr=QString::fromLocal8Bit(name);
+
+    g_key_file_free(keyfile);
+    return namestr;
+}
