@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -41,6 +41,7 @@ FileLauchDialog::FileLauchDialog(const QString &uri, QWidget *parent) : QDialog(
     m_layout = new QVBoxLayout(this);
     setLayout(m_layout);
 
+    setWindowTitle(tr("Applications"));
     m_layout->addWidget(new QLabel(tr("Choose an Application to open this file"), this));
     m_view = new QListWidget(this);
     m_view->setIconSize(QSize(48, 48));
@@ -68,7 +69,7 @@ FileLauchDialog::FileLauchDialog(const QString &uri, QWidget *parent) : QDialog(
         m_hash.insert(item, action);
     }
 
-    connect(this, &QDialog::accepted, [=](){
+    connect(this, &QDialog::accepted, [=]() {
         if (m_view->currentItem()) {
             auto action = m_hash.value(m_view->currentItem());
             if (m_check_box->isChecked()) {
@@ -82,4 +83,9 @@ FileLauchDialog::FileLauchDialog(const QString &uri, QWidget *parent) : QDialog(
 
     connect(m_button_box, &QDialogButtonBox::accepted, this, &QDialog::accept);
     connect(m_button_box, &QDialogButtonBox::rejected, this, &QDialog::reject);
+
+    auto info = FileInfo::fromUri(uri);
+    if (info->isDir() || info->isDesktopFile()) {
+        m_check_box->setEnabled(false);
+    }
 }
