@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -142,7 +142,7 @@ const QList<FileLaunchAction*> FileLaunchManager::getAllActions(const QString &u
     return actions;
 }
 
-void FileLaunchManager::openSync(const QString &uri)
+void FileLaunchManager::openSync(const QString &uri, bool forceWithArg, bool skipDialog)
 {
     QString tmp = uri;
     auto targetUri = FileUtils::getTargetUri(uri);
@@ -150,11 +150,11 @@ void FileLaunchManager::openSync(const QString &uri)
         tmp = targetUri;
     }
     auto action = getDefaultAction(tmp);
-    action->lauchFileSync();
+    action->lauchFileSync(forceWithArg, skipDialog);
     action->deleteLater();
 }
 
-void FileLaunchManager::openAsync(const QString &uri)
+void FileLaunchManager::openAsync(const QString &uri, bool forceWithArg, bool skipDialog)
 {
     QString tmp = uri;
     auto targetUri = FileUtils::getTargetUri(uri);
@@ -163,7 +163,20 @@ void FileLaunchManager::openAsync(const QString &uri)
         qDebug()<<"open async"<<targetUri;
     }
     auto action = getDefaultAction(tmp);
-    action->lauchFileAsync();
+    action->lauchFileAsync(forceWithArg, skipDialog);
+    action->deleteLater();
+}
+
+void FileLaunchManager::openAsync(const QStringList &files, bool forceWithArg, bool skipDialog)
+{
+    QString tmp = files.at(0);
+    auto targetUri = FileUtils::getTargetUri(tmp);
+    if (!targetUri.isNull()) {
+        tmp = targetUri;
+        qDebug()<<"open async"<<targetUri;
+    }
+    auto action = getDefaultAction(tmp);
+    action->lauchFilesAsync(files, forceWithArg, skipDialog);
     action->deleteLater();
 }
 
