@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,8 @@
 
 #include "file-operation-manager.h"
 
+#include "clipboard-utils.h"
+#include <QProcess>
 #include <QDebug>
 
 using namespace Peony;
@@ -94,9 +96,19 @@ static void handleDuplicate(FileNode *node) {
 
 FileCopyOperation::FileCopyOperation(QStringList sourceUris, QString destDirUri, QObject *parent) : FileOperation (parent)
 {
-    if (sourceUris.first().contains(destDirUri)) {
+    QUrl destDirUrl = destDirUri;
+    QUrl firstSrcUrl = sourceUris.first();
+    if (destDirUrl.isParentOf(firstSrcUrl)) {
         m_is_duplicated_copy = true;
+    } else {
+        auto lastPasteDirectoryUri = ClipboardUtils::getInstance()->getLastTargetDirectoryUri();
+        QUrl lastPasteDirectoryUrl = lastPasteDirectoryUri;
+        if (destDirUrl == lastPasteDirectoryUrl) {
+            m_is_duplicated_copy = true;
+        }
     }
+
+    m_conflict_files.clear();
     m_source_uris = sourceUris;
     m_dest_dir_uri = destDirUri;
     m_reporter = new FileNodeReporter;
@@ -108,11 +120,17 @@ FileCopyOperation::FileCopyOperation(QStringList sourceUris, QString destDirUri,
 FileCopyOperation::~FileCopyOperation()
 {
     delete m_reporter;
+    m_conflict_files.clear();
 }
 
-FileOperation::ResponseType FileCopyOperation::prehandle(GError *err)
+ExceptionResponse FileCopyOperation::prehandle(GError *err)
 {
     setHasError(true);
+
+    if (G_IO_ERROR_NO_SPACE == err->code) {
+        return Other;
+    }
+
     if (m_is_duplicated_copy)
         return BackupAll;
 
@@ -123,9 +141,12 @@ FileOperation::ResponseType FileCopyOperation::prehandle(GError *err)
 }
 
 void FileCopyOperation::progress_callback(goffset current_num_bytes,
-                                          goffset total_num_bytes,
-                                          FileCopyOperation *p_this)
+        goffset total_num_bytes,
+        FileCopyOperation *p_this)
 {
+    if (total_num_bytes < current_num_bytes)
+        return;
+
     auto currnet = p_this->m_current_offset + current_num_bytes;
     auto total = p_this->m_total_szie;
     qDebug()<<currnet*1.0/total;
@@ -143,7 +164,8 @@ void FileCopyOperation::copyRecursively(FileNode *node)
     node->setState(FileNode::Handling);
 
 fallback_retry:
-    QString destFileUri = node->resoveDestFileUri(m_dest_dir_uri);
+    QString destFileUri = node->resolveDestFileUri(m_dest_dir_uri);
+    QUrl destFileUrl = destFileUri;
     node->setDestUri(destFileUri);
     qDebug()<<"dest file uri:"<<destFileUri;
 
@@ -160,16 +182,29 @@ fallback_retry:
                               getCancellable().get()->get(),
                               &err);
         if (err) {
+            FileOperationError except;
             if (err->code == G_IO_ERROR_CANCELLED) {
                 return;
             }
             auto errWrapperPtr = GErrorWrapper::wrapFrom(err);
-            ResponseType handle_type = prehandle(err);
+            int handle_type = prehandle(err);
+            except.errorType = ET_GIO;
+            except.srcUri = m_current_src_uri;
+            except.destDirUri = m_current_dest_dir_uri;
+            except.title = tr("File copy");
+            except.errorCode = err->code;
             if (handle_type == Other) {
-                qDebug()<<"send error";
-                auto typeData = errored(m_current_src_uri, m_current_dest_dir_uri, errWrapperPtr);
-                qDebug()<<"get return";
-                handle_type = typeData.value<ResponseType>();
+                if (G_IO_ERROR_EXISTS == err->code) {
+                    except.dlgType = ED_CONFLICT;
+                    Q_EMIT errored(except);
+                    auto typeData = except.respCode;
+                    handle_type = typeData;
+                } else {
+                    except.dlgType = ED_WARNING;
+                    Q_EMIT errored(except);
+                    auto typeData = except.respCode;
+                    handle_type = typeData;
+                }
             }
             //handle.
             switch (handle_type) {
@@ -199,7 +234,22 @@ fallback_retry:
             case BackupOne: {
                 node->setState(FileNode::Handled);
                 node->setErrorResponse(BackupOne);
-                while (FileUtils::isFileExsit(node->resoveDestFileUri(m_dest_dir_uri))) {
+                // use custom name
+                QString name = "";
+                QStringList extendStr = node->destBaseName().split(".");
+                if (extendStr.length() > 0) {
+                    extendStr.removeAt(0);
+                }
+                QString endStr = extendStr.join(".");
+                if (except.respValue.contains("name")) {
+                    name = except.respValue["name"].toString();
+                    if (endStr != "" && name.endsWith(endStr)) {
+                        node->setDestFileName(name);
+                    } else if ("" != endStr && "" != name) {
+                        node->setDestFileName(name + "." + endStr);
+                    }
+                }
+                while (FileUtils::isFileExsit(node->resolveDestFileUri(m_dest_dir_uri))) {
                     handleDuplicate(node);
                 }
                 goto fallback_retry;
@@ -207,7 +257,7 @@ fallback_retry:
             case BackupAll: {
                 node->setState(FileNode::Handled);
                 node->setErrorResponse(BackupOne);
-                while (FileUtils::isFileExsit(node->resoveDestFileUri(m_dest_dir_uri))) {
+                while (FileUtils::isFileExsit(node->resolveDestFileUri(m_dest_dir_uri))) {
                     handleDuplicate(node);
                 }
                 //make dir has no backup
@@ -246,16 +296,39 @@ fallback_retry:
                     &err);
 
         if (err) {
+            FileOperationError except;
             if (err->code == G_IO_ERROR_CANCELLED) {
                 return;
             }
+            if (err->code == G_IO_ERROR_EXISTS) {
+                char* destFileName = g_file_get_uri(destFile.get()->get());
+                if (NULL != destFileName) {
+                    m_conflict_files << destFileName;
+                    g_free(destFileName);
+                }
+            }
             auto errWrapperPtr = GErrorWrapper::wrapFrom(err);
-            ResponseType handle_type = prehandle(err);
+            int handle_type = prehandle(err);
+            except.errorType = ET_GIO;
+            except.title = tr("File copy");
+            except.srcUri = m_current_src_uri;
+            except.errorCode = err->code;
+            except.errorStr = err->message;
+            except.destDirUri = m_current_dest_dir_uri;
             if (handle_type == Other) {
-                qDebug()<<"send error";
-                auto typeData = errored(m_current_src_uri, m_current_dest_dir_uri, errWrapperPtr);
-                qDebug()<<"get return";
-                handle_type = typeData.value<ResponseType>();
+                if (G_IO_ERROR_EXISTS == err->code) {
+                    except.dlgType = ED_CONFLICT;
+                    Q_EMIT errored(except);
+                    auto typeData = except.respCode;
+                    qDebug()<<"get return";
+                    handle_type = typeData;
+                } else {
+                    except.dlgType = ED_WARNING;
+                    Q_EMIT errored(except);
+                    auto typeData = except.respCode;
+                    qDebug()<<"get return";
+                    handle_type = typeData;
+                }
             }
             //handle.
             switch (handle_type) {
@@ -298,7 +371,23 @@ fallback_retry:
             case BackupOne: {
                 node->setState(FileNode::Handled);
                 node->setErrorResponse(BackupOne);
-                while (FileUtils::isFileExsit(node->resoveDestFileUri(m_dest_dir_uri))) {
+                // use custom name
+                QString name = "";
+                QStringList extendStr = node->destBaseName().split(".");
+                if (extendStr.length() > 0) {
+                    extendStr.removeAt(0);
+                }
+                QString endStr = extendStr.join(".");
+                if (except.respValue.contains("name")) {
+                    name = except.respValue["name"].toString();
+                    if (endStr != "" && name.endsWith(endStr)) {
+                        node->setDestFileName(name);
+                    } else if ("" != endStr && "" != name) {
+                        node->setDestFileName(name + "." + endStr);
+                    }
+                }
+
+                while (FileUtils::isFileExsit(node->resolveDestFileUri(m_dest_dir_uri))) {
                     handleDuplicate(node);
                 }
                 goto fallback_retry;
@@ -306,7 +395,7 @@ fallback_retry:
             case BackupAll: {
                 node->setState(FileNode::Handled);
                 node->setErrorResponse(BackupOne);
-                while (FileUtils::isFileExsit(node->resoveDestFileUri(m_dest_dir_uri))) {
+                while (FileUtils::isFileExsit(node->resolveDestFileUri(m_dest_dir_uri))) {
                     handleDuplicate(node);
                 }
                 m_prehandle_hash.insert(err->code, BackupOne);
@@ -327,6 +416,7 @@ fallback_retry:
             node->setState(FileNode::Handled);
         }
         m_current_offset += node->size();
+
         Q_EMIT operationProgressedOne(node->uri(), node->destUri(), node->size());
     }
     destFile.reset();
@@ -337,11 +427,6 @@ void FileCopyOperation::rollbackNodeRecursively(FileNode *node)
     switch (node->state()) {
     case FileNode::Handling:
     case FileNode::Handled: {
-        //do not clear the dest file if ignored or overwrite or backuped.
-        //FIXME: should i put it after recusion?
-        if (node->responseType() != Other)
-            break;
-
         if (node->isFolder()) {
             auto children = node->children();
             for (auto child : *children) {
@@ -367,9 +452,11 @@ void FileCopyOperation::rollbackNodeRecursively(FileNode *node)
             }
             g_object_unref(dest_file);
         } else {
-            GFile *dest_file = g_file_new_for_uri(node->destUri().toUtf8().constData());
-            g_file_delete(dest_file, nullptr, nullptr);
-            g_object_unref(dest_file);
+            if (!m_conflict_files.contains(node->destUri())) {
+                GFile *dest_file = g_file_new_for_uri(node->destUri().toUtf8().constData());
+                g_file_delete(dest_file, nullptr, nullptr);
+                g_object_unref(dest_file);
+            }
         }
         operationRollbackedOne(node->destUri(), node->uri());
         break;
@@ -403,8 +490,9 @@ void FileCopyOperation::run()
         FileNode *node = new FileNode(uri, nullptr, m_reporter);
         node->findChildrenRecursively();
         node->computeTotalSize(total_size);
-        nodes<<node;
+        nodes << node;
     }
+
     Q_EMIT operationPrepared();
 
     m_total_szie = *total_size;
@@ -425,13 +513,50 @@ void FileCopyOperation::run()
         }
     }
 
+    setHasError(false);
+
     for (auto node : nodes) {
+        if (!isCancelled())
+            m_info->m_node_map.insert(node->uri(), node->destUri());
         delete node;
     }
 
     nodes.clear();
 
+    // judge if the operation should sync.
+    bool needSync = false;
+    GFile *src_first_file = g_file_new_for_uri(m_source_uris.first().toUtf8().constData());
+    GMount *src_first_mount = g_file_find_enclosing_mount(src_first_file, nullptr, nullptr);
+    if (src_first_mount) {
+        needSync = g_mount_can_unmount(src_first_mount);
+        g_object_unref(src_first_mount);
+    } else {
+        // maybe a vfs file.
+        needSync = true;
+    }
+    g_object_unref(src_first_file);
+
+    GFile *dest_dir_file = g_file_new_for_uri(m_dest_dir_uri.toUtf8().constData());
+    GMount *dest_dir_mount = g_file_find_enclosing_mount(dest_dir_file, nullptr, nullptr);
+    if (src_first_mount) {
+        needSync = g_mount_can_unmount(dest_dir_mount);
+        g_object_unref(dest_dir_mount);
+    } else {
+        needSync = true;
+    }
+    g_object_unref(dest_dir_file);
+
+    //needSync = true;
+
+    if (needSync) {
+        operationStartSnyc();
+        QProcess p;
+        p.start("sync");
+        p.waitForFinished(-1);
+    }
+
     Q_EMIT operationFinished();
+    //notifyFileWatcherOperationFinished();
 }
 
 void FileCopyOperation::cancel()
