@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,8 @@
 
 #include "file-operation-manager.h"
 
+#include <QProcess>
+
 using namespace Peony;
 
 static void handleDuplicate(FileNode *node) {
@@ -101,9 +103,12 @@ FileMoveOperation::~FileMoveOperation()
 }
 
 void FileMoveOperation::progress_callback(goffset current_num_bytes,
-                                          goffset total_num_bytes,
-                                          FileMoveOperation *p_this)
+        goffset total_num_bytes,
+        FileMoveOperation *p_this)
 {
+    if (total_num_bytes < current_num_bytes)
+        return;
+
     auto currnet = p_this->m_current_offset + current_num_bytes;
     auto total = p_this->m_total_szie;
     Q_EMIT p_this->FileProgressCallback(p_this->m_current_src_uri,
@@ -113,7 +118,7 @@ void FileMoveOperation::progress_callback(goffset current_num_bytes,
     //format: move srcUri to destDirUri: curent_bytes(count) of total_bytes(count).
 }
 
-FileOperation::ResponseType FileMoveOperation::prehandle(GError *err)
+ExceptionResponse FileMoveOperation::prehandle(GError *err)
 {
     setHasError(true);
     if (m_prehandle_hash.contains(err->code))
@@ -150,7 +155,7 @@ void FileMoveOperation::move()
         auto srcFile = wrapGFile(g_file_new_for_uri(srcUri.toUtf8().constData()));
         char *base_name = g_file_get_basename(srcFile.get()->get());
         auto destFile = wrapGFile(g_file_resolve_relative_path(destDir.get()->get(),
-                                                               base_name));
+                                  base_name));
 
         char *dest_uri = g_file_get_uri(destFile.get()->get());
         file->setDestUri(dest_uri);
@@ -187,12 +192,26 @@ retry:
             default:
                 break;
             }
-            ResponseType handle_type = prehandle(err);
+            int handle_type = prehandle(err);
+            FileOperationError except;
+            except.srcUri = srcUri;
+            except.destDirUri = m_dest_dir_uri;
+            except.isCritical = false;
+            except.title = tr("Move file");
+            except.errorCode = err->code;
+            except.errorType = ET_GIO;
             if (handle_type == Other) {
-                qDebug()<<"send error";
-                auto responseTypeWrapper = Q_EMIT errored(srcUri, m_dest_dir_uri, errWrapper);
-                qDebug()<<"get return";
-                handle_type = responseTypeWrapper.value<ResponseType>();
+                auto responseTypeWrapper = Invalid;
+                if (G_IO_ERROR_EXISTS == err->code) {
+                    except.dlgType = ED_CONFLICT;
+                    Q_EMIT errored(except);
+                    responseTypeWrapper = except.respCode;
+                } else {
+                    except.dlgType = ED_WARNING;
+                    Q_EMIT errored(except);
+                    responseTypeWrapper = except.respCode;
+                }
+                handle_type = responseTypeWrapper;
                 //block until error has been handled.
             }
 
@@ -200,19 +219,19 @@ retry:
             switch (handle_type) {
             case IgnoreOne: {
                 file->setState(FileNode::Unhandled);
-                file->setErrorResponse(FileOperation::IgnoreOne);
+                file->setErrorResponse(IgnoreOne);
                 //skip to next loop.
                 break;
             }
             case IgnoreAll: {
                 file->setState(FileNode::Unhandled);
-                file->setErrorResponse(FileOperation::IgnoreOne);
+                file->setErrorResponse(IgnoreOne);
                 m_prehandle_hash.insert(err->code, IgnoreOne);
                 break;
             }
             case OverWriteOne: {
                 file->setState(FileNode::Handled);
-                file->setErrorResponse(FileOperation::OverWriteOne);
+                file->setErrorResponse(OverWriteOne);
                 g_file_move(srcFile.get()->get(),
                             destFile.get()->get(),
                             GFileCopyFlags(G_FILE_COPY_NOFOLLOW_SYMLINKS|
@@ -226,7 +245,7 @@ retry:
             }
             case OverWriteAll: {
                 file->setState(FileNode::Handled);
-                file->setErrorResponse(FileOperation::OverWriteOne);
+                file->setErrorResponse(OverWriteOne);
                 g_file_move(srcFile.get()->get(),
                             destFile.get()->get(),
                             GFileCopyFlags(G_FILE_COPY_NOFOLLOW_SYMLINKS|
@@ -241,9 +260,26 @@ retry:
             }
             case BackupOne: {
                 file->setState(FileNode::Handled);
-                file->setErrorResponse(FileOperation::BackupOne);
-                handleDuplicate(file);
-                auto handledDestFileUri = file->resoveDestFileUri(m_dest_dir_uri);
+                file->setErrorResponse(BackupOne);
+                // use custom name
+                QString name = "";
+                QStringList extendStr = file->destBaseName().split(".");
+                if (extendStr.length() > 0) {
+                    extendStr.removeAt(0);
+                }
+                QString endStr = extendStr.join(".");
+                if (except.respValue.contains("name")) {
+                    name = except.respValue["name"].toString();
+                    if (endStr != "" && name.endsWith(endStr)) {
+                        file->setDestFileName(name);
+                    } else if ("" != endStr && "" != name) {
+                        file->setDestFileName(name + "." + endStr);
+                    }
+                }
+                if (FileUtils::isFileExsit(file->destUri())) {
+                    handleDuplicate(file);
+                }
+                auto handledDestFileUri = file->resolveDestFileUri(m_dest_dir_uri);
                 auto handledDestFile = wrapGFile(g_file_new_for_uri(handledDestFileUri.toUtf8()));
                 g_file_copy(srcFile.get()->get(),
                             handledDestFile.get()->get(),
@@ -256,8 +292,8 @@ retry:
             }
             case BackupAll: {
                 file->setState(FileNode::Handled);
-                file->setErrorResponse(FileOperation::BackupOne);
-                auto handledDestFileUri = file->resoveDestFileUri(m_dest_dir_uri);
+                file->setErrorResponse(BackupOne);
+                auto handledDestFileUri = file->resolveDestFileUri(m_dest_dir_uri);
                 auto handledDestFile = wrapGFile(g_file_new_for_uri(handledDestFileUri.toUtf8()));
                 g_file_copy(srcFile.get()->get(),
                             handledDestFile.get()->get(),
@@ -281,9 +317,24 @@ retry:
                 break;
             }
 
+            except.srcUri = srcUri;
+            except.errorType = ET_GIO;
+            except.errorCode = err->code;
+            except.title = tr("Move file");
+            except.destDirUri = m_dest_dir_uri;
+            except.isCritical = true;
             if (handled_err) {
                 auto handledErr = GErrorWrapper::wrapFrom(handled_err);
-                this->errored(srcUri, m_dest_dir_uri, handledErr, true);
+                FileOperationError except;
+                if (G_IO_ERROR_EXISTS == handled_err->code) {
+                    except.dlgType = ED_CONFLICT;
+                    Q_EMIT errored(except);
+                } else {
+                    except.dlgType = ED_WARNING;
+                    Q_EMIT errored(except);
+                }
+
+                auto response = except.respCode;
             }
         } else {
             file->setState(FileNode::Handled);
@@ -480,7 +531,7 @@ void FileMoveOperation::copyRecursively(FileNode *node)
     //FIXME: the smart pointers' deconstruction spends too much time.
     GFileWrapperPtr destRoot = wrapGFile(g_file_new_for_uri(m_dest_dir_uri.toUtf8().constData()));
     GFileWrapperPtr destFile = wrapGFile(g_file_resolve_relative_path(destRoot.get()->get(),
-                                                                      relativePath.toUtf8().constData()));
+                                         relativePath.toUtf8().constData()));
 
     char *dest_file_uri = g_file_get_uri(destFile.get()->get());
     node->setDestUri(dest_file_uri);
@@ -505,16 +556,35 @@ fallback_retry:
                               getCancellable().get()->get(),
                               &err);
         if (err) {
+            FileOperationError except;
             if (err->code == G_IO_ERROR_CANCELLED) {
                 return;
             }
             auto errWrapperPtr = GErrorWrapper::wrapFrom(err);
-            ResponseType handle_type = prehandle(err);
+            int handle_type = prehandle(err);
+            except.errorType = ET_GIO;
+            except.title = tr("Move file");
+            except.errorCode = err->code;
+            except.errorStr = err->message;
+            except.srcUri = m_current_src_uri;
+            except.destDirUri = m_current_dest_dir_uri;
+            except.isCritical = false;
             if (handle_type == Other) {
-                qDebug()<<"send error";
-                auto typeData = errored(m_current_src_uri, m_current_dest_dir_uri, errWrapperPtr);
-                qDebug()<<"get return";
-                handle_type = typeData.value<ResponseType>();
+                auto typeData = Invalid;
+                if (G_IO_ERROR_EXISTS == err->code) {
+                    except.dlgType = ED_CONFLICT;
+                    Q_EMIT errored(except);
+                    typeData = except.respCode;
+                } else {
+                    except.dlgType = ED_WARNING;
+                    Q_EMIT errored(except);
+                    typeData = except.respCode;
+                }
+                // ignore multiple bounces
+                if (except.errorCode == G_IO_ERROR_NOT_SUPPORTED) {
+                    m_prehandle_hash.insert(err->code, IgnoreOne);
+                }
+                handle_type = typeData;
             }
             //handle.
             switch (handle_type) {
@@ -544,7 +614,26 @@ fallback_retry:
             case BackupOne: {
                 node->setState(FileNode::Handled);
                 node->setErrorResponse(BackupOne);
-                //make dir has no backup
+                // use custom name
+                QString name = "";
+                QStringList extendStr = node->destBaseName().split(".");
+                if (extendStr.length() > 0) {
+                    extendStr.removeAt(0);
+                }
+                QString endStr = extendStr.join(".");
+                if (except.respValue.contains("name")) {
+                    name = except.respValue["name"].toString();
+                    if (endStr != "" && name.endsWith(endStr)) {
+                        node->setDestFileName(name);
+                    } else if ("" != endStr && "" != name) {
+                        node->setDestFileName(name + "." + endStr);
+                    } else if ("" == endStr) {
+                        node->setDestFileName(name);
+                    }
+                }
+                if (FileUtils::isFileExsit(node->destUri())) {
+                    handleDuplicate(node);
+                }
                 break;
             }
             case BackupAll: {
@@ -590,16 +679,31 @@ fallback_retry:
                     &err);
 
         if (err) {
+            FileOperationError except;
             if (err->code == G_IO_ERROR_CANCELLED) {
                 return;
             }
             auto errWrapperPtr = GErrorWrapper::wrapFrom(err);
-            ResponseType handle_type = prehandle(err);
+            int handle_type = prehandle(err);
+            except.isCritical = true;
+            except.errorType = ET_GIO;
+            except.errorCode = err->code;
+            except.errorStr = err->message;
+            except.title = tr("Create file");
+            except.srcUri = m_current_src_uri;
+            except.destDirUri = m_current_dest_dir_uri;
             if (handle_type == Other) {
-                qDebug()<<"send error";
-                auto typeData = errored(m_current_src_uri, m_current_dest_dir_uri, errWrapperPtr);
-                qDebug()<<"get return";
-                handle_type = typeData.value<ResponseType>();
+                auto typeData = Invalid;
+                if (G_IO_ERROR_EXISTS == err->code) {
+                    except.dlgType = ED_CONFLICT;
+                    Q_EMIT errored(except);
+                    typeData = except.respCode;
+                } else {
+                    except.dlgType = ED_WARNING;
+                    Q_EMIT errored(except);
+                    typeData = except.respCode;
+                }
+                handle_type = typeData;
             }
             //handle.
             switch (handle_type) {
@@ -640,8 +744,25 @@ fallback_retry:
                 break;
             }
             case BackupOne: {
-                handleDuplicate(node);
-                auto handledDestFileUri = node->resoveDestFileUri(m_dest_dir_uri);
+                // use custom name
+                QString name = "";
+                QStringList extendStr = node->destBaseName().split(".");
+                if (extendStr.length() > 0) {
+                    extendStr.removeAt(0);
+                }
+                QString endStr = extendStr.join(".");
+                if (except.respValue.contains("name")) {
+                    name = except.respValue["name"].toString();
+                    if (endStr != "" && name.endsWith(endStr)) {
+                        node->setDestFileName(name);
+                    } else if ("" != endStr && "" != name) {
+                        node->setDestFileName(name + "." + endStr);
+                    }
+                }
+                if (FileUtils::isFileExsit(node->destUri())) {
+                    handleDuplicate(node);
+                }
+                auto handledDestFileUri = node->resolveDestFileUri(m_dest_dir_uri);
                 auto handledDestFile = wrapGFile(g_file_new_for_uri(handledDestFileUri.toUtf8()));
                 g_file_copy(sourceFile.get()->get(),
                             handledDestFile.get()->get(),
@@ -656,7 +777,7 @@ fallback_retry:
             }
             case BackupAll: {
                 handleDuplicate(node);
-                auto handledDestFileUri = node->resoveDestFileUri(m_dest_dir_uri);
+                auto handledDestFileUri = node->resolveDestFileUri(m_dest_dir_uri);
                 auto handledDestFile = wrapGFile(g_file_new_for_uri(handledDestFileUri.toUtf8()));
                 g_file_copy(sourceFile.get()->get(),
                             handledDestFile.get()->get(),
@@ -757,7 +878,7 @@ void FileMoveOperation::moveForceUseFallback()
 
     for (auto file : nodes) {
         qDebug()<<file->uri();
-        if (isCancelled()) {
+        if (isCancelled() && !hasError()) {
             rollbackNodeRecursively(file);
         }
     }
@@ -771,14 +892,15 @@ void FileMoveOperation::moveForceUseFallback()
 
 bool FileMoveOperation::isValid()
 {
+    int index = 0;
     bool isInvalid = false;
     for (auto srcUri : m_source_uris) {
         auto srcFile = wrapGFile(g_file_new_for_uri(srcUri.toUtf8().constData()));
         auto destFile = wrapGFile(g_file_new_for_uri(m_dest_dir_uri.toUtf8().constData()));
         auto parentFile = wrapGFile(g_file_get_parent(srcFile.get()->get()));
         if (g_file_equal(destFile.get()->get(), parentFile.get()->get())) {
-            isInvalid = true;
-            invalidOperation(tr("Invalid move operation, cannot move a file itself."));
+            m_source_uris.removeAt(index);
+            --index;
         }
         //BUG: some special basename like test and test2, will lead the operation invalid.
         /*
@@ -789,6 +911,7 @@ bool FileMoveOperation::isValid()
         */
         //FIXME: find if destUriDirFile is srcFile's child.
         //it will call G_IO_ERROR_INVALID_FILENAME
+        ++index;
     }
     if (isInvalid)
         invalidExited(tr("Invalid Operation."));
@@ -800,14 +923,17 @@ void FileMoveOperation::run()
     Q_EMIT operationStarted();
 start:
     if (!isValid()) {
-        auto response = errored(nullptr,
-                                nullptr,
-                                GErrorWrapper::wrapFrom(g_error_new(G_IO_ERROR,
-                                                                    G_IO_ERROR_INVAL,
-                                                                    tr("Invalid Operation").toUtf8().constData(),
-                                                                    nullptr)),
-                                true);
-        switch (response.value<ResponseType>()) {
+        FileOperationError except;
+        except.errorType = ET_GIO;
+        except.dlgType = ED_WARNING;
+        except.srcUri = nullptr;
+        except.destDirUri = nullptr;
+        except.title = tr("File delete");
+        except.errorCode = G_IO_ERROR_INVAL;
+        except.errorStr = tr("Invalid Operation");
+        Q_EMIT errored(except);
+        auto response = except.respCode;
+        switch (response) {
         case Retry:
             goto start;
         case Cancel:
@@ -830,10 +956,15 @@ start:
     //ensure again
     if (m_force_use_fallback) {
         moveForceUseFallback();
+        operationStartSnyc();
+        QProcess p;
+        p.start("sync");
+        p.waitForFinished(-1);
     }
     qDebug()<<"finished";
 end:
     Q_EMIT operationFinished();
+    //notifyFileWatcherOperationFinished();
 }
 
 void FileMoveOperation::cancel()
