@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -50,7 +50,7 @@ GlobalSettings::GlobalSettings(QObject *parent) : QObject(parent)
     m_cache.insert(SIDEBAR_BG_OPACITY, 50);
     if (QGSettings::isSchemaInstalled("org.ukui.style")) {
         m_gsettings = new QGSettings("org.ukui.style", QByteArray(), this);
-        connect(m_gsettings, &QGSettings::changed, this, [=](const QString &key){
+        connect(m_gsettings, &QGSettings::changed, this, [=](const QString &key) {
             if (key == "peonySideBarTransparency") {
                 m_cache.remove(SIDEBAR_BG_OPACITY);
                 m_cache.insert(SIDEBAR_BG_OPACITY, m_gsettings->get(key).toString());
@@ -60,6 +60,19 @@ GlobalSettings::GlobalSettings(QObject *parent) : QObject(parent)
         m_cache.remove(SIDEBAR_BG_OPACITY);
         m_cache.insert(SIDEBAR_BG_OPACITY, m_gsettings->get("peonySideBarTransparency").toString());
     }
+
+    if (m_cache.value(DEFAULT_WINDOW_SIZE).isNull()) {
+        setValue(DEFAULT_WINDOW_SIZE, QSize(850, 850*0.618));
+        setValue(DEFAULT_SIDEBAR_WIDTH, 180);
+    }
+
+    if (m_cache.value(DEFAULT_VIEW_ID).isNull()) {
+        setValue(DEFAULT_VIEW_ID, "Icon View");
+    }
+
+    if (m_cache.value(DEFAULT_VIEW_ZOOM_LEVEL).isNull()) {
+        setValue(DEFAULT_VIEW_ZOOM_LEVEL, 25);
+    }
 }
 
 GlobalSettings::~GlobalSettings()
@@ -80,7 +93,7 @@ bool GlobalSettings::isExist(const QString &key)
 void GlobalSettings::reset(const QString &key)
 {
     m_cache.remove(key);
-    QtConcurrent::run([=](){
+    QtConcurrent::run([=]() {
         if (m_mutex.tryLock(1000)) {
             m_settings->remove(key);
             m_settings->sync();
@@ -97,7 +110,7 @@ void GlobalSettings::resetAll()
     for (auto key : tmp) {
         Q_EMIT this->valueChanged(key);
     }
-    QtConcurrent::run([=](){
+    QtConcurrent::run([=]() {
         if (m_mutex.tryLock(1000)) {
             m_settings->clear();
             m_settings->sync();
@@ -109,7 +122,7 @@ void GlobalSettings::resetAll()
 void GlobalSettings::setValue(const QString &key, const QVariant &value)
 {
     m_cache.insert(key, value);
-    QtConcurrent::run([=](){
+    QtConcurrent::run([=]() {
         if (m_mutex.tryLock(1000)) {
             m_settings->setValue(key, value);
             m_settings->sync();
