@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -83,6 +83,19 @@ const QModelIndex FileItemProxyFilterSortModel::indexFromUri(const QString &uri)
 
 bool FileItemProxyFilterSortModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
 {
+    //comment these improve code to fix disorder issue
+    //and chinese first or folder first has not effect issue
+    //Fix me with better solutions
+//    if (left.data().isNull())
+//        return sortOrder() == Qt::AscendingOrder? false: true;
+
+//    if (right.data().isNull())
+//        return sortOrder() == Qt::AscendingOrder? true: false;
+
+//    if (left.column() != 0 || right.column() != 0) {
+//        return true;
+//    }
+
     //qDebug()<<left<<right;
     if (left.isValid() && right.isValid()) {
         FileItemModel *model = static_cast<FileItemModel*>(sourceModel());
@@ -110,18 +123,7 @@ default_sort:
             if (m_use_default_name_sort_order) {
                 QString leftDisplayName = leftItem->m_info->displayName();
                 QString rightDisplayName = rightItem->m_info->displayName();
-                bool leftStartWithChinese = startWithChinese(leftDisplayName);
-                bool rightStartWithChinese = startWithChinese(rightDisplayName);
-                //all start with Chinese, use the default compare directly
-                if (leftStartWithChinese && rightStartWithChinese)
-                    return comparer.compare(leftDisplayName, rightDisplayName) < 0;
-                //simplify the logic
-                if (leftStartWithChinese || rightStartWithChinese) {
-                    if (sortOrder() == Qt::AscendingOrder) {
-                        return leftStartWithChinese;
-                    }
-                    return rightStartWithChinese;
-                }
+                return comparer.compare(leftDisplayName, rightDisplayName) < 0;
             }
             return leftItem->m_info->displayName().toLower() < rightItem->m_info->displayName().toLower();
         }
@@ -163,12 +165,14 @@ bool FileItemProxyFilterSortModel::filterAcceptsRow(int sourceRow, const QModelI
         //regExp
 
         //check the file info filter conditions
-        //qDebug()<<"start filter conditions check";
-        if (m_show_file_type != ALL_FILE && ! checkFileTypeFilter(item->m_info->type()))
+        //qDebug()<<"start filter conditions check"<<item->m_info->displayName()<<item->m_info->type();
+        if (! checkFileTypeFilter(item->m_info->type()))
             return false;
-        if (m_show_modify_time != ALL_FILE && ! checkFileModifyTimeFilter(item->m_info->modifiedDate()))
+        if (! checkFileModifyTimeFilter(item->m_info->modifiedTime()))
             return false;
-        if (m_show_file_size != ALL_FILE && ! checkFileSizeFilter(item->m_info->size()))
+        if (! checkFileSizeFilter(item->m_info->size()))
+            return false;
+        if (! checkFileNameFilter(item->m_info->displayName()))
             return false;
 
         //check the file label filter conditions
@@ -184,28 +188,28 @@ bool FileItemProxyFilterSortModel::filterAcceptsRow(int sourceRow, const QModelI
 
             if (m_label_color != Qt::transparent)
             {
-                 auto colors = FileLabelModel::getGlobalModel()->getFileColors(uri);
-                 if (! colors.contains(m_label_color))
-                     return false;
+                auto colors = FileLabelModel::getGlobalModel()->getFileColors(uri);
+                if (! colors.contains(m_label_color))
+                    return false;
             }
         }
 
-        //check mutiple label filter conditions, file has any one of these label is accepted
+        //check multiple label filter conditions, file has any one of these label is accepted
         if(m_show_label_names.size() >0 || m_show_label_colors.size() >0)
         {
             bool bfind = false;
             QString uri = item->m_info->uri();
             if (m_show_label_names.size() >0 )
             {
-               auto names = FileLabelModel::getGlobalModel()->getFileLabels(uri);
-               for(auto temp : m_show_label_names)
-               {
-                   if(names.contains(temp))
-                   {
-                       bfind = true;
-                       break;
-                   }
-               }
+                auto names = FileLabelModel::getGlobalModel()->getFileLabels(uri);
+                for(auto temp : m_show_label_names)
+                {
+                    if(names.contains(temp))
+                    {
+                        bfind = true;
+                        break;
+                    }
+                }
             }
 
             if (! bfind && m_show_label_colors.size() >0)
@@ -234,7 +238,7 @@ bool FileItemProxyFilterSortModel::filterAcceptsRow(int sourceRow, const QModelI
             for(auto temp : names)
             {
                 if ((m_case_sensitive && temp.indexOf(m_blur_name) >= 0) ||
-                   (! m_case_sensitive && temp.toLower().indexOf(m_blur_name.toLower()) >= 0))
+                        (! m_case_sensitive && temp.toLower().indexOf(m_blur_name.toLower()) >= 0))
                 {
                     find = true;
                     break;
@@ -247,177 +251,206 @@ bool FileItemProxyFilterSortModel::filterAcceptsRow(int sourceRow, const QModelI
     return true;
 }
 
+bool FileItemProxyFilterSortModel::checkFileNameFilter(const QString &displayName) const
+{
+    if (m_file_name_list.size() == 0)
+        return true;
+
+    for(auto key:m_file_name_list)
+    {
+        if (displayName.contains(key))
+            return true;
+    }
+
+    return false;
+}
+
 bool FileItemProxyFilterSortModel::checkFileTypeFilter(QString type) const
 {
-    //qDebug()<<"m_show_file_type: "<<m_show_file_type<<" "<<item->info()->type();
-    switch (m_show_file_type)
+    //qDebug()<<"m_show_file_type: "<<m_show_file_type<<" "<<type;
+    //multiple condition, advance search and default search
+    if (m_show_file_type == ALL_FILE && m_file_type_list.count() == 0
+        || m_file_type_list.contains(ALL_FILE))
+        return true;
+
+    //support multiple file type choose
+    QList<int> totalTypeList;
+    if (m_file_type_list.count() > 0)
+        totalTypeList.append(m_file_type_list);
+    if (! totalTypeList.contains(m_show_file_type) && m_show_file_type != ALL_FILE)
+        totalTypeList.append(m_show_file_type);
+
+    for(int i=0; i<totalTypeList.count(); i++)
     {
+        auto cur = totalTypeList[i];
+        switch (cur)
+        {
         case FILE_FOLDER:
         {
-            if (type != Folder_Type)
-                return false;
+            if (type == Folder_Type)
+                return true;
             break;
         }
         case PICTURE:
         {
-            if (! type.contains(Image_Type))
-                return false;
+            if (type.contains(Image_Type))
+                return true;
             break;
         }
         case VIDEO:
         {
-            if (! type.contains(Video_Type))
-                return false;
+            if (type.contains(Video_Type))
+                return true;
             break;
         }
         case TXT_FILE:
         {
-            if (! type.contains(Text_Type))
-                return false;
+            if (type.contains(Text_Type))
+                return true;
+            break;
+        }
+        case WPS_FILE:
+        {
+            if (type.contains(Wps_Type))
+                return true;
             break;
         }
         case AUDIO:
         {
-            if (! type.contains(Audio_Type))
-                return false;
+            if (type.contains(Audio_Type))
+                return true;
             break;
         }
         case OTHERS:
         {
             //exclude classfied types, show the rest other types
-            if (type == Folder_Type || type.contains(Image_Type) || type.contains(Video_Type)
-                    || type.contains(Text_Type) || type.contains(Audio_Type))
-                return false;
+            if (type != Folder_Type && ! type.contains(Image_Type) && ! type.contains(Video_Type)
+                    && ! type.contains(Text_Type) && !type.contains(Wps_Type) && ! type.contains(Audio_Type))
+                return true;
             break;
         }
         default:
             break;
+        }
     }
-    return true;
+
+    return false;
 }
 
-bool FileItemProxyFilterSortModel::checkFileModifyTimeFilter(QString modifiedDate) const
+bool FileItemProxyFilterSortModel::checkFileModifyTimeFilter(quint64 modifiedTime) const
 {
     //qDebug()<<"checkFileModifyTimeFilter";
-    //invalide date
-    if (modifiedDate.size() < 8)
-        return false;
-    QDate date = QDate::currentDate();
+    //have no date filter
+    if (m_modify_time_list.count() == 0 || m_modify_time_list.contains(ALL_FILE))
+        return true;
+
+    auto time = QDateTime::currentMSecsSinceEpoch();
+    auto dateTime = QDateTime::fromMSecsSinceEpoch(time);
+    //qDebug() << "cur QDateTime:" <<dateTime.toString(Qt::SystemLocaleShortDate);
+    QDate date = dateTime.date();
     int year = date.year();
     int month = date.month();
     int day = date.day();
-    QString md_year, md_month, md_day;
-    auto parts = modifiedDate.split('/', QString::KeepEmptyParts);
-    md_year = parts[0];
-    md_month = parts[1];
-    md_day = parts[2].split(' ', QString::KeepEmptyParts)[0];
-    if (md_year.toInt() ==0 || md_month.toInt() ==0 || md_day.toInt() ==0)
-        qDebug()<<"date format error:" <<md_year<<" "<<md_month<<" "<<md_day;
+    QDate md_date =  QDateTime::fromMSecsSinceEpoch(modifiedTime * 1000).date();
+    //qDebug() << "modify time:" <<QDateTime::fromMSecsSinceEpoch(modifiedTime *1000).toString(Qt::SystemLocaleShortDate);
+    int md_year= md_date.year();
+    int md_month = md_date.month();
+    int md_day = md_date.day();
 
-    switch(m_show_modify_time)
+    for(int i=0; i<m_modify_time_list.size(); i++)
     {
+        auto cur = m_modify_time_list[i];
+        switch(cur)
+        {
         case TODAY:
         {
-            if (day != md_day.toInt())
-                return false;
+            if (year == md_year && month == md_month && day == md_day)
+                return true;
             break;
         }
         case THIS_WEEK:
         {
-            //find a future time, return false
-            if (year < md_year.toInt() || (year == md_year.toInt() && month < md_month.toInt())
-                    || (year == md_year.toInt() && month == md_month.toInt() && day < md_day.toInt()))
-            {
-                qDebug()<<"Modify time is a future time, please check your system time is correct!";
-                return false;
-            }
-            if (year - md_year.toInt() >1) //more than a year
-                return false;
-            if (year - md_year.toInt() ==1)
-            {
-                if (month+12-md_month.toInt() > 1) //more than a month
-                    return false;
-                if (month+12-md_month.toInt() == 1 && day+31-md_day.toInt() >= date.dayOfWeek())
-                    return false;
-            }
-            if (month - md_month.toInt() > 1) //more than a month
-                return false;
-            if (month - md_month.toInt() == 1)
-            {
-                QDate *modify_data = new QDate(md_year.toInt(), md_month.toInt(), md_day.toInt());
-                if (day + modify_data->daysInMonth() - md_day.toInt() >= date.dayOfWeek())
-                    return false;
-            }
-            if (day - md_day.toInt() >= date.dayOfWeek()) //same year,same month
-                return false;
-
+            QDate md_date(md_year, md_month, md_day);
+            int week, md_week,week_year, md_week_year;
+            week = date.weekNumber(&week_year);
+            md_week = md_date.weekNumber(&md_week_year);
+            if (week == md_week && week_year == md_week_year)
+                return true;
             break;
         }
         case THIS_MONTH:
         {
-            if (month != md_month.toInt())
-                return false;
+            if (year == md_year && month == md_month)
+                return true;
             break;
         }
         case THIS_YEAR:
         {
-            if (year != md_year.toInt())
-                return false;
+            if (year == md_year)
+                return true;
             break;
         }
         case YEAR_AGO:
         {
-            if(year == md_year.toInt())
-                return false;
+            if(year > md_year)
+                return true;
             break;
         }
         default:
             break;
+        }
     }
 
-    return true;
+    return false;
 }
 
 bool FileItemProxyFilterSortModel::checkFileSizeFilter(quint64 size) const
 {
     //qDebug()<<"checkFileSizeFilter: "<<m_show_file_size<<" "<<size;
-    switch (m_show_file_size)
+    if (m_file_size_list.count() == 0 || m_file_size_list.contains(ALL_FILE))
+        return true;
+
+    for(int i=0; i<m_file_size_list.count(); i++)
     {
+        auto cur = m_file_size_list[i];
+        switch (cur)
+        {
         case TINY: //[0-16K)
         {
-            if (size >= 16 * K_BASE)
-                return false;
+            if (size < 16 * K_BASE)
+                return true;
             break;
         }
         case SMALL:  //[16k-1M]
         {
-            if(size < 16 * K_BASE || size >K_BASE * K_BASE)
-                return false;
+            if(size >= 16 * K_BASE && size <=K_BASE * K_BASE)
+                return true;
             break;
         }
         case MEDIUM: //(1M-100M]
         {
-            if(size <= K_BASE * K_BASE || size >100 * K_BASE * K_BASE)
-                return false;
+            if(size > K_BASE * K_BASE && size <= 100 * K_BASE * K_BASE)
+                return true;
             break;
         }
         case BIG:  //(100M-1G]
         {
-            if(size <= 100 * K_BASE * K_BASE || size >K_BASE * K_BASE * K_BASE)
-                return false;
+            if(size > 100 * K_BASE * K_BASE && size <= K_BASE * K_BASE * K_BASE)
+                return true;
             break;
         }
         case LARGE: //>1G
         {
-            if (size <= K_BASE * K_BASE * K_BASE)
-                return false;
+            if (size > K_BASE * K_BASE * K_BASE)
+                return true;
             break;
         }
         default:
             break;
+        }
     }
 
-    return true;
+    return false;
 }
 
 
@@ -451,6 +484,69 @@ void FileItemProxyFilterSortModel::setFolderFirst(bool folderFirst)
     endResetModel();
 }
 
+void FileItemProxyFilterSortModel::addFileNameFilter(QString key, bool updateNow)
+{
+    m_file_name_list.append(key);
+    if (updateNow)
+        invalidateFilter();
+}
+
+void FileItemProxyFilterSortModel::addFilterCondition(int option, int classify, bool updateNow)
+{
+    switch (option) {
+    case 1:
+        if (! m_file_type_list.contains(classify))
+            m_file_type_list.append(classify);
+
+        break;
+    case 2:
+        if (! m_modify_time_list.contains(classify))
+            m_modify_time_list.append(classify);
+        break;
+    case 3:
+        if (! m_file_size_list.contains(classify))
+            m_file_size_list.append(classify);
+        break;
+    default:
+        break;
+    }
+
+    if (updateNow)
+        invalidateFilter();
+}
+
+void FileItemProxyFilterSortModel::removeFilterCondition(int option, int classify, bool updateNow)
+{
+    switch (option) {
+    case 1:
+        if (! m_file_type_list.contains(classify))
+            m_file_type_list.removeOne(classify);
+
+        break;
+    case 2:
+        if (! m_modify_time_list.contains(classify))
+            m_modify_time_list.removeOne(classify);
+        break;
+    case 3:
+        if (! m_file_size_list.contains(classify))
+            m_file_size_list.removeOne(classify);
+        break;
+    default:
+        break;
+    }
+
+    if (updateNow)
+        invalidateFilter();
+}
+
+void FileItemProxyFilterSortModel::clearConditions()
+{
+    m_file_name_list.clear();
+    m_file_type_list.clear();
+    m_file_size_list.clear();
+    m_modify_time_list.clear();
+}
+
 void FileItemProxyFilterSortModel::setFilterConditions(int fileType, int modifyTime, int fileSize)
 {
     m_show_file_type = fileType;
