@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,12 +26,14 @@
 #include "file-info-manager.h"
 #include "file-watcher.h"
 #include "file-utils.h"
+#include "file-operation-utils.h"
 
 #include "file-item-model.h"
 
 #include "thumbnail-manager.h"
 
 #include "gerror-wrapper.h"
+#include "bookmark-manager.h"
 
 #include <QDebug>
 #include <QStandardPaths>
@@ -48,6 +50,33 @@ FileItem::FileItem(std::shared_ptr<Peony::FileInfo> info, FileItem *parentItem,
     m_children = new QVector<FileItem*>();
 
     m_model = model;
+
+    m_backend_enumerator = new FileEnumerator(this);
+
+    m_thumbnail_watcher = std::make_shared<Peony::FileWatcher>("thumbnail://");
+    connect(m_thumbnail_watcher.get(), &FileWatcher::fileChanged, this, [=](const QString &uri){
+        auto index = m_model->indexFromUri(uri);
+        if (index.isValid()) {
+            auto item = m_model->itemFromIndex(index);
+            if (item) {
+                /*!
+                  \note
+                  fix the probabilistic jamming while thumbnailing with list view.
+
+                  we have to only trigger first column index dataChanged signal,
+                  otherwise there will be probility stucked whole program.
+
+                  i'm not sure if it is a bug of qtreeview.
+                  */
+
+                //m_model->dataChanged(item->firstColumnIndex(), item->lastColumnIndex());
+                m_model->dataChanged(item->firstColumnIndex(), item->firstColumnIndex());
+            }
+        }
+    });
+
+    // avoid call any method when model is deleted.
+    setParent(m_model);
 }
 
 FileItem::~FileItem()
@@ -114,7 +143,36 @@ void FileItem::findChildrenAsync()
     //NOTE: entry a new root might destroyed the current enumeration work.
     //the root item will be delete, so we should cancel the previous enumeration.
     enumerator->connect(this, &FileItem::cancelFindChildren, enumerator, &FileEnumerator::cancel);
-    enumerator->connect(enumerator, &FileEnumerator::prepared, this, [=](std::shared_ptr<GErrorWrapper> err){
+    enumerator->connect(enumerator, &FileEnumerator::prepared, this, [=](std::shared_ptr<GErrorWrapper> err, const QString &targetUri, bool critical) {
+        if (critical) {
+            QMessageBox::critical(nullptr, tr("Error"), err->message());
+            enumerator->cancel();
+            return;
+        }
+
+        if (!targetUri.isNull()) {
+            if (targetUri != this->uri()) {
+                this->m_info = FileInfo::fromUri(targetUri);
+
+                GFile *targetFile = g_file_new_for_uri(targetUri.toUtf8().constData());
+                QUrl targetUrl = targetUri;
+                auto path = g_file_get_path(targetFile);
+                if (path && !targetUrl.isLocalFile()) {
+                    QString localUri = QString("file://%1").arg(path);
+                    this->m_info = FileInfo::fromUri(localUri);
+                    enumerator->setEnumerateDirectory(localUri);
+                    g_free(path);
+                } else {
+                    enumerator->setEnumerateDirectory(targetFile);
+                }
+
+                g_object_unref(targetFile);
+            }
+
+            enumerator->enumerateAsync();
+            return;
+        }
+
         auto target = FileUtils::getTargetUri(m_info->uri());
         if (!target.isEmpty()) {
             enumerator->cancel();
@@ -124,10 +182,26 @@ void FileItem::findChildrenAsync()
         }
         if (err) {
             qDebug()<<err->message();
-            if (err.get()->code() == G_IO_ERROR_NOT_FOUND) {
+            if (err.get()->code() == G_IO_ERROR_NOT_FOUND || err.get()->code() == G_IO_ERROR_PERMISSION_DENIED) {
                 enumerator->cancel();
                 //enumerator->deleteLater();
                 m_model->setRootUri(FileUtils::getParentUri(this->uri()));
+                auto fileInfo = FileInfo::fromUri(this->uri(), false);
+                if (err.get()->code() == G_IO_ERROR_NOT_FOUND && fileInfo->isSymbolLink())
+                {
+                    auto result = QMessageBox::question(nullptr, tr("Open Link failed"),
+                                          tr("File not exist, do you want to delete the link file?"));
+                    if (result == QMessageBox::Yes) {
+                        qDebug() << "Delete unused symbollink.";
+                        QStringList selections;
+                        selections.push_back(this->uri());
+                        FileOperationUtils::trash(selections, true);
+                    }
+                }
+                return;
+            } else {
+                QMessageBox::critical(nullptr, tr("Error"), err->message());
+                enumerator->cancel();
                 return;
             }
         }
@@ -135,7 +209,7 @@ void FileItem::findChildrenAsync()
     });
 
     if (!m_model->isPositiveResponse()) {
-        enumerator->connect(enumerator, &Peony::FileEnumerator::enumerateFinished, this, [=](bool successed){
+        enumerator->connect(enumerator, &Peony::FileEnumerator::enumerateFinished, this, [=](bool successed) {
             if (successed) {
                 auto infos = enumerator->getChildren(true);
                 m_async_count = infos.count();
@@ -156,7 +230,7 @@ void FileItem::findChildrenAsync()
                         qDebug()<<shared_info->iconName()<<row;
                     });
                     */
-                    connect(job, &FileInfoJob::infoUpdated, this, [=](){
+                    connect(job, &FileInfoJob::queryAsyncFinished, this, [=]() {
                         //the query job is finished and will be deleted soon,
                         //whatever info was updated, we need decrease the async count.
                         m_async_count--;
@@ -165,10 +239,15 @@ void FileItem::findChildrenAsync()
                             Q_EMIT this->m_model->findChildrenFinished();
                             Q_EMIT m_model->updated();
                             for (auto info : infos) {
-                                ThumbnailManager::getInstance()->createThumbnail(info->uri(), m_watcher);
+                                ThumbnailManager::getInstance()->createThumbnail(info->uri(), m_thumbnail_watcher);
                             }
                         }
                     });
+
+                    connect(job, &FileInfoJob::infoUpdated, this, [=](){
+                        m_model->dataChanged(child->firstColumnIndex(), child->lastColumnIndex());
+                    });
+
                     job->queryAsync();
                 }
             } else {
@@ -179,59 +258,67 @@ void FileItem::findChildrenAsync()
             enumerator->cancel();
             delete enumerator;
 
-             m_watcher = std::make_shared<FileWatcher>(this->m_info->uri());
+            m_watcher = std::make_shared<FileWatcher>(this->m_info->uri());
             m_watcher->setMonitorChildrenChange(true);
-            connect(m_watcher.get(), &FileWatcher::fileCreated, this, [=](QString uri){
+            connect(m_watcher.get(), &FileWatcher::fileCreated, this, [=](QString uri) {
                 //add new item to m_children
                 //tell the model update
                 this->onChildAdded(uri);
                 Q_EMIT this->childAdded(uri);
             });
-            connect(m_watcher.get(), &FileWatcher::fileDeleted, this, [=](QString uri){
+            connect(m_watcher.get(), &FileWatcher::fileDeleted, this, [=](QString uri) {
+                //check bookmark and delete
+                auto info = FileInfo::fromUri(uri, false);
+                if (info->isDir())
+                {
+                    BookMarkManager::getInstance()->removeBookMark(uri);
+                }
                 //remove the crosponding child
                 //tell the model update
                 this->onChildRemoved(uri);
                 Q_EMIT this->childRemoved(uri);
             });
-            connect(m_watcher.get(), &FileWatcher::fileChanged, this, [=](const QString &uri){
+            connect(m_watcher.get(), &FileWatcher::fileChanged, this, [=](const QString &uri) {
                 auto index = m_model->indexFromUri(uri);
                 if (index.isValid()) {
                     auto infoJob = new FileInfoJob(FileInfo::fromUri(index.data(FileItemModel::UriRole).toString()));
                     infoJob->setAutoDelete();
-                    connect(infoJob, &FileInfoJob::queryAsyncFinished, this, [=](){
+                    connect(infoJob, &FileInfoJob::queryAsyncFinished, this, [=]() {
                         m_model->dataChanged(m_model->indexFromUri(uri), m_model->indexFromUri(uri));
                         auto info = FileInfo::fromUri(uri);
                         if (info->isDesktopFile()) {
-                            ThumbnailManager::getInstance()->updateDesktopFileThumbnail(info->uri(), m_watcher);
+                            ThumbnailManager::getInstance()->updateDesktopFileThumbnail(info->uri(), m_thumbnail_watcher);
                         }
                     });
                     infoJob->queryAsync();
                 }
             });
-            connect(m_watcher.get(), &FileWatcher::thumbnailUpdated, this, [=](const QString &uri){
+            connect(m_watcher.get(), &FileWatcher::thumbnailUpdated, this, [=](const QString &uri) {
                 m_model->dataChanged(m_model->indexFromUri(uri), m_model->indexFromUri(uri));
             });
-            connect(m_watcher.get(), &FileWatcher::directoryDeleted, this, [=](QString uri){
+            connect(m_watcher.get(), &FileWatcher::directoryDeleted, this, [=](QString uri) {
                 //clean all the children, if item index is root index, cd up.
                 //this might use FileItemModel::setRootItem()
                 Q_EMIT this->deleted(uri);
                 this->onDeleted(uri);
             });
 
-            connect(m_watcher.get(), &FileWatcher::locationChanged, this, [=](QString oldUri, QString newUri){
+            connect(m_watcher.get(), &FileWatcher::locationChanged, this, [=](QString oldUri, QString newUri) {
                 //this might use FileItemModel::setRootItem()
                 Q_EMIT this->renamed(oldUri, newUri);
                 this->onRenamed(oldUri, newUri);
             });
 
-            connect(m_watcher.get(), &FileWatcher::directoryUnmounted, this, [=](){
+            connect(m_watcher.get(), &FileWatcher::directoryUnmounted, this, [=]() {
                 m_model->setRootUri("computer:///");
             });
             //qDebug()<<"startMonitor";
+
+            connect(m_watcher.get(), &FileWatcher::requestUpdateDirectory, this, &FileItem::onUpdateDirectoryRequest);
             m_watcher->startMonitor();
         });
     } else {
-        enumerator->connect(enumerator, &Peony::FileEnumerator::childrenUpdated, this, [=](const QStringList &uris){
+        enumerator->connect(enumerator, &Peony::FileEnumerator::childrenUpdated, this, [=](const QStringList &uris) {
             if (uris.isEmpty()) {
                 Q_EMIT m_model->findChildrenFinished();
             }
@@ -250,16 +337,16 @@ void FileItem::findChildrenAsync()
                 m_model->endInsertRows();
                 auto infoJob = new FileInfoJob(info);
                 infoJob->setAutoDelete();
-                infoJob->connect(infoJob, &FileInfoJob::infoUpdated, this, [=](){
+                infoJob->connect(infoJob, &FileInfoJob::infoUpdated, this, [=]() {
                     Q_EMIT m_model->dataChanged(item->firstColumnIndex(), item->lastColumnIndex());
                     //Q_EMIT m_model->updated();
-                    ThumbnailManager::getInstance()->createThumbnail(info->uri(), m_watcher);
+                    ThumbnailManager::getInstance()->createThumbnail(info->uri(), m_thumbnail_watcher);
                 });
                 infoJob->queryAsync();
             }
         });
 
-        enumerator->connect(enumerator, &Peony::FileEnumerator::enumerateFinished, this, [=](){
+        enumerator->connect(enumerator, &Peony::FileEnumerator::enumerateFinished, this, [=]() {
             delete enumerator;
             if (!m_model||!m_children||!m_info)
                 return;
@@ -269,25 +356,31 @@ void FileItem::findChildrenAsync()
 
             m_watcher = std::make_shared<FileWatcher>(this->m_info->uri());
             m_watcher->setMonitorChildrenChange(true);
-            connect(m_watcher.get(), &FileWatcher::fileCreated, this, [=](QString uri){
+            connect(m_watcher.get(), &FileWatcher::fileCreated, this, [=](QString uri) {
                 //add new item to m_children
                 //tell the model update
                 this->onChildAdded(uri);
                 Q_EMIT this->childAdded(uri);
-                ThumbnailManager::getInstance()->createThumbnail(uri, m_watcher);
+                ThumbnailManager::getInstance()->createThumbnail(uri, m_thumbnail_watcher);
             });
-            connect(m_watcher.get(), &FileWatcher::fileDeleted, this, [=](QString uri){
+            connect(m_watcher.get(), &FileWatcher::fileDeleted, this, [=](QString uri) {
+                //check bookmark and delete
+                auto info = FileInfo::fromUri(uri, false);
+                if (info->isDir())
+                {
+                    BookMarkManager::getInstance()->removeBookMark(uri);
+                }
                 //remove the crosponding child
                 //tell the model update
                 this->onChildRemoved(uri);
                 Q_EMIT this->childRemoved(uri);
             });
-            connect(m_watcher.get(), &FileWatcher::fileChanged, this, [=](const QString &uri){
+            connect(m_watcher.get(), &FileWatcher::fileChanged, this, [=](const QString &uri) {
                 auto index = m_model->indexFromUri(uri);
                 if (index.isValid()) {
                     auto infoJob = new FileInfoJob(FileInfo::fromUri(index.data(FileItemModel::UriRole).toString()));
                     infoJob->setAutoDelete();
-                    connect(infoJob, &FileInfoJob::queryAsyncFinished, this, [=](){
+                    connect(infoJob, &FileInfoJob::queryAsyncFinished, this, [=]() {
                         m_model->dataChanged(m_model->indexFromUri(uri), m_model->indexFromUri(uri));
                         auto info = FileInfo::fromUri(uri);
                         if (info->isDesktopFile()) {
@@ -297,25 +390,26 @@ void FileItem::findChildrenAsync()
                     infoJob->queryAsync();
                 }
             });
-            connect(m_watcher.get(), &FileWatcher::thumbnailUpdated, this, [=](const QString &uri){
+            connect(m_watcher.get(), &FileWatcher::thumbnailUpdated, this, [=](const QString &uri) {
                 m_model->dataChanged(m_model->indexFromUri(uri), m_model->indexFromUri(uri));
             });
-            connect(m_watcher.get(), &FileWatcher::directoryDeleted, this, [=](QString uri){
+            connect(m_watcher.get(), &FileWatcher::directoryDeleted, this, [=](QString uri) {
                 //clean all the children, if item index is root index, cd up.
                 //this might use FileItemModel::setRootItem()
                 Q_EMIT this->deleted(uri);
                 this->onDeleted(uri);
             });
-            connect(m_watcher.get(), &FileWatcher::locationChanged, this, [=](QString oldUri, QString newUri){
+            connect(m_watcher.get(), &FileWatcher::locationChanged, this, [=](QString oldUri, QString newUri) {
                 //this might use FileItemModel::setRootItem()
                 Q_EMIT this->renamed(oldUri, newUri);
                 this->onRenamed(oldUri, newUri);
             });
 
-            connect(m_watcher.get(), &FileWatcher::directoryUnmounted, this, [=](){
+            connect(m_watcher.get(), &FileWatcher::directoryUnmounted, this, [=]() {
                 m_model->setRootUri("computer:///");
             });
             //qDebug()<<"startMonitor";
+            connect(m_watcher.get(), &FileWatcher::requestUpdateDirectory, this, &FileItem::onUpdateDirectoryRequest);
             m_watcher->startMonitor();
         });
     }
@@ -342,9 +436,10 @@ bool FileItem::hasChildren()
 FileItem *FileItem::getChildFromUri(QString uri)
 {
     for (auto item : *m_children) {
+        QUrl itemUrl = item->uri();
         QUrl url = uri;
         QString decodedUri = url.toDisplayString();
-        if (decodedUri == item->uri())
+        if (decodedUri == itemUrl.toDisplayString())
             return item;
     }
     return nullptr;
@@ -357,26 +452,29 @@ void FileItem::onChildAdded(const QString &uri)
     if (child) {
         qDebug()<<"has added";
         //child info maybe changed, so need sync update again
-        child->updateInfoSync();
-        m_model->updated();
+        child->updateInfoAsync();
+        //m_model->updated();
         return;
     }
     FileItem *newChild = new FileItem(FileInfo::fromUri(uri), this, m_model);
+    m_model->beginInsertRows(this->firstColumnIndex(), m_children->count(), m_children->count());
     m_children->append(newChild);
-    m_model->insertRow(m_children->count() - 1, this->firstColumnIndex());
+    m_model->endInsertRows();
     //use sync update here.
-    newChild->updateInfoSync();
-    m_model->updated();
+    newChild->updateInfoAsync();
+    //m_model->updated();
 }
 
 void FileItem::onChildRemoved(const QString &uri)
 {
     FileItem *child = getChildFromUri(uri);
     if (child) {
-        m_model->removeRow(m_children->indexOf(child), this->firstColumnIndex());
+        int index = m_children->indexOf(child);
+        m_model->beginRemoveRows(this->firstColumnIndex(), index, index);
         m_children->removeOne(child);
+        delete child;
+        m_model->endRemoveRows();
     }
-    delete child;
     m_model->updated();
 }
 
@@ -429,11 +527,52 @@ void FileItem::onRenamed(const QString &oldUri, const QString &newUri)
     }
 }
 
+void FileItem::onUpdateDirectoryRequest()
+{
+    auto enumerator = new FileEnumerator(this);
+    enumerator->setEnumerateDirectory(m_model->getRootUri());
+    connect(enumerator, &FileEnumerator::enumerateFinished, m_model, [=](){
+        if (m_model->getRootUri() != enumerator->getEnumerateUri())
+            return;
+
+        auto currentUris = enumerator->getChildrenUris();
+        QStringList rawUris;
+        QStringList removedUris;
+        QStringList addedUris;
+
+        for (auto child : *m_model->m_root_item->m_children) {
+            rawUris<<child->uri();
+            if (!currentUris.contains(child->uri())) {
+                removedUris<<child->uri();
+                m_model->m_root_item->onChildRemoved(child->uri());
+            }
+        }
+
+        for (auto uri : currentUris) {
+            if (!rawUris.contains(uri)) {
+                addedUris<<uri;
+                m_model->m_root_item->onChildAdded(uri);
+            }
+        }
+
+        for (auto uri : currentUris) {
+            if (!addedUris.contains(uri) && !removedUris.contains(uri)) {
+                //m_model->m_root_item->getChildFromUri(uri)->updateInfoAsync();
+            }
+        }
+
+        enumerator->deleteLater();
+    });
+
+    enumerator->enumerateAsync();
+}
+
 void FileItem::updateInfoSync()
 {
     FileInfoJob *job = new FileInfoJob(m_info);
     if (job->querySync()) {
         m_model->dataChanged(this->firstColumnIndex(), this->lastColumnIndex());
+        ThumbnailManager::getInstance()->createThumbnail(this->uri(), m_thumbnail_watcher);
     }
     job->deleteLater();
 }
@@ -442,8 +581,9 @@ void FileItem::updateInfoAsync()
 {
     FileInfoJob *job = new FileInfoJob(m_info);
     job->setAutoDelete();
-    job->connect(job, &FileInfoJob::infoUpdated, this, [=](){
+    job->connect(job, &FileInfoJob::infoUpdated, this, [=]() {
         m_model->dataChanged(this->firstColumnIndex(), this->lastColumnIndex());
+        ThumbnailManager::getInstance()->createThumbnail(this->uri(), m_thumbnail_watcher);
     });
     job->queryAsync();
 }
