@@ -1,7 +1,7 @@
 /*
  * Peony-Qt
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include <QPushButton>
 #include <QWidget>
 #include <QPainter>
+#include <QFileInfo>
 
 #include <QApplication>
 
@@ -79,21 +80,24 @@ void DesktopIconViewDelegate::paint(QPainter *painter, const QStyleOptionViewIte
     if (!view->indexWidget(index)) {
         //painter->setClipRect(opt.rect);
         painter->save();
+        painter->setRenderHint(QPainter::Antialiasing);
         if (opt.state.testFlag(QStyle::State_MouseOver) && !opt.state.testFlag(QStyle::State_Selected)) {
             QColor color = m_styled_button->palette().highlight().color();
             color.setAlpha(255*0.3);//half transparent
-            painter->fillRect(opt.rect, color);
+            //painter->fillRect(opt.rect, color);
             color.setAlpha(255*0.5);
             painter->setPen(color.darker(100));
-            painter->drawRect(opt.rect.adjusted(0, 0, -1, -1));
+            painter->setBrush(color);
+            painter->drawRoundedRect(opt.rect.adjusted(1, 1, -1, -1), 6, 6);
         }
         if (opt.state.testFlag(QStyle::State_Selected)) {
             QColor color = m_styled_button->palette().highlight().color();
             color.setAlpha(255*0.7);//half transparent
-            painter->fillRect(opt.rect, color);
+            //painter->fillRect(opt.rect, color);
             color.setAlpha(255*0.8);
             painter->setPen(color);
-            painter->drawRect(opt.rect.adjusted(0, 0, -1, -1));
+            painter->setBrush(color);
+            painter->drawRoundedRect(opt.rect.adjusted(1, 1, -1, -1), 6, 6);
         }
         painter->restore();
     }
@@ -252,11 +256,11 @@ QWidget *DesktopIconViewDelegate::createEditor(QWidget *parent, const QStyleOpti
     //NOTE: if we directly call this method, there will be
     //nothing happen. add a very short delay will ensure that
     //the edit be resized.
-    QTimer::singleShot(1, [=](){
+    QTimer::singleShot(1, [=]() {
         edit->minimalAdjust();
     });
 
-    connect(edit, &IconViewEditor::returnPressed, [=](){
+    connect(edit, &IconViewEditor::returnPressed, [=]() {
         this->setModelData(edit, nullptr, index);
         edit->deleteLater();
     });
@@ -275,7 +279,8 @@ void DesktopIconViewDelegate::setEditorData(QWidget *editor, const QModelIndex &
     cursor.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
     //qDebug()<<cursor.position();
     if (edit->toPlainText().contains(".") && !edit->toPlainText().startsWith(".")) {
-        cursor.movePosition(QTextCursor::WordLeft, QTextCursor::KeepAnchor, 2);
+        cursor.movePosition(QTextCursor::WordLeft, QTextCursor::KeepAnchor, 1);
+        cursor.movePosition(QTextCursor::Left, QTextCursor::KeepAnchor, 1);
         //qDebug()<<cursor.position();
     }
     //qDebug()<<cursor.anchor();
@@ -294,7 +299,7 @@ void DesktopIconViewDelegate::updateEditorGeometry(QWidget *editor, const QStyle
     //auto iconRect = opt.widget->style()->subElementRect(QStyle::SE_ItemViewItemDecoration, &opt, opt.widget);
     //auto y_delta = iconExpectedSize.height() - iconRect.height();
     //edit->move(opt.rect.x(), opt.rect.y() + y_delta + 10);
-    edit->move(opt.rect.x(), opt.rect.y() + iconExpectedSize.height() + 15);
+    edit->move(opt.rect.x(), opt.rect.y() + iconExpectedSize.height() + 5);
 
     edit->resize(edit->document()->size().width(), edit->document()->size().height() + 10);
 }
@@ -305,12 +310,19 @@ void DesktopIconViewDelegate::setModelData(QWidget *editor, QAbstractItemModel *
     if (!edit)
         return;
     auto newName = edit->toPlainText();
-    if (!newName.isNull()) {
-        if (newName != index.data(Qt::DisplayRole).toString()) {
-            auto fileOpMgr = FileOperationManager::getInstance();
-            auto renameOp = new FileRenameOperation(index.data(Qt::UserRole).toString(), newName);
-            fileOpMgr->startOperation(renameOp, true);
-        }
+    auto oldName = index.data(Qt::DisplayRole).toString();
+    QFileInfo info(index.data().toUrl().path());
+    auto suffix = "." + info.suffix();
+    if (newName.isNull())
+        return;
+    //process special name . or .. or only space
+    if (newName == "." || newName == ".." || newName.trimmed() == "")
+        newName = "";
+    if (newName.length() >0 && newName != oldName && newName != suffix) {
+        auto fileOpMgr = FileOperationManager::getInstance();
+        auto renameOp = new FileRenameOperation(index.data(Qt::UserRole).toString(), newName);
+        getView()->setRenaming(true);
+        fileOpMgr->startOperation(renameOp, true);
     }
 }
 
