@@ -67,6 +67,9 @@
 #include <QWheelEvent>
 #include <QApplication>
 
+#include <QStringList>
+#include <QMessageBox>
+
 #include <QDebug>
 
 using namespace Peony;
@@ -75,13 +78,16 @@ using namespace Peony;
 
 DesktopIconView::DesktopIconView(QWidget *parent) : QListView(parent)
 {
+    //m_refresh_timer.setInterval(500);
+    //m_refresh_timer.setSingleShot(true);
+
     installEventFilter(this);
 
     initShoutCut();
     //initMenu();
     initDoubleClick();
 
-    connect(qApp, &QApplication::paletteChanged, this, [=](){
+    connect(qApp, &QApplication::paletteChanged, this, [=]() {
         viewport()->update();
     });
 
@@ -150,64 +156,27 @@ DesktopIconView::DesktopIconView(QWidget *parent) : QListView(parent)
 
     //connect(m_model, &DesktopItemModel::dataChanged, this, &DesktopIconView::clearAllIndexWidgets);
 
-    connect(m_model, &DesktopItemModel::refreshed, this, [=](){
-        this->updateItemPosistions(nullptr);
-        this->m_is_refreshing = false;
+    connect(m_model, &DesktopItemModel::refreshed, this, [=]() {
+        this->setCursor(QCursor(Qt::ArrowCursor));
+        m_is_refreshing = false;
+
+        // check if there are items overlapped.
+        QTimer::singleShot(150, this, [=](){
+            if (isItemsOverlapped()) {
+                // refresh again?
+                //this->refresh();
+            }
+        });
+        return;
     });
 
     connect(m_model, &DesktopItemModel::requestClearIndexWidget, this, &DesktopIconView::clearAllIndexWidgets);
 
-    connect(m_model, &DesktopItemModel::requestLayoutNewItem, this, [=](const QString &uri){
-        auto index = m_proxy_model->mapFromSource(m_model->indexFromUri(uri));
-        //qDebug()<<"=====================layout new item"<<index.data();
-
-        //qDebug()<<"=====================find a new empty place put new item";
-        auto rect = QRect(QPoint(0, 0), gridSize());
-        rect.moveTo(this->contentsMargins().left(), this->contentsMargins().top());
-
-        auto grid = this->gridSize();
-        auto viewRect = this->rect();
-        auto next = rect;
-        while (true) {
-            if (this->indexAt(next.center()).data(Qt::UserRole).toString() == uri) {
-                this->setPositionForIndex(next.topLeft(), index);
-                this->saveItemPositionInfo(uri);
-                return;
-            }
-            if (!this->indexAt(next.center()).isValid()) {
-                //put it into empty
-                qDebug()<<"put"<<index.data()<<next.topLeft();
-                this->setPositionForIndex(next.topLeft(), index);
-                this->saveItemPositionInfo(uri);
-                break;
-            }
-            //aligin.
-            next = QListView::visualRect(this->indexAt(next.center()));
-            next.translate(0, grid.height());
-            if (next.bottom() > viewRect.bottom()) {
-                int top = next.y();
-                while (true) {
-                    if (top < gridSize().height()) {
-                        break;
-                    }
-                    top-=gridSize().height();
-                }
-                //put item to next column first column
-                next.moveTo(next.x() + grid.width(), top);
-            }
-        }
-    });
-
-    connect(m_model, &DesktopItemModel::requestUpdateItemPositions, this, &DesktopIconView::updateItemPosistions);
-
-    connect(m_model, &DesktopItemModel::fileCreated, this, [=](const QString &uri){
+    connect(m_model, &DesktopItemModel::fileCreated, this, [=](const QString &uri) {
         if (m_new_files_to_be_selected.isEmpty()) {
             m_new_files_to_be_selected<<uri;
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-            QTimer::singleShot(500, this, [=](){
-#else
-            QTimer::singleShot(500, [=](){
-#endif
+
+            QTimer::singleShot(500, this, [=]() {
                 if (this->state() & QAbstractItemView::EditingState)
                     return;
                 this->setSelections(m_new_files_to_be_selected);
@@ -218,9 +187,10 @@ DesktopIconView::DesktopIconView(QWidget *parent) : QListView(parent)
                 m_new_files_to_be_selected<<uri;
             }
         }
+        //refresh();
     });
 
-    connect(m_proxy_model, &QSortFilterProxyModel::layoutChanged, this, [=](){
+    connect(m_proxy_model, &QSortFilterProxyModel::layoutChanged, this, [=]() {
         //qDebug()<<"layout changed=========================\n\n\n\n\n";
         if (m_proxy_model->getSortType() == DesktopItemProxyModel::Other) {
             return;
@@ -229,24 +199,23 @@ DesktopIconView::DesktopIconView(QWidget *parent) : QListView(parent)
             return;
         }
         //qDebug()<<"save====================================";
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-        QTimer::singleShot(100, this, [=](){
-#else
-        QTimer::singleShot(100, [=](){
-#endif
-            this->saveAllItemPosistionInfos();
+
+        QTimer::singleShot(100, this, [=]() {
+            //this->saveAllItemPosistionInfos();
         });
     });
 
-    connect(this, &QListView::iconSizeChanged, this, [=](){
+    connect(this, &QListView::iconSizeChanged, this, [=]() {
         //qDebug()<<"save=============";
         this->setSortType(GlobalSettings::getInstance()->getValue(LAST_DESKTOP_SORT_ORDER).toInt());
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-        QTimer::singleShot(100, this, [=](){
-#else
-        QTimer::singleShot(100, [=](){
-#endif
+
+        QTimer::singleShot(100, this, [=]() {
             this->saveAllItemPosistionInfos();
+            for (int i = 0; i < m_proxy_model->rowCount(); i++) {
+                auto index = m_proxy_model->index(i, 0);
+                m_item_rect_hash.insert(index.data(Qt::UserRole).toString(), QListView::visualRect(index));
+                updateItemPosByUri(index.data(Qt::UserRole).toString(), QListView::visualRect(index).topLeft());
+            }
         });
     });
 
@@ -263,9 +232,10 @@ DesktopIconView::~DesktopIconView()
 
 bool DesktopIconView::eventFilter(QObject *obj, QEvent *e)
 {
+    //fixme:
     if (e->type() == QEvent::StyleChange) {
         if (m_model)
-            updateItemPosistions();
+            refresh();
     }
     return false;
 }
@@ -305,6 +275,7 @@ void DesktopIconView::initShoutCut()
     connect(trashAction, &QAction::triggered, [=]() {
         auto selectedUris = this->getSelections();
         if (!selectedUris.isEmpty()) {
+            clearAllIndexWidgets();
             auto op = new FileTrashOperation(selectedUris);
             FileOperationManager::getInstance()->startOperation(op, true);
         }
@@ -355,13 +326,14 @@ void DesktopIconView::initShoutCut()
     removeAction->setShortcut(QKeySequence(Qt::SHIFT + Qt::Key_Delete));
     connect(removeAction, &QAction::triggered, [=]() {
         qDebug() << "delete" << this->getSelections();
+        clearAllIndexWidgets();
         FileOperationUtils::executeRemoveActionWithDialog(this->getSelections());
     });
     addAction(removeAction);
 
     QAction *helpAction = new QAction(this);
     helpAction->setShortcut(Qt::Key_F1);
-    connect(helpAction, &QAction::triggered, this, [=](){
+    connect(helpAction, &QAction::triggered, this, [=]() {
         PeonyDesktopApplication::showGuide();
     });
     addAction(helpAction);
@@ -369,7 +341,7 @@ void DesktopIconView::initShoutCut()
     auto propertiesWindowAction = new QAction(this);
     propertiesWindowAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::ALT + Qt::Key_Return)
                                          <<QKeySequence(Qt::ALT + Qt::Key_Enter));
-    connect(propertiesWindowAction, &QAction::triggered, this, [=](){
+    connect(propertiesWindowAction, &QAction::triggered, this, [=]() {
         if (this->getSelections().count() > 0)
         {
             PropertiesWindow *w = new PropertiesWindow(this->getSelections());
@@ -380,36 +352,65 @@ void DesktopIconView::initShoutCut()
 
     auto newFolderAction = new QAction(this);
     newFolderAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_N));
-    connect(newFolderAction, &QAction::triggered, this, [=](){
+    connect(newFolderAction, &QAction::triggered, this, [=]() {
         CreateTemplateOperation op(this->getDirectoryUri(), CreateTemplateOperation::EmptyFolder, tr("New Folder"));
         op.run();
         auto targetUri = op.target();
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-            QTimer::singleShot(500, this, [=](){
-#else
-            QTimer::singleShot(500, [=](){
-#endif
+
+        QTimer::singleShot(500, this, [=]() {
             this->scrollToSelection(targetUri);
         });
     });
     addAction(newFolderAction);
 
+    QAction *refreshWinAction = new QAction(this);
+    refreshWinAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
+    connect(refreshWinAction, &QAction::triggered, [=]() {
+        this->refresh();
+    });
+    addAction(refreshWinAction);
+
+    QAction *reverseSelectAction = new QAction(this);
+    reverseSelectAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_L));
+    connect(reverseSelectAction, &QAction::triggered, [=]() {
+        this->invertSelections();
+    });
+    addAction(reverseSelectAction);
+
+    QAction *normalIconAction = new QAction(this);
+    normalIconAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_0));
+    connect(normalIconAction, &QAction::triggered, [=]() {
+        this->setDefaultZoomLevel(DesktopIconView::Normal);
+    });
+    addAction(normalIconAction);
+
     auto refreshAction = new QAction(this);
     refreshAction->setShortcut(Qt::Key_F5);
-    connect(refreshAction, &QAction::triggered, this, [=](){
+    connect(refreshAction, &QAction::triggered, this, [=]() {
         this->refresh();
     });
     addAction(refreshAction);
 
     QAction *editAction = new QAction(this);
     editAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::ALT + Qt::Key_E)<<Qt::Key_F2);
-    connect(editAction, &QAction::triggered, this, [=](){
+    connect(editAction, &QAction::triggered, this, [=]() {
         auto selections = this->getSelections();
         if (selections.count() == 1) {
             this->editUri(selections.first());
         }
     });
     addAction(editAction);
+
+    auto settings = GlobalSettings::getInstance();
+    m_show_hidden = settings->isExist("show-hidden")? settings->getValue("show-hidden").toBool(): false;
+    //show hidden action
+    QAction *showHiddenAction = new QAction(this);
+    showHiddenAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
+    addAction(showHiddenAction);
+    connect(showHiddenAction, &QAction::triggered, this, [=]() {
+        //qDebug() << "show hidden";
+        this->setShowHidden();
+    });
 }
 
 void DesktopIconView::initMenu()
@@ -465,32 +466,53 @@ void DesktopIconView::initMenu()
     }, Qt::UniqueConnection);
 }
 
+void DesktopIconView::setShowHidden()
+{
+    m_show_hidden = ! m_show_hidden;
+    qDebug() << "DesktopIconView::setShowHidden:" <<m_show_hidden;
+    m_proxy_model->setShowHidden(m_show_hidden);
+}
+
+void DesktopIconView::openFileByUri(QString uri)
+{
+    auto info = FileInfo::fromUri(uri, false);
+    auto job = new FileInfoJob(info);
+    job->setAutoDelete();
+    job->connect(job, &FileInfoJob::queryAsyncFinished, [=]() {
+        if ((info->isDir() || info->isVolume() || info->isVirtual())) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+            QProcess p;
+            QUrl url = uri;
+            p.setProgram("peony");
+            p.setArguments(QStringList() << url.toEncoded() <<"%U&");
+            p.startDetached();
+#else
+            QProcess p;
+            QString strq;
+            for (int i = 0;i < uri.length();++i) {
+                if(uri[i] == ' '){
+                    strq += "%20";
+                }else{
+                    strq += uri[i];
+                }
+            }
+
+            p.startDetached("peony", QStringList()<<strq<<"%U&");
+#endif
+        } else {
+            FileLaunchManager::openAsync(uri, false, false);
+        }
+        this->clearSelection();
+    });
+    job->queryAsync();
+}
+
 void DesktopIconView::initDoubleClick()
 {
     connect(this, &QListView::doubleClicked, this, [=](const QModelIndex &index) {
         qDebug() << "double click" << index.data(FileItemModel::UriRole);
         auto uri = index.data(FileItemModel::UriRole).toString();
-        auto info = FileInfo::fromUri(uri, false);
-        auto job = new FileInfoJob(info);
-        job->setAutoDelete();
-        job->connect(job, &FileInfoJob::queryAsyncFinished, [=]() {
-            if (info->isDir() || info->isVolume() || info->isVirtual()) {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
-                QProcess p;
-                QUrl url = uri;
-                p.setProgram("peony");
-                p.setArguments(QStringList() << url.toEncoded());
-                p.startDetached();
-#else
-                QProcess p;
-                p.startDetached("peony", QStringList()<<uri);
-#endif
-            } else {
-                FileLaunchManager::openAsync(uri);
-            }
-            this->clearSelection();
-        });
-        job->queryAsync();
+        openFileByUri(uri);
     }, Qt::UniqueConnection);
 }
 
@@ -515,20 +537,15 @@ void DesktopIconView::saveAllItemPosistionInfos()
 
 void DesktopIconView::saveItemPositionInfo(const QString &uri)
 {
-    auto index = m_proxy_model->mapFromSource(m_model->indexFromUri(uri));
-    auto indexRect = QListView::visualRect(index);
-    QStringList topLeft;
-    topLeft<<QString::number(indexRect.top());
-    topLeft<<QString::number(indexRect.left());
-    auto metaInfo = FileMetaInfo::fromUri(index.data(Qt::UserRole).toString());
-    if (metaInfo) {
-        //qDebug()<<"save"<<uri<<topLeft;
-        metaInfo->setMetaInfoStringList(ITEM_POS_ATTRIBUTE, topLeft);
-    }
+    return;
 }
 
 void DesktopIconView::resetAllItemPositionInfos()
 {
+    if (!m_proxy_model)
+        return;
+
+    m_item_rect_hash.clear();
     for (int i = 0; i < m_proxy_model->rowCount(); i++) {
         auto index = m_proxy_model->index(i, 0);
         auto indexRect = QListView::visualRect(index);
@@ -546,55 +563,84 @@ void DesktopIconView::resetAllItemPositionInfos()
 
 void DesktopIconView::resetItemPosistionInfo(const QString &uri)
 {
-    auto metaInfo = FileMetaInfo::fromUri(uri);
-    if (metaInfo)
-        metaInfo->removeMetaInfo(ITEM_POS_ATTRIBUTE);
+    return;
 }
 
 void DesktopIconView::updateItemPosistions(const QString &uri)
 {
-    if (uri.isNull()) {
-        for (int i = 0; i < m_proxy_model->rowCount(); i++) {
-            auto index = m_proxy_model->index(i, 0);
-            auto metaInfo = FileMetaInfo::fromUri(index.data(Qt::UserRole).toString());
-            if (metaInfo) {
-                updateItemPosistions(index.data(Qt::UserRole).toString());
-            }
-        }
-        return;
-    }
+    return;
+}
 
-    auto index = m_proxy_model->mapFromSource(m_model->indexFromUri(uri));
-    //qDebug()<<"update"<<uri<<index.data();
+QPoint DesktopIconView::getFileMetaInfoPos(const QString &uri)
+{
+    auto value = m_item_rect_hash.value(uri);
+    if (!value.isEmpty())
+        return value.topLeft();
 
-    if (!index.isValid()) {
-        //qDebug()<<"err: index invalid";
-        return;
-    }
     auto metaInfo = FileMetaInfo::fromUri(uri);
-    if (!metaInfo) {
-        //qDebug()<<"err: no meta data";
-        return;
-    }
-
-    auto list = metaInfo->getMetaInfoStringList(ITEM_POS_ATTRIBUTE);
-    if (!list.isEmpty()) {
-        if (list.count() == 2) {
-            int top = list.first().toInt();
-            int left = list.at(1).toInt();
-            if (top >= 0 && left >= 0) {
-//                auto rect = visualRect(index);
-//                auto grid = gridSize();
-//                if (abs(rect.top() - top) < grid.width() && abs(rect.left() - left))
-//                    return;
-                QPoint p(left, top);
-                //qDebug()<<"set"<<index.data()<<p;
-                setPositionForIndex(QPoint(left, top), index);
-            } else {
-                saveItemPositionInfo(uri);
+    if (metaInfo) {
+        auto list = metaInfo->getMetaInfoStringList(ITEM_POS_ATTRIBUTE);
+        if (!list.isEmpty()) {
+            if (list.count() == 2) {
+                int top = list.first().toInt();
+                int left = list.at(1).toInt();
+                if (top > 0 && left >= 0) {
+                    QPoint p(left, top);
+                    return p;
+                }
             }
         }
     }
+    return QPoint(-1, -1);
+}
+
+void DesktopIconView::setFileMetaInfoPos(const QString &uri, const QPoint &pos)
+{
+    auto srcIndex = m_model->indexFromUri("computer:///");
+    auto index = m_proxy_model->mapFromSource(srcIndex);
+    m_item_rect_hash.remove(uri);
+    m_item_rect_hash.insert(uri, QRect(pos, QListView::visualRect(index).size()));
+
+    auto metaInfo = FileMetaInfo::fromUri(uri);
+    if (metaInfo) {
+        QStringList topLeft;
+        topLeft<<QString::number(pos.y());
+        topLeft<<QString::number(pos.x());
+        metaInfo->setMetaInfoStringList(ITEM_POS_ATTRIBUTE, topLeft);
+    }
+}
+
+QHash<QString, QRect> DesktopIconView::getCurrentItemRects()
+{
+    return m_item_rect_hash;
+}
+
+void DesktopIconView::removeItemRect(const QString &uri)
+{
+    m_item_rect_hash.remove(uri);
+}
+
+void DesktopIconView::updateItemPosByUri(const QString &uri, const QPoint &pos)
+{
+    auto srcIndex = m_model->indexFromUri(uri);
+    auto index = m_proxy_model->mapFromSource(srcIndex);
+    if (index.isValid()) {
+        setPositionForIndex(pos, index);
+        m_item_rect_hash.remove(uri);
+        m_item_rect_hash.insert(uri, QRect(pos, QListView::visualRect(index).size()));
+    }
+}
+
+void DesktopIconView::ensureItemPosByUri(const QString &uri)
+{
+    auto srcIndex = m_model->indexFromUri(uri);
+    auto index = m_proxy_model->mapFromSource(srcIndex);
+    auto rect = QListView::visualRect(index);
+    if (index.isValid()) {
+        m_item_rect_hash.remove(uri);
+        m_item_rect_hash.insert(uri, rect);
+        setFileMetaInfoPos(uri, rect.topLeft());
+    }
 }
 
 const QStringList DesktopIconView::getSelections()
@@ -604,12 +650,18 @@ const QStringList DesktopIconView::getSelections()
     for (auto index : indexes) {
         uris<<index.data(Qt::UserRole).toString();
     }
+    uris.removeDuplicates();
     return uris;
 }
 
 const QStringList DesktopIconView::getAllFileUris()
 {
-
+    QStringList uris;
+    for (int i = 0; i < m_proxy_model->rowCount(); i++) {
+        auto index = m_proxy_model->index(i, 0);
+        uris<<index.data(Qt::UserRole).toString();
+    }
+    return uris;
 }
 
 void DesktopIconView::setSelections(const QStringList &uris)
@@ -625,7 +677,11 @@ void DesktopIconView::setSelections(const QStringList &uris)
 
 void DesktopIconView::invertSelections()
 {
-
+    QItemSelectionModel *selectionModel = this->selectionModel();
+    const QItemSelection currentSelection = selectionModel->selection();
+    this->selectAll();
+    selectionModel->select(currentSelection, QItemSelectionModel::Deselect);
+    clearAllIndexWidgets();
 }
 
 void DesktopIconView::scrollToSelection(const QString &uri)
@@ -640,10 +696,17 @@ int DesktopIconView::getSortType()
 
 void DesktopIconView::setSortType(int sortType)
 {
+    m_item_rect_hash.clear();
     //resetAllItemPositionInfos();
     m_proxy_model->setSortType(sortType);
     m_proxy_model->sort(1);
     m_proxy_model->sort(0, m_proxy_model->sortOrder());
+    saveAllItemPosistionInfos();
+    for (int i = 0; i < m_proxy_model->rowCount(); i++) {
+        auto index = m_proxy_model->index(i, 0);
+        m_item_rect_hash.insert(index.data(Qt::UserRole).toString(), QListView::visualRect(index));
+        updateItemPosByUri(index.data(Qt::UserRole).toString(), QListView::visualRect(index).topLeft());
+    }
 }
 
 int DesktopIconView::getSortOrder()
@@ -659,12 +722,9 @@ void DesktopIconView::setSortOrder(int sortOrder)
 void DesktopIconView::editUri(const QString &uri)
 {
     clearAllIndexWidgets();
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-    QTimer::singleShot(100, this, [=](){
-#else
-    QTimer::singleShot(100, [=](){
-#endif
-        edit(m_proxy_model->mapFromSource(m_model->indexFromUri(uri)));
+    auto origin = FileUtils::getOriginalUri(uri);
+    QTimer::singleShot(100, this, [=]() {
+        edit(m_proxy_model->mapFromSource(m_model->indexFromUri(origin)));
     });
 }
 
@@ -687,12 +747,11 @@ void DesktopIconView::wheelEvent(QWheelEvent *e)
 {
     if (QApplication::keyboardModifiers() == Qt::ControlModifier)
     {
-       if (e->delta() > 0) {
-           zoomIn();
-       } else {
-           zoomOut();
-       }
-       resetAllItemPositionInfos();
+        if (e->delta() > 0) {
+            zoomIn();
+        } else {
+            zoomOut();
+        }
     }
 }
 
@@ -712,6 +771,12 @@ void DesktopIconView::keyPressEvent(QKeyEvent *e)
                 selectionModel()->select(upIndex, QItemSelectionModel::SelectCurrent);
                 auto delegate = qobject_cast<DesktopIconViewDelegate *>(itemDelegate());
                 setIndexWidget(upIndex, new DesktopIndexWidget(delegate, viewOptions(), upIndex, this));
+
+                for (auto uri : getAllFileUris()) {
+                    auto pos = getFileMetaInfoPos(uri);
+                    if (pos.x() >= 0)
+                        updateItemPosByUri(uri, pos);
+                }
             }
         }
         return;
@@ -729,6 +794,12 @@ void DesktopIconView::keyPressEvent(QKeyEvent *e)
                 selectionModel()->select(downIndex, QItemSelectionModel::SelectCurrent);
                 auto delegate = qobject_cast<DesktopIconViewDelegate *>(itemDelegate());
                 setIndexWidget(downIndex, new DesktopIndexWidget(delegate, viewOptions(), downIndex, this));
+
+                for (auto uri : getAllFileUris()) {
+                    auto pos = getFileMetaInfoPos(uri);
+                    if (pos.x() >= 0)
+                        updateItemPosByUri(uri, pos);
+                }
             }
         }
         return;
@@ -746,6 +817,12 @@ void DesktopIconView::keyPressEvent(QKeyEvent *e)
                 selectionModel()->select(leftIndex, QItemSelectionModel::SelectCurrent);
                 auto delegate = qobject_cast<DesktopIconViewDelegate *>(itemDelegate());
                 setIndexWidget(leftIndex, new DesktopIndexWidget(delegate, viewOptions(), leftIndex, this));
+
+                for (auto uri : getAllFileUris()) {
+                    auto pos = getFileMetaInfoPos(uri);
+                    if (pos.x() >= 0)
+                        updateItemPosByUri(uri, pos);
+                }
             }
         }
         return;
@@ -763,6 +840,12 @@ void DesktopIconView::keyPressEvent(QKeyEvent *e)
                 selectionModel()->select(rightIndex, QItemSelectionModel::SelectCurrent);
                 auto delegate = qobject_cast<DesktopIconViewDelegate *>(itemDelegate());
                 setIndexWidget(rightIndex, new DesktopIndexWidget(delegate, viewOptions(), rightIndex, this));
+
+                for (auto uri : getAllFileUris()) {
+                    auto pos = getFileMetaInfoPos(uri);
+                    if (pos.x() >= 0)
+                        updateItemPosByUri(uri, pos);
+                }
             }
         }
         return;
@@ -771,6 +854,16 @@ void DesktopIconView::keyPressEvent(QKeyEvent *e)
     case Qt::Key_Control:
         m_ctrl_or_shift_pressed = true;
         break;
+    case Qt::Key_Enter:
+    case Qt::Key_Return:
+    {
+        auto selections = this->getSelections();
+        for (auto uri : selections)
+        {
+           openFileByUri(uri);
+        }
+    }
+        break;
     default:
         return QListView::keyPressEvent(e);
     }
@@ -782,12 +875,66 @@ void DesktopIconView::keyReleaseEvent(QKeyEvent *e)
     m_ctrl_or_shift_pressed = false;
 }
 
+void DesktopIconView::focusOutEvent(QFocusEvent *e)
+{
+    QListView::focusOutEvent(e);
+    m_ctrl_or_shift_pressed = false;
+}
+
 void DesktopIconView::resizeEvent(QResizeEvent *e)
 {
     QListView::resizeEvent(e);
     refresh();
 }
 
+void DesktopIconView::rowsInserted(const QModelIndex &parent, int start, int end)
+{
+    QListView::rowsInserted(parent, start, end);
+    for (auto uri : getAllFileUris()) {
+        auto pos = getFileMetaInfoPos(uri);
+        if (pos.x() >= 0)
+            updateItemPosByUri(uri, pos);
+    }
+}
+
+void DesktopIconView::rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
+{
+    QListView::rowsAboutToBeRemoved(parent, start, end);
+    QTimer::singleShot(1, this, [=](){
+        for (auto uri : getAllFileUris()) {
+            auto pos = getFileMetaInfoPos(uri);
+            if (pos.x() >= 0)
+                updateItemPosByUri(uri, pos);
+        }
+    });
+}
+
+bool DesktopIconView::isItemsOverlapped()
+{
+    QList<QRect> itemRects;
+    if (model()) {
+        for (int i = 0; i < model()->rowCount(); i++) {
+            auto index = model()->index(i, 0);
+            auto rect = QListView::visualRect(index);
+            if (itemRects.contains(rect))
+                return true;
+            itemRects<<QListView::visualRect(index);
+        }
+    }
+
+    return false;
+}
+
+bool DesktopIconView::isRenaming()
+{
+    return m_is_renaming;
+}
+
+void DesktopIconView::setRenaming(bool renaming)
+{
+    m_is_renaming = renaming;
+}
+
 void DesktopIconView::zoomOut()
 {
     clearAllIndexWidgets();
@@ -802,7 +949,7 @@ void DesktopIconView::zoomOut()
         setDefaultZoomLevel(Small);
         break;
     default:
-        setDefaultZoomLevel(zoomLevel());
+        //setDefaultZoomLevel(zoomLevel());
         break;
     }
 }
@@ -821,7 +968,7 @@ void DesktopIconView::zoomIn()
         setDefaultZoomLevel(Huge);
         break;
     default:
-        setDefaultZoomLevel(zoomLevel());
+        //setDefaultZoomLevel(zoomLevel());
         break;
     }
 }
@@ -863,6 +1010,8 @@ void DesktopIconView::setDefaultZoomLevel(ZoomLevel level)
     } else {
 
     }
+
+    resetAllItemPositionInfos();
 }
 
 DesktopIconView::ZoomLevel DesktopIconView::zoomLevel() const
@@ -914,6 +1063,12 @@ void DesktopIconView::mousePressEvent(QMouseEvent *e)
             if (!indexWidget(m_last_index)) {
                 setIndexWidget(m_last_index,
                                new DesktopIndexWidget(qobject_cast<DesktopIconViewDelegate *>(itemDelegate()), viewOptions(), m_last_index));
+
+                for (auto uri : getAllFileUris()) {
+                    auto pos = getFileMetaInfoPos(uri);
+                    if (pos.x() >= 0)
+                        updateItemPosByUri(uri, pos);
+                }
             }
         }
     }
@@ -940,16 +1095,32 @@ void DesktopIconView::mouseDoubleClickEvent(QMouseEvent *event)
 void DesktopIconView::dragEnterEvent(QDragEnterEvent *e)
 {
     m_real_do_edit = false;
-    //qDebug()<<"drag enter event";
+    if (e->keyboardModifiers() && Qt::ControlModifier)
+        m_ctrl_key_pressed = true;
+    else
+        m_ctrl_key_pressed = false;
+
+    auto action = m_ctrl_key_pressed ? Qt::CopyAction : Qt::MoveAction;
+    qDebug()<<"drag enter event" <<action;
     if (e->mimeData()->hasUrls()) {
-        e->setDropAction(Qt::MoveAction);
+        e->setDropAction(action);
         e->acceptProposedAction();
     }
+
+    if (e->source() == this) {
+        m_drag_indexes = selectedIndexes();
+    }
 }
 
 void DesktopIconView::dragMoveEvent(QDragMoveEvent *e)
 {
     m_real_do_edit = false;
+    if (e->keyboardModifiers() && Qt::ControlModifier)
+        m_ctrl_key_pressed = true;
+    else
+        m_ctrl_key_pressed = false;
+
+    auto action = m_ctrl_key_pressed ? Qt::CopyAction : Qt::MoveAction;
     auto index = indexAt(e->pos());
     if (index.isValid() && index != m_last_index) {
         QHoverEvent he(QHoverEvent::HoverMove, e->posF(), e->posF());
@@ -960,12 +1131,12 @@ void DesktopIconView::dragMoveEvent(QDragMoveEvent *e)
     }
     if (e->isAccepted())
         return;
-    //qDebug()<<"drag move event";
+    qDebug()<<"drag move event" <<action;
     if (this == e->source()) {
         e->accept();
         return QListView::dragMoveEvent(e);
     }
-    e->setDropAction(Qt::CopyAction);
+    e->setDropAction(action);
     e->accept();
 }
 
@@ -982,16 +1153,146 @@ void DesktopIconView::dropEvent(QDropEvent *e)
       is incorrect.
       */
     m_edit_trigger_timer.stop();
-    if (this == e->source()) {
+    if (e->keyboardModifiers() && Qt::ControlModifier)
+        m_ctrl_key_pressed = true;
+    else
+        m_ctrl_key_pressed = false;
 
+    auto action = m_ctrl_key_pressed ? Qt::CopyAction : Qt::MoveAction;
+    qDebug() << "DesktopIconView dropEvent" <<action;
+    if (this == e->source() && !m_ctrl_key_pressed)
+    {
         auto index = indexAt(e->pos());
+        qDebug() <<"DesktopIconView index:" <<index <<index.isValid();
+        bool bmoved = false;
         if (index.isValid()) {
             auto info = FileInfo::fromUri(index.data(Qt::UserRole).toString());
             if (!info->isDir())
                 return;
+            bmoved = true;
         }
 
-        QListView::dropEvent(e);
+        if (bmoved)
+        {
+            //move file to desktop folder
+            qDebug() << "DesktopIconView move file to folder";
+            m_model->dropMimeData(e->mimeData(), action, -1, -1, this->indexAt(e->pos()));
+        }
+        else
+            QListView::dropEvent(e);
+
+        QHash<QModelIndex, QRect> currentIndexesRects;
+        for (int i = 0; i < m_proxy_model->rowCount(); i++) {
+            auto tmp = m_proxy_model->index(i, 0);
+            currentIndexesRects.insert(tmp, QListView::visualRect(tmp));
+        }
+
+        //fixme: handle overlapping.
+        if (!m_drag_indexes.isEmpty()) {
+            QModelIndexList overlappedIndexes;
+            QModelIndexList unoverlappedIndexes = m_drag_indexes;
+            for (auto value : currentIndexesRects.values()) {
+                auto keys = currentIndexesRects.keys(value);
+                if (keys.count() > 1) {
+                    for (auto key : keys) {
+                        if (m_drag_indexes.contains(key) && !overlappedIndexes.contains(key)) {
+                            overlappedIndexes<<key;
+                            unoverlappedIndexes.removeOne(key);
+                        }
+                    }
+                }
+            }
+
+            for (auto index : unoverlappedIndexes) {
+                // save pos
+                QTimer::singleShot(1, this, [=](){
+                    setFileMetaInfoPos(index.data(Qt::UserRole).toString(), QListView::visualRect(index).topLeft());
+                });
+            }
+
+            auto grid = this->gridSize();
+            auto viewRect = this->rect();
+
+            QRegion notEmptyRegion;
+            for (auto rect : currentIndexesRects) {
+                notEmptyRegion += rect;
+            }
+
+            for (auto dragedIndex : overlappedIndexes) {
+                auto indexRect = QListView::visualRect(dragedIndex);
+                if (notEmptyRegion.contains(indexRect.center())) {
+                    // move index to closest empty grid.
+                    auto next = indexRect;
+                    bool isEmptyPos = false;
+                    while (!isEmptyPos) {
+                        next.translate(0, grid.height());
+                        if (next.bottom() > viewRect.bottom()) {
+                            int top = next.y();
+                            while (true) {
+                                if (top < gridSize().height()) {
+                                    break;
+                                }
+                                top-=gridSize().height();
+                            }
+                            //put item to next column first column
+                            next.moveTo(next.x() + grid.width(), top);
+                        }
+                        if (notEmptyRegion.contains(next.center()))
+                            continue;
+
+                        isEmptyPos = true;
+                        setPositionForIndex(next.topLeft(), dragedIndex);
+                        setFileMetaInfoPos(dragedIndex.data(Qt::UserRole).toString(), next.topLeft());
+                        notEmptyRegion += next;
+                    }
+                }
+            }
+
+            // check if there is any item out of view
+            for (auto index : m_drag_indexes) {
+                auto indexRect = QListView::visualRect(index);
+                if (this->viewport()->rect().contains(indexRect.center())) {
+                    continue;
+                }
+
+                // try relocating invisible item
+
+                QRect next;
+                for (auto existedRect : notEmptyRegion.rects()) {
+                    if (this->viewport()->rect().contains(existedRect)) {
+                        next = existedRect;
+                        break;
+                    }
+                }
+                while (next.translated(-grid.width(), 0).x() > 0) {
+                    next.translate(-grid.width(), 0);
+                }
+                while (next.translated(0, -grid.height()).top() > 0) {
+                    next.translate(0, -grid.height());
+                }
+
+                while (notEmptyRegion.contains(next.center())) {
+                    next.translate(0, grid.height());
+                    if (next.bottom() > viewRect.bottom()) {
+                        int top = next.y();
+                        while (true) {
+                            if (top < gridSize().height()) {
+                                break;
+                            }
+                            top-=gridSize().height();
+                        }
+                        //put item to next column first column
+                        next.moveTo(next.x() + grid.width(), top);
+                    }
+                }
+
+                setPositionForIndex(next.topLeft(), index);
+                setFileMetaInfoPos(index.data(Qt::UserRole).toString(), next.topLeft());
+                notEmptyRegion += next;
+            }
+        }
+
+        m_drag_indexes.clear();
 
         auto urls = e->mimeData()->urls();
         for (auto url : urls) {
@@ -1001,30 +1302,31 @@ void DesktopIconView::dropEvent(QDropEvent *e)
         }
         return;
     }
-    m_model->dropMimeData(e->mimeData(), Qt::MoveAction, -1, -1, this->indexAt(e->pos()));
+    m_model->dropMimeData(e->mimeData(), action, -1, -1, this->indexAt(e->pos()));
     //FIXME: save item position
 }
 
 const QFont DesktopIconView::getViewItemFont(QStyleOptionViewItem *item)
 {
-    auto font = item->font;
-    if (font.pixelSize() <= 0) {
-        font = QApplication::font();
-    }
-    switch (zoomLevel()) {
-    case DesktopIconView::Small:
-        font.setPixelSize(int(font.pixelSize() * 0.8));
-        break;
-    case DesktopIconView::Large:
-        font.setPixelSize(int(font.pixelSize() * 1.2));
-        break;
-    case DesktopIconView::Huge:
-        font.setPixelSize(int(font.pixelSize() * 1.4));
-        break;
-    default:
-        break;
-    }
-    return font;
+    return item->font;
+//    auto font = item->font;
+//    if (font.pixelSize() <= 0) {
+//        font = QApplication::font();
+//    }
+//    switch (zoomLevel()) {
+//    case DesktopIconView::Small:
+//        font.setPixelSize(int(font.pixelSize() * 0.8));
+//        break;
+//    case DesktopIconView::Large:
+//        font.setPixelSize(int(font.pixelSize() * 1.2));
+//        break;
+//    case DesktopIconView::Huge:
+//        font.setPixelSize(int(font.pixelSize() * 1.4));
+//        break;
+//    default:
+//        break;
+//    }
+//    return font;
 }
 
 void DesktopIconView::clearAllIndexWidgets()
@@ -1043,6 +1345,10 @@ void DesktopIconView::clearAllIndexWidgets()
 
 void DesktopIconView::refresh()
 {
+    this->setCursor(QCursor(Qt::WaitCursor));
+//    if (m_refresh_timer.isActive())
+//        return;
+
     if (!m_model)
         return;
 
@@ -1050,6 +1356,8 @@ void DesktopIconView::refresh()
         return;
     m_is_refreshing = true;
     m_model->refresh();
+
+    //m_refresh_timer.start();
 }
 
 QRect DesktopIconView::visualRect(const QModelIndex &index) const
