@@ -1,7 +1,7 @@
 /*
  * Peony-Qt
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "desktop-menu.h"
 
 #include "directory-view-plugin-iface.h"
-
+#include "file-info-job.h"
 #include "file-info.h"
 #include "file-utils.h"
 #include "file-launch-action.h"
@@ -46,12 +46,15 @@
 
 #include <QUrl>
 #include <QDir>
-#include <QFileIconProvider>
 
 using namespace Peony;
 
 DesktopMenu::DesktopMenu(DirectoryViewIface *view, QWidget *parent) : QMenu(parent)
 {
+    setProperty("useIconHighlightEffect", true);
+    setProperty("iconHighlightEffectMode", 1);
+    setProperty("fillIconSymbolicColor", true);
+
     m_view = view;
     m_directory = view->getDirectoryUri();
     m_selections = view->getSelections();
@@ -86,7 +89,7 @@ void DesktopMenu::fillActions()
         addSeparator();
 
     //add propertries actions
-    //if this is a backgroud menu, then it should not add propertries actions
+    //if this is a background menu, then it should not add propertries actions
     bool isBackgroundMenu = m_selections.isEmpty();
     if(!isBackgroundMenu) {
         auto propertiesAction = constructFilePropertiesActions();
@@ -100,18 +103,31 @@ const QList<QAction *> DesktopMenu::constructOpenOpActions()
     bool isBackgroundMenu = m_selections.isEmpty();
     if (isBackgroundMenu) {
         l<<addAction(QIcon::fromTheme("window-new-symbolic"), tr("&Open in new Window"));
-        connect(l.last(), &QAction::triggered, [=](){
+        connect(l.last(), &QAction::triggered, [=]() {
             this->openWindow(m_directory);
         });
+
+        l<<addAction(tr("Select &All"));
+        connect(l.last(), &QAction::triggered, [=]() {
+            //qDebug() << "select all";
+            m_view->invertSelections();
+        });
     } else {
         if (m_selections.count() == 1) {
             auto info = FileInfo::fromUri(m_selections.first());
             auto displayName = info->displayName();
             if (displayName.isEmpty())
                 displayName = FileUtils::getFileDisplayName(info->uri());
+            //when name is too long, show elideText
+            //qDebug() << "charWidth:" <<charWidth;
+            if (displayName.length() > ELIDE_TEXT_LENGTH)
+            {
+                int  charWidth = fontMetrics().averageCharWidth();
+                displayName = fontMetrics().elidedText(displayName, Qt::ElideRight, ELIDE_TEXT_LENGTH * charWidth);
+            }
             if (info->isDir()) {
                 l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open \"%1\"").arg(displayName));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     this->openWindow(m_selections);
                 });
 
@@ -128,14 +144,14 @@ const QList<QAction *> DesktopMenu::constructOpenOpActions()
                     openWithMenu->addAction(static_cast<QAction*>(action));
                 }
                 openWithMenu->addSeparator();
-                openWithMenu->addAction(tr("&More applications..."), [=](){
+                openWithMenu->addAction(tr("&More applications..."), [=]() {
                     FileLauchDialog d(m_selections.first());
                     d.exec();
                 });
                 openWithAction->setMenu(openWithMenu);
             } else if (!info->isVolume()) {
                 l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open \"%1\"").arg(displayName));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     auto uri = m_selections.first();
                     FileLaunchManager::openAsync(uri);
                 });
@@ -153,21 +169,21 @@ const QList<QAction *> DesktopMenu::constructOpenOpActions()
                     openWithMenu->addAction(static_cast<QAction*>(action));
                 }
                 openWithMenu->addSeparator();
-                openWithMenu->addAction(tr("&More applications..."), [=](){
+                openWithMenu->addAction(tr("&More applications..."), [=]() {
                     FileLauchDialog d(m_selections.first());
                     d.exec();
                 });
                 openWithAction->setMenu(openWithMenu);
             } else {
                 l<<addAction(tr("&Open"));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     auto uri = m_selections.first();
                     //FIXME:
                 });
             }
         } else {
             l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open %1 selected files").arg(m_selections.count()));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 qDebug()<<"triggered";
                 QStringList dirs;
                 QStringList files;
@@ -188,6 +204,12 @@ const QList<QAction *> DesktopMenu::constructOpenOpActions()
                 }
             });
         }
+
+        l<<addAction(tr("Reverse Select"));
+        connect(l.last(), &QAction::triggered, [=]() {
+            //qDebug() << "Reverse select";
+            m_view->invertSelections();
+        });
     }
 
     return l;
@@ -197,7 +219,7 @@ const QList<QAction *> DesktopMenu::constructCreateTemplateActions()
 {
     QList<QAction *> l;
     if (m_selections.isEmpty()) {
-        auto createAction = new QAction(tr("&New..."), this);
+        auto createAction = new QAction(tr("New..."), this);
         l<<createAction;
         QMenu *subMenu = new QMenu(this);
         createAction->setMenu(subMenu);
@@ -207,25 +229,56 @@ const QList<QAction *> DesktopMenu::constructCreateTemplateActions()
         QDir templateDir(g_get_user_special_dir(G_USER_DIRECTORY_TEMPLATES));
         auto templates = templateDir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot);
         if (!templates.isEmpty()) {
-            QFileIconProvider p;
             for (auto t : templates) {
-                QFileInfo info(t);
-                QAction *action = new QAction(p.icon(info), info.baseName(), this);
-                connect(action, &QAction::triggered, [=](){
+                QFileInfo qinfo(templateDir, t);
+                GFile *gtk_file = g_file_new_for_path(qinfo.filePath().toUtf8().data());
+                char *uri_str = g_file_get_uri(gtk_file);
+                std::shared_ptr<FileInfo> info = FileInfo::fromUri(uri_str);
+
+                QString mimeType = info->mimeType();
+                if (mimeType.isEmpty()) {
+                    FileInfoJob job(info);
+                    job.querySync();
+                    mimeType = info->mimeType();
+                }
+
+                QIcon tmpIcon;
+                GList *app_infos = g_app_info_get_recommended_for_type(mimeType.toUtf8().constData());
+                GList *l = app_infos;
+                QList<FileLaunchAction *> actions;
+                bool isOnlyUnref = false;
+                while (l) {
+                    auto app_info = static_cast<GAppInfo*>(l->data);
+                    if (!isOnlyUnref) {
+                        GThemedIcon *icon = G_THEMED_ICON(g_app_info_get_icon(app_info));
+                        const char * const * icon_names = g_themed_icon_get_names(icon);
+                        if (icon_names)
+                            tmpIcon = QIcon::fromTheme(*icon_names);
+                        if(!tmpIcon.isNull())
+                            isOnlyUnref = true;
+                    }
+                    g_object_unref(app_infos);
+                    l = l->next;
+                }
+
+                QAction *action = new QAction(tmpIcon, qinfo.baseName(), this);
+                connect(action, &QAction::triggered, [=]() {
                     CreateTemplateOperation op(m_directory, CreateTemplateOperation::Template, t);
                     op.run();
                     auto target = op.target();
                     m_uris_to_edit<<target;
                 });
                 subMenu->addAction(action);
+                g_free(uri_str);
+                g_object_unref(gtk_file);
             }
             subMenu->addSeparator();
         }
 
-         QList<QAction *> actions;
+        QList<QAction *> actions;
         auto createEmptyFileAction = new QAction(QIcon::fromTheme("document-new-symbolic"), tr("Empty &File"), this);
         actions<<createEmptyFileAction;
-        connect(actions.last(), &QAction::triggered, [=](){
+        connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory);
             CreateTemplateOperation op(m_directory);
             op.run();
@@ -235,7 +288,7 @@ const QList<QAction *> DesktopMenu::constructCreateTemplateActions()
         });
         auto createFolderActions = new QAction(QIcon::fromTheme("folder-new-symbolic"), tr("&Folder"), this);
         actions<<createFolderActions;
-        connect(actions.last(), &QAction::triggered, [=](){
+        connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory, nullptr, CreateTemplateOperation::EmptyFolder);
             CreateTemplateOperation op(m_directory, CreateTemplateOperation::EmptyFolder, tr("New Folder"));
             op.run();
@@ -260,16 +313,16 @@ const QList<QAction *> DesktopMenu::constructViewOpActions()
         auto desktopView = dynamic_cast<DesktopIconView*>(m_view);
         auto zoomLevel = desktopView->zoomLevel();
 
-        auto smallAction = viewTypeSubMenu->addAction(tr("&Small"), [=](){
+        auto smallAction = viewTypeSubMenu->addAction(tr("&Small"), [=]() {
             desktopView->setDefaultZoomLevel(DesktopIconView::Small);
         });
-        auto normalAction = viewTypeSubMenu->addAction(tr("&Normal"), [=](){
+        auto normalAction = viewTypeSubMenu->addAction(tr("&Normal"), [=]() {
             desktopView->setDefaultZoomLevel(DesktopIconView::Normal);
         });
-        auto largeAction = viewTypeSubMenu->addAction(tr("&Large"), [=](){
+        auto largeAction = viewTypeSubMenu->addAction(tr("&Large"), [=]() {
             desktopView->setDefaultZoomLevel(DesktopIconView::Large);
         });
-        auto hugeAction = viewTypeSubMenu->addAction(tr("&Huge"), [=](){
+        auto hugeAction = viewTypeSubMenu->addAction(tr("&Huge"), [=]() {
             desktopView->setDefaultZoomLevel(DesktopIconView::Huge);
         });
 
@@ -306,14 +359,19 @@ const QList<QAction *> DesktopMenu::constructViewOpActions()
         tmp<<sortTypeMenu->addAction(tr("File Type"));
         tmp<<sortTypeMenu->addAction(tr("File Size"));
         tmp<<sortTypeMenu->addAction(tr("Modified Date"));
-//        int sortType = m_view->getSortType();
-//        if (sortType >= 0) {
-//            tmp.at(sortType)->setCheckable(true);
-//            tmp.at(sortType)->setChecked(true);
-//        }
+
+        int sortType = m_view->getSortType();
+        qDebug() << "sortType:" <<sortType <<tmp.count();
+        if (sortType < 0 || sortType >= tmp.count()) {
+            sortType = GlobalSettings::getInstance()->getValue(LAST_DESKTOP_SORT_ORDER).toInt();
+        }
+        tmp.at(sortType)->setCheckable(true);
+        tmp.at(sortType)->setChecked(true);
+
 
         for (int i = 0; i < tmp.count(); i++) {
-            connect(tmp.at(i), &QAction::triggered, [=](){
+            connect(tmp.at(i), &QAction::triggered, [=]() {
+                qDebug() << "setSortType in menu:" <<i;
                 m_view->setSortType(i);
                 GlobalSettings::getInstance()->setValue(LAST_DESKTOP_SORT_ORDER, i);
             });
@@ -329,9 +387,9 @@ const QList<QAction *> DesktopMenu::constructViewOpActions()
         tmp.clear();
         tmp<<sortOrderMenu->addAction(tr("Ascending Order"));
         tmp<<sortOrderMenu->addAction(tr("Descending Order"));
-//        int sortOrder = m_view->getSortOrder();
-//        tmp.at(sortOrder)->setCheckable(true);
-//        tmp.at(sortOrder)->setChecked(true);
+        //        int sortOrder = m_view->getSortOrder();
+        //        tmp.at(sortOrder)->setCheckable(true);
+        //        tmp.at(sortOrder)->setChecked(true);
 
         for (int i = 0; i < tmp.count(); i++) {
             connect(tmp.at(i), &QAction::triggered, [=](){
@@ -367,14 +425,14 @@ const QList<QAction *> DesktopMenu::constructFileOpActions()
             e.setEnumerateDirectory("trash:///");
             e.enumerateSync();
             auto trashChildren = e.getChildrenUris();
-            l<<addAction(QIcon::fromTheme("view-refresh-symbolic"), tr("&Restore all"), [=](){
+            l<<addAction(QIcon::fromTheme("view-refresh-symbolic"), tr("&Restore all"), [=]() {
                 FileOperationUtils::restore(trashChildren);
             });
             l.last()->setEnabled(!trashChildren.isEmpty());
-            l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("&Clean the trash"), [=](){
+            l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("&Clean the trash"), [=]() {
                 auto result = QMessageBox::question(nullptr, tr("Delete Permanently"), tr("Are you sure that you want to delete these files? "
-                                                                                          "Once you start a deletion, the files deleting will never be "
-                                                                                          "restored again."));
+                                                    "Once you start a deletion, the files deleting will never be "
+                                                    "restored again."));
                 if (result == QMessageBox::Yes) {
                     FileEnumerator e;
                     FileOperationUtils::remove(trashChildren);
@@ -385,24 +443,29 @@ const QList<QAction *> DesktopMenu::constructFileOpActions()
 
         } else if (! m_selections.contains(homeUri)) {
             l<<addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("&Copy"));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 ClipboardUtils::setClipboardFiles(m_selections, false);
             });
-            l<<addAction(QIcon::fromTheme("edit-cut-symbolic"), tr("Cu&t"));
-            connect(l.last(), &QAction::triggered, [=](){
+            l<<addAction(QIcon::fromTheme("edit-cut-symbolic"), tr("Cut"));
+            connect(l.last(), &QAction::triggered, [=]() {
                 ClipboardUtils::setClipboardFiles(m_selections, true);
             });
 
             if (!m_selections.contains("trash:///")) {
-                l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("&Delete"));
-                connect(l.last(), &QAction::triggered, [=](){
+                l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("&Delete to trash"));
+                connect(l.last(), &QAction::triggered, [=]() {
                     FileOperationUtils::trash(m_selections, true);
                 });
+                //add delete forever option
+                l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
+                connect(l.last(), &QAction::triggered, [=]() {
+                    FileOperationUtils::executeRemoveActionWithDialog(m_selections);
+                });
             }
 
             if (m_selections.count() == 1) {
-                l<<addAction(QIcon::fromTheme("document-edit-symbolic"), tr("&Rename"));
-                connect(l.last(), &QAction::triggered, [=](){
+                l<<addAction(QIcon::fromTheme("document-edit-symbolic"), tr("Rename"));
+                connect(l.last(), &QAction::triggered, [=]() {
                     m_view->editUri(m_selections.first());
                 });
             }
@@ -411,11 +474,11 @@ const QList<QAction *> DesktopMenu::constructFileOpActions()
         auto pasteAction = addAction(QIcon::fromTheme("edit-paste-symbolic"), tr("&Paste"));
         l<<pasteAction;
         pasteAction->setEnabled(ClipboardUtils::isClipboardHasFiles());
-        connect(l.last(), &QAction::triggered, [=](){
+        connect(l.last(), &QAction::triggered, [=]() {
             ClipboardUtils::pasteClipboardFiles(m_directory);
         });
         l<<addAction(QIcon::fromTheme("view-refresh-symbolic"), tr("&Refresh"));
-        connect(l.last(), &QAction::triggered, [=](){
+        connect(l.last(), &QAction::triggered, [=]() {
             auto desktopView = dynamic_cast<DesktopIconView*>(m_view);
             desktopView->refresh();
         });
@@ -428,8 +491,8 @@ const QList<QAction *> DesktopMenu::constructFilePropertiesActions()
 {
     QList<QAction *> l;
 
-    l<<addAction(QIcon::fromTheme("preview-file"), tr("&Properties"));
-    connect(l.last(), &QAction::triggered, [=](){
+    l<<addAction(QIcon::fromTheme("preview-file"), tr("Properties"));
+    connect(l.last(), &QAction::triggered, [=]() {
         //FIXME:
         if (m_selections.isEmpty()) {
             this->showProperties(m_directory);
@@ -546,6 +609,10 @@ void DesktopMenu::gotoAboutComputer()
     p.setProgram("ukui-control-center");
     //-a para to show about computer infos
     p.setArguments(QStringList()<<"-a");
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
     p.startDetached();
+#else
+    p.startDetached("ukui-control-center", QStringList()<<"-a");
+#endif
     p.waitForFinished(-1);
 }
