@@ -1,7 +1,7 @@
 /*
  * Peony-Qt
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -69,11 +69,19 @@
 // backup settings
 #include <QSettings>
 
+#include <QX11Info>
+
+#include <X11/Xlib.h>
+#include <QX11Info>
+#include <X11/Xatom.h>
+#include <X11/Xproto.h>
+
 #include <QDebug>
 
 #define BACKGROUND_SETTINGS "org.mate.background"
 #define PICTRUE "picture-filename"
 #define FALLBACK_COLOR "primary-color"
+#define FONT_SETTINGS "org.ukui.style"
 
 using namespace Peony;
 
@@ -81,15 +89,16 @@ DesktopWindow::DesktopWindow(QScreen *screen, bool is_primary, QWidget *parent)
     : QMainWindow(parent) {
     initGSettings();
 
+    setWindowTitle(tr("Desktop"));
     m_opacity = new QVariantAnimation(this);
     m_opacity->setDuration(1000);
     m_opacity->setStartValue(double(0));
     m_opacity->setEndValue(double(1));
-    connect(m_opacity, &QVariantAnimation::valueChanged, this, [=](){
+    connect(m_opacity, &QVariantAnimation::valueChanged, this, [=]() {
         this->update();
     });
 
-    connect(m_opacity, &QVariantAnimation::finished, this, [=](){
+    connect(m_opacity, &QVariantAnimation::finished, this, [=]() {
         m_bg_back_pixmap = m_bg_font_pixmap;
         m_bg_back_cache_pixmap = m_bg_font_cache_pixmap;
         m_last_pure_color = m_color_to_be_set;
@@ -105,10 +114,22 @@ DesktopWindow::DesktopWindow(QScreen *screen, bool is_primary, QWidget *parent)
 
     qDebug() << "DesktopWindow is_primary:" << is_primary << screen->objectName()
              << screen->name();
-    setWindowFlags(Qt::FramelessWindowHint);
+    auto flags = windowFlags() &~Qt::WindowMinMaxButtonsHint;
+    setWindowFlags(flags |Qt::FramelessWindowHint);
     setAttribute(Qt::WA_X11NetWmWindowTypeDesktop);
     setAttribute(Qt::WA_TranslucentBackground);
 
+    //fix qt5.6 setAttribute as desktop has no effect issue
+#if (QT_VERSION < QT_VERSION_CHECK(5, 7, 0))
+    if (QX11Info::isPlatformX11()) {
+        Atom m_WindowType = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", true);
+        Atom m_DesktopType = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE_DESKTOP", true);
+        XDeleteProperty(QX11Info::display(), winId(), m_WindowType);
+        XChangeProperty(QX11Info::display(), winId(), m_WindowType,
+                        XA_ATOM, 32, 1, (unsigned char *)&m_DesktopType, 1);
+    }
+#endif
+
     setGeometry(screen->geometry());
 
     setContextMenuPolicy(Qt::CustomContextMenu);
@@ -173,6 +194,7 @@ DesktopWindow::DesktopWindow(QScreen *screen, bool is_primary, QWidget *parent)
 DesktopWindow::~DesktopWindow() {}
 
 void DesktopWindow::initGSettings() {
+    qDebug() <<"DesktopWindow initGSettings";
     if (!QGSettings::isSchemaInstalled(BACKGROUND_SETTINGS)) {
         m_backup_setttings = new QSettings("org.ukui", "peony-qt-desktop", this);
         if (m_backup_setttings->value("color").isNull()) {
@@ -182,34 +204,68 @@ void DesktopWindow::initGSettings() {
         return;
     }
 
+    //font monitor
+    QGSettings *fontSetting = new QGSettings(FONT_SETTINGS, QByteArray(), this);
+    connect(fontSetting, &QGSettings::changed, this, [=](const QString &key){
+        qDebug() << "fontSetting changed:" << key;
+        if (key == "systemFont" || key == "systemFontSize")
+        {
+            QFont font = this->font();
+            for(auto widget : qApp->allWidgets())
+                widget->setFont(font);
+        }
+    });
+
     m_bg_settings = new QGSettings(BACKGROUND_SETTINGS, QByteArray(), this);
 
     connect(m_bg_settings, &QGSettings::changed, this, [=](const QString &key) {
         qDebug() << "bg settings changed:" << key;
         if (key == "pictureFilename") {
             auto bg_path = m_bg_settings->get("pictureFilename").toString();
-            if (!QFile::exists(bg_path)) {
+            //control center set color bg
+            if (bg_path == "")
+            {
+                qDebug() << "bg_path == """;
                 // use pure color;
                 auto colorString = m_bg_settings->get("primary-color").toString();
                 auto color = QColor(colorString);
-                qDebug() << colorString;
+                qDebug() <<"pure color bg"<< colorString;
                 this->setBg(color);
-            } else {
+                m_current_bg_path = "";
+                return;
+            }
+
+            //can not find bg file, usually the file is moved, use default bg
+            if (!QFile::exists(bg_path)) {
+                QString path = "/usr/share/backgrounds/default.jpg";
+#if (QT_VERSION < QT_VERSION_CHECK(5, 7, 0))
+                path = "/usr/share/backgrounds/kylin/kylin-background.png";
+#endif
+                bool success = m_bg_settings->trySet("pictureFilename", path);
+                if (success)
+                {
+                    m_current_bg_path = "file://" + path;
+                }
+                else
+                {
+                    qDebug() << "use default bg picture fail, reset";
+                    m_bg_settings->reset("pictureFilename");
+                }
+            }
+            else {
+                qDebug() << "bg_path:" <<bg_path << m_current_bg_path;
                 if (m_current_bg_path == bg_path)
                     return;
+                qDebug() << "set a new bg picture:" <<bg_path;
                 this->setBg(bg_path);
             }
         }
-        if (key == "primaryColor") {
-            auto bg_path = m_bg_settings->get("pictureFilename").toString();
-            if (!bg_path.startsWith("/")) {
-                auto colorString = m_bg_settings->get("primary-color").toString();
-                auto color = QColor(colorString);
-                qDebug() << colorString;
-                this->setBg(color);
-            } else {
-                // do nothing
-            }
+        if (key == "primaryColor")
+        {
+            auto colorString = m_bg_settings->get("primary-color").toString();
+            auto color = QColor(colorString);
+            qDebug() <<"set color bg"<< colorString;
+            this->setBg(color);
         }
     });
 }
@@ -220,7 +276,11 @@ void DesktopWindow::gotoSetBackground()
     p.setProgram("ukui-control-center");
     //old version use -a, new version use -b as para
     p.setArguments(QStringList()<<"-b");
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
     p.startDetached();
+#else
+    p.startDetached("ukui-control-center", QStringList()<<"-b");
+#endif
     p.waitForFinished(-1);
 }
 
@@ -279,7 +339,7 @@ const QColor DesktopWindow::getCurrentColor()
 }
 
 void DesktopWindow::setBg(const QString &path) {
-    qDebug() << path;
+    qDebug() << "DesktopWindow::setBg:"<<path;
     if (path.isNull()) {
         setBg(getCurrentColor());
         return;
@@ -374,8 +434,19 @@ void DesktopWindow::scaleBg(const QRect &geometry) {
      * but screen will be black a while.
      * this is not user's expected.
      */
-    setWindowFlag(Qt::FramelessWindowHint, false);
-    setWindowFlag(Qt::FramelessWindowHint);
+    //setWindowFlag(Qt::FramelessWindowHint, false);
+    auto flags = windowFlags() &~Qt::WindowMinMaxButtonsHint;
+    setWindowFlags(flags |Qt::FramelessWindowHint);
+
+    //fix qt5.6 setAttribute as desktop has no effect issue
+#if QT_VERSION_CHECK(5, 6, 0)
+    Atom m_WindowType = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", true);
+    Atom m_DesktopType = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE_DESKTOP", true);
+    XDeleteProperty(QX11Info::display(), winId(), m_WindowType);
+    XChangeProperty(QX11Info::display(), winId(), m_WindowType,
+                    XA_ATOM, 32, 1, (unsigned char *)&m_DesktopType, 1);
+#endif
+
     show();
 
     m_bg_back_cache_pixmap = m_bg_back_pixmap.scaled(geometry.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
@@ -476,7 +547,7 @@ void DesktopWindow::initShortcut() {
     auto propertiesWindowAction = new QAction(this);
     propertiesWindowAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::ALT + Qt::Key_Return)
                                          <<QKeySequence(Qt::ALT + Qt::Key_Enter));
-    connect(propertiesWindowAction, &QAction::triggered, this, [=](){
+    connect(propertiesWindowAction, &QAction::triggered, this, [=]() {
         if (PeonyDesktopApplication::getIconView()->getSelections().count() > 0)
         {
             PropertiesWindow *w = new PropertiesWindow(PeonyDesktopApplication::getIconView()->getSelections());
@@ -487,15 +558,12 @@ void DesktopWindow::initShortcut() {
 
     auto newFolderAction = new QAction(this);
     newFolderAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_N));
-    connect(newFolderAction, &QAction::triggered, this, [=](){
+    connect(newFolderAction, &QAction::triggered, this, [=]() {
         CreateTemplateOperation op(PeonyDesktopApplication::getIconView()->getDirectoryUri(), CreateTemplateOperation::EmptyFolder, tr("New Folder"));
         op.run();
         auto targetUri = op.target();
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-            QTimer::singleShot(500, this, [=](){
-#else
-            QTimer::singleShot(500, [=](){
-#endif
+
+        QTimer::singleShot(500, this, [=]() {
             PeonyDesktopApplication::getIconView()->scrollToSelection(targetUri);
         });
     });
@@ -503,14 +571,14 @@ void DesktopWindow::initShortcut() {
 
     auto refreshAction = new QAction(this);
     refreshAction->setShortcut(Qt::Key_F5);
-    connect(refreshAction, &QAction::triggered, this, [=](){
+    connect(refreshAction, &QAction::triggered, this, [=]() {
         PeonyDesktopApplication::getIconView()->refresh();
     });
     addAction(refreshAction);
 
     QAction *editAction = new QAction(PeonyDesktopApplication::getIconView());
     editAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::ALT + Qt::Key_E)<<Qt::Key_F2);
-    connect(editAction, &QAction::triggered, this, [=](){
+    connect(editAction, &QAction::triggered, this, [=]() {
         auto selections = PeonyDesktopApplication::getIconView()->getSelections();
         if (selections.count() == 1) {
             PeonyDesktopApplication::getIconView()->editUri(selections.first());
