@@ -32,6 +32,12 @@
 #include "directory-view-widget.h"
 #include "advanced-location-bar.h"
 
+#include "directory-view-factory-manager.h"
+#include "directory-view-plugin-iface2.h"
+#include "search-vfs-uri-parser.h"
+#include "file-info.h"
+#include "file-info-job.h"
+
 #include <QHBoxLayout>
 #include <QUrl>
 #include <QMessageBox>
@@ -41,6 +47,10 @@
 #include <QStyleOptionToolButton>
 
 #include <QEvent>
+#include <QApplication>
+#include <QTimer>
+
+#include <KWindowSystem>
 
 #include <QDebug>
 
@@ -49,6 +59,8 @@ static QString terminal_cmd = nullptr;
 
 HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
 {
+    setAttribute(Qt::WA_AcceptTouchEvents);
+
     setMouseTracking(true);
     setStyle(HeaderBarStyle::getStyle());
 
@@ -64,7 +76,7 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
 
     setMovable(false);
 
-    auto a = addAction(QIcon::fromTheme("folder-new-symbolic"), tr("Create Folder"), [=](){
+    auto a = addAction(QIcon::fromTheme("folder-new-symbolic"), tr("Create Folder"), [=]() {
         //use the same function
         m_window->createFolderOperation();
     });
@@ -72,12 +84,13 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
     createFolder->setAutoRaise(false);
     createFolder->setFixedSize(QSize(40, 40));
     createFolder->setIconSize(QSize(16, 16));
+    m_create_folder = createFolder;
 
     addSpacing(2);
 
     //find a terminal when init
     findDefaultTerminal();
-    a = addAction(QIcon::fromTheme("terminal-app-symbolic"), tr("Open Terminal"), [=](){
+    a = addAction(QIcon::fromTheme("terminal-app-symbolic"), tr("Open Terminal"), [=]() {
         //open the default terminal
         openDefaultTerminal();
     });
@@ -95,9 +108,6 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
     goBack->setFixedSize(QSize(36, 28));
     goBack->setIcon(QIcon::fromTheme("go-previous-symbolic"));
     addWidget(goBack);
-    connect(goBack, &QPushButton::clicked, m_window, [=](){
-        m_window->getCurrentPage()->goBack();
-    });
 
     auto goForward = new HeadBarPushButton(this);
     m_go_forward = goForward;
@@ -106,7 +116,7 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
     goForward->setFixedSize(QSize(36, 28));
     goForward->setIcon(QIcon::fromTheme("go-next-symbolic"));
     addWidget(goForward);
-    connect(goForward, &QPushButton::clicked, m_window, [=](){
+    connect(goForward, &QPushButton::clicked, m_window, [=]() {
         m_window->getCurrentPage()->goForward();
     });
 
@@ -116,10 +126,29 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
     m_location_bar = locationBar;
     addWidget(locationBar);
 
+    connect(goBack, &QPushButton::clicked, m_window, [=]() {
+        m_window->getCurrentPage()->goBack();
+        m_location_bar->clearSearchBox();
+    });
+
     connect(m_location_bar, &Peony::AdvancedLocationBar::refreshRequest, [=]()
     {
-         m_window->updateTabPageTitle();
+        m_window->updateTabPageTitle();
     });
+    connect(m_location_bar, &Peony::AdvancedLocationBar::updateFileTypeFilter, [=](const int &index) {
+        m_window->getCurrentPage()->setSortFilter(index);
+    });
+    connect(m_location_bar, &Peony::AdvancedLocationBar::searchRequest, [=](const QString &path, const QString &key){
+        //key is null, clean search content, show all files
+        if (key == "" || key.isNull())
+            Q_EMIT this->updateLocationRequest(path, false);
+        else
+        {
+            auto targetUri = Peony::SearchVFSUriParser::parseSearchKey(path, key, true, false, "", m_search_recursive);
+            Q_EMIT this->updateLocationRequest(targetUri, false);
+        }
+    });
+
     connect(m_location_bar, &Peony::AdvancedLocationBar::updateWindowLocationRequest, this, &HeaderBar::updateLocationRequest);
 
     addSpacing(9);
@@ -143,12 +172,21 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
     m_view_type_menu = new ViewTypeMenu(viewType);
     viewType->setMenu(m_view_type_menu);
 
-    connect(m_view_type_menu, &ViewTypeMenu::switchViewRequest, this, [=](const QString &id, const QIcon &icon){
+    connect(m_view_type_menu, &ViewTypeMenu::switchViewRequest, this, [=](const QString &id, const QIcon &icon, bool resetToZoomLevel) {
         viewType->setText(id);
         viewType->setIcon(icon);
         this->viewTypeChangeRequest(id);
+        if (resetToZoomLevel) {
+            auto viewId = m_window->getCurrentPage()->getView()->viewId();
+            auto factoryManger = Peony::DirectoryViewFactoryManager2::getInstance();
+            auto factory = factoryManger->getFactory(viewId);
+            int zoomLevelHint = factory->zoom_level_hint();
+            m_window->getCurrentPage()->setZoomLevelRequest(zoomLevelHint);
+        }
     });
 
+    connect(m_view_type_menu, &ViewTypeMenu::updateZoomLevelHintRequest, this, &HeaderBar::updateZoomLevelHintRequest);
+
     addSpacing(2);
 
     a = addAction(QIcon::fromTheme("view-sort-ascending-symbolic"), tr("Sort Type"));
@@ -162,7 +200,7 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
     sortType->setMenu(m_sort_type_menu);
 
     connect(m_sort_type_menu, &SortTypeMenu::switchSortTypeRequest, m_window, &MainWindow::setCurrentSortColumn);
-    connect(m_sort_type_menu, &SortTypeMenu::switchSortOrderRequest, m_window, [=](Qt::SortOrder order){
+    connect(m_sort_type_menu, &SortTypeMenu::switchSortOrderRequest, m_window, [=](Qt::SortOrder order) {
         if (order == Qt::AscendingOrder) {
             sortType->setIcon(QIcon::fromTheme("view-sort-ascending-symbolic"));
         } else {
@@ -170,7 +208,7 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
         }
         m_window->setCurrentSortOrder(order);
     });
-    connect(m_sort_type_menu, &QMenu::aboutToShow, m_sort_type_menu, [=](){
+    connect(m_sort_type_menu, &QMenu::aboutToShow, m_sort_type_menu, [=]() {
         m_sort_type_menu->setSortType(m_window->getCurrentSortColumn());
         m_sort_type_menu->setSortOrder(m_window->getCurrentSortOrder());
     });
@@ -187,6 +225,12 @@ HeaderBar::HeaderBar(MainWindow *parent) : QToolBar(parent)
 
     m_operation_menu = new OperationMenu(m_window, this);
     popMenu->setMenu(m_operation_menu);
+
+    for (auto action : actions()) {
+        auto w = widgetForAction(action);
+        w->setProperty("useIconHighlightEffect", true);
+        w->setProperty("iconHighlightEffectMode", 1);
+    }
 }
 
 void HeaderBar::findDefaultTerminal()
@@ -219,49 +263,65 @@ void HeaderBar::openDefaultTerminal()
         return;
     }
 
-    if (terminal_cmd == "mate-terminal" || terminal_cmd == "gnome-terminal")
-    {
-        auto url = m_window->getCurrentUri();
-        auto absPath = url.replace("file://", "");
-        //qDebug() << "working-directory url" <<url<<absPath;
-        QProcess p;
-        p.setProgram(terminal_cmd);
-        p.setArguments(QStringList()<<"--working-directory"<<absPath);
-        p.startDetached();
-        p.waitForFinished(-1);
-    }
-    else
-    {
-        QUrl url = m_window->getCurrentUri();
-        auto directory = url.path().toUtf8().constData();
-        gchar **argv = nullptr;
-        g_shell_parse_argv (terminal_cmd.toUtf8().constData(), nullptr, &argv, nullptr);
-        GError *err = nullptr;
-        g_spawn_async (directory,
-                       argv,
-                       nullptr,
-                       G_SPAWN_SEARCH_PATH,
-                       nullptr,
-                       nullptr,
-                       nullptr,
-                       &err);
-        if (err) {
-            qDebug()<<err->message;
-            g_error_free(err);
-            err = nullptr;
-        }
-        g_strfreev (argv);
+    QUrl url = m_window->getCurrentUri();
+    auto directory = url.path().toUtf8().constData();
+    gchar **argv = nullptr;
+    g_shell_parse_argv (terminal_cmd.toUtf8().constData(), nullptr, &argv, nullptr);
+    GError *err = nullptr;
+    g_spawn_async (directory,
+                   argv,
+                   nullptr,
+                   G_SPAWN_SEARCH_PATH,
+                   nullptr,
+                   nullptr,
+                   nullptr,
+                   &err);
+    if (err) {
+        qDebug()<<err->message;
+        g_error_free(err);
+        err = nullptr;
+        tryOpenAgain();
     }
+    g_strfreev (argv);
+}
+
+void HeaderBar::tryOpenAgain()
+{
+    QUrl url = m_window->getCurrentUri();
+    auto absPath = url.path();
+    qDebug() << "tryOpenAgain url" <<url<<absPath<<terminal_cmd;
+    QProcess p;
+    p.setProgram(terminal_cmd);
+    p.setArguments(QStringList()<<"--working-directory"<<absPath);
+    p.startDetached(p.program(), p.arguments());
+    p.waitForFinished(-1);
 }
 
 void HeaderBar::searchButtonClicked()
 {
     m_search_mode = ! m_search_mode;
-    //qDebug() << "searchButtonClicked" <<m_search_mode;
-    m_search_button->setCheckable(m_search_mode);
-    m_search_button->setChecked(m_search_mode);
-    m_search_button->setDown(m_search_mode);
-    m_location_bar->switchEditMode(m_search_mode);
+    qDebug() << "searchButtonClicked" <<m_search_mode;
+    Q_EMIT this->updateSearchRequest(m_search_mode);
+    setSearchMode(m_search_mode);
+}
+
+void HeaderBar::setSearchMode(bool mode)
+{
+    m_search_button->setCheckable(mode);
+    m_search_button->setChecked(mode);
+    m_search_button->setDown(mode);
+    m_location_bar->switchEditMode(mode);
+}
+
+void HeaderBar::closeSearch()
+{
+    m_search_mode = false;
+    setSearchMode(false);
+}
+
+void HeaderBar::updateSearchRecursive(bool recursive)
+{
+    m_search_recursive = recursive;
 }
 
 void HeaderBar::addSpacing(int pixel)
@@ -312,7 +372,8 @@ void HeaderBar::updateIcons()
     qDebug()<<m_window->getCurrentUri();
     qDebug()<<m_window->getCurrentSortColumn();
     qDebug()<<m_window->getCurrentSortOrder();
-    m_view_type_menu->setCurrentView(m_window->getCurrentPage()->getView()->viewId());
+    m_view_type_menu->setCurrentDirectory(m_window->getCurrentUri());
+    m_view_type_menu->setCurrentView(m_window->getCurrentPage()->getView()->viewId(), true);
     m_sort_type_menu->switchSortTypeRequest(m_window->getCurrentSortColumn());
     m_sort_type_menu->switchSortOrderRequest(m_window->getCurrentSortOrder());
 
@@ -320,6 +381,21 @@ void HeaderBar::updateIcons()
     m_go_back->setEnabled(m_window->getCurrentPage()->canGoBack());
     m_go_forward->setEnabled(m_window->getCurrentPage()->canGoForward());
 
+    //fix create folder fail issue in special path
+    auto curUri = m_window->getCurrentUri();
+    auto info = Peony::FileInfo::fromUri(curUri, false);
+    Peony::FileInfoJob job(info);
+    job.querySync();
+    if (info->canWrite())
+        m_create_folder->setEnabled(true);
+    else
+        m_create_folder->setEnabled(false);
+
+    m_go_back->setProperty("useIconHighlightEffect", true);
+    m_go_back->setProperty("iconHighlightEffectMode", 1);
+    m_go_forward->setProperty("useIconHighlightEffect", true);
+    m_go_forward->setProperty("iconHighlightEffectMode", 1);
+
     //maximize & restore
     updateMaximizeState();
 }
@@ -424,14 +500,30 @@ HeaderBarContainer::HeaderBarContainer(QWidget *parent) : QToolBar(parent)
 bool HeaderBarContainer::eventFilter(QObject *obj, QEvent *e)
 {
     Q_UNUSED(obj)
-    if (e->type() == QEvent::MouseMove) {
-        //auto w = qobject_cast<QWidget *>(obj);
-        QCursor c;
-        c.setShape(Qt::ArrowCursor);
-        //this->setCursor(c);
-        //w->setCursor(c);
-        this->topLevelWidget()->setCursor(c);
+    auto window = qobject_cast<MainWindow *>(obj);
+    if (window) {
+        if (e->type() == QEvent::Resize) {
+            if (window->isMaximized()) {
+                m_max_or_restore->setIcon(QIcon::fromTheme("window-restore-symbolic"));
+                //m_max_or_restore->setToolTip(tr("Restore"));
+            } else {
+                m_max_or_restore->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
+                //m_max_or_restore->setToolTip(tr("Maximize"));
+            }
+        }
+        return false;
+    } else {
+        if (e->type() == QEvent::MouseMove) {
+            //auto w = qobject_cast<QWidget *>(obj);
+            QCursor c;
+            c.setShape(Qt::ArrowCursor);
+            //this->setCursor(c);
+            //w->setCursor(c);
+            this->topLevelWidget()->setCursor(c);
+        }
+
     }
+
     return false;
 }
 
@@ -448,6 +540,8 @@ void HeaderBarContainer::addHeaderBar(HeaderBar *headerBar)
 
     m_internal_widget->setLayout(m_layout);
     addWidget(m_internal_widget);
+
+    m_header_bar->m_window->installEventFilter(this);
 }
 
 void HeaderBarContainer::addWindowButtons()
@@ -465,27 +559,32 @@ void HeaderBarContainer::addWindowButtons()
     minimize->setAutoRaise(false);
     minimize->setFixedSize(QSize(40, 40));
     minimize->setIconSize(QSize(16, 16));
-    connect(minimize, &QToolButton::clicked, this, [=](){
-        m_header_bar->m_window->showMinimized();
+    connect(minimize, &QToolButton::clicked, this, [=]() {
+        KWindowSystem::minimizeWindow(m_header_bar->m_window->winId());
+        //m_header_bar->m_window->showMinimized();
     });
 
     //window-maximize-symbolic
     //window-restore-symbolic
     auto maximizeAndRestore = new QToolButton(m_internal_widget);
+    maximizeAndRestore->setToolTip(tr("Maximize/Restore"));
     maximizeAndRestore->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
     maximizeAndRestore->setAutoRaise(false);
     maximizeAndRestore->setFixedSize(QSize(40, 40));
     maximizeAndRestore->setIconSize(QSize(16, 16));
-    connect(maximizeAndRestore, &QToolButton::clicked, this, [=](){
+    connect(maximizeAndRestore, &QToolButton::clicked, this, [=]() {
         m_header_bar->m_window->maximizeOrRestore();
 
         bool maximized = m_header_bar->m_window->isMaximized();
         if (maximized) {
             maximizeAndRestore->setIcon(QIcon::fromTheme("window-restore-symbolic"));
+            //maximizeAndRestore->setToolTip(tr("Restore"));
         } else {
             maximizeAndRestore->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
+            //maximizeAndRestore->setToolTip(tr("Maximize"));
         }
     });
+    m_max_or_restore = maximizeAndRestore;
 
     auto close = new QToolButton(m_internal_widget);
     close->setIcon(QIcon::fromTheme("window-close-symbolic"));
@@ -493,10 +592,21 @@ void HeaderBarContainer::addWindowButtons()
     close->setAutoRaise(false);
     close->setFixedSize(QSize(40, 40));
     close->setIconSize(QSize(16, 16));
-    connect(close, &QToolButton::clicked, this, [=](){
+    connect(close, &QToolButton::clicked, this, [=]() {
         m_header_bar->m_window->close();
     });
 
+    connect(qApp, &QApplication::paletteChanged, close, [=](){
+        QTimer::singleShot(100, this, [=](){
+            auto palette = qApp->palette();
+            palette.setColor(QPalette::Highlight, QColor("#E54A50"));
+            close->setPalette(palette);
+        });
+    });
+    auto palette = qApp->palette();
+    palette.setColor(QPalette::Highlight, QColor("#E54A50"));
+    close->setPalette(palette);
+
     layout->addWidget(minimize);
     layout->addWidget(maximizeAndRestore);
     layout->addWidget(close);
@@ -509,4 +619,10 @@ void HeaderBarContainer::addWindowButtons()
     maximizeAndRestore->installEventFilter(this);
     close->setMouseTracking(true);
     close->installEventFilter(this);
+
+    for (int i = 0; i < 3; i++) {
+        auto w = layout->itemAt(i)->widget();
+        w->setProperty("useIconHighlightEffect", true);
+        w->setProperty("iconHighlightEffectMode", 1);
+    }
 }
