@@ -29,10 +29,12 @@
 #include <QLabel>
 #include <QToolButton>
 #include <QWidgetAction>
+#include <KWindowSystem>
 
 #include "global-settings.h"
 #include "clipboard-utils.h"
 #include "file-operation-utils.h"
+#include "file-operation-manager.h"
 
 OperationMenu::OperationMenu(MainWindow *window, QWidget *parent) : QMenu(parent)
 {
@@ -60,28 +62,39 @@ OperationMenu::OperationMenu(MainWindow *window, QWidget *parent) : QMenu(parent
 
 //    addSeparator();
 
-    auto keepAllow = addAction(tr("Keep Allow"), this, [=](bool checked){
-        m_window->setWindowFlag(Qt::WindowStaysOnTopHint, checked);
-        m_window->show();
+    auto keepAllow = addAction(tr("Keep Allow"), this, [=](bool checked) {
+//        if (checked)
+//            m_window->setWindowFlags(Qt::WindowStaysOnTopHint|m_window->windowFlags());
+//        else
+//            m_window->setWindowFlags(m_window->windowFlags() & ~Qt::WindowStaysOnTopHint);
+
+//        m_window->show();
+
+        //use kf5 interface to fix set on top has no effect issue
+        if (checked)
+            KWindowSystem::setState(m_window->winId(), KWindowSystem::KeepAbove);
+        else
+            KWindowSystem::clearState(m_window->winId(), KWindowSystem::KeepAbove);
     });
     keepAllow->setCheckable(true);
 
-    auto showHidden = addAction(tr("Show Hidden"), this, [=](bool checked){
+    auto showHidden = addAction(tr("Show Hidden"), this, [=](bool checked) {
         //window set show hidden
         m_window->setShowHidden();
     });
     m_show_hidden = showHidden;
     showHidden->setCheckable(true);
 
-    auto forbidThumbnailing = addAction(tr("Forbid thumbnailing"), this, [=](bool checked){
+    auto forbidThumbnailing = addAction(tr("Forbid thumbnailing"), this, [=](bool checked) {
         //FIXME:
         Peony::GlobalSettings::getInstance()->setValue("do-not-thumbnail", checked);
+        m_window->refresh();
     });
     m_forbid_thumbnailing = forbidThumbnailing;
     forbidThumbnailing->setCheckable(true);
     forbidThumbnailing->setChecked(Peony::GlobalSettings::getInstance()->getValue("do-not-thumbnail").toBool());
 
-    auto residentInBackend = addAction(tr("Resident in Backend"), this, [=](bool checked){
+    auto residentInBackend = addAction(tr("Resident in Backend"), this, [=](bool checked) {
         //FIXME:
         Peony::GlobalSettings::getInstance()->setValue("resident", checked);
         qApp->setQuitOnLastWindowClosed(!checked);
@@ -90,13 +103,19 @@ OperationMenu::OperationMenu(MainWindow *window, QWidget *parent) : QMenu(parent
     residentInBackend->setCheckable(true);
     residentInBackend->setChecked(Peony::GlobalSettings::getInstance()->getValue("resident").toBool());
 
+    auto allowFileOpParallel = addAction(tr("Parallel Operations"), this, [=](bool checked){
+        Peony::FileOperationManager::getInstance()->setAllowParallel(checked);
+    });
+    allowFileOpParallel->setCheckable(true);
+    allowFileOpParallel->setChecked(Peony::FileOperationManager::getInstance()->isAllowParallel());
+
     addSeparator();
 
-    addAction(QIcon::fromTheme("help-app-symbolic"), tr("Help"), this, [=](){
+    addAction(QIcon::fromTheme("help-app-symbolic"), tr("Help"), this, [=]() {
         PeonyApplication::help();
     });
 
-    addAction(QIcon::fromTheme("help-about", QIcon::fromTheme("gtk-about-symbolic")), tr("About"), this, [=](){
+    addAction(QIcon::fromTheme("help-about", QIcon::fromTheme("gtk-about-symbolic")), tr("About"), this, [=]() {
         PeonyApplication::about();
     });
 }
@@ -104,16 +123,16 @@ OperationMenu::OperationMenu(MainWindow *window, QWidget *parent) : QMenu(parent
 void OperationMenu::updateMenu()
 {
     m_show_hidden->setChecked(Peony::GlobalSettings::getInstance()->isExist("show-hidden")?
-                                  Peony::GlobalSettings::getInstance()->getValue("show-hidden").toBool():
-                                  false);
+                              Peony::GlobalSettings::getInstance()->getValue("show-hidden").toBool():
+                              false);
 
     m_forbid_thumbnailing->setChecked(Peony::GlobalSettings::getInstance()->isExist("do-not-thumbnail")?
-                                          Peony::GlobalSettings::getInstance()->getValue("do-not-thumbnail").toBool():
-                                          false);
+                                      Peony::GlobalSettings::getInstance()->getValue("do-not-thumbnail").toBool():
+                                      false);
 
     m_resident_in_backend->setChecked(Peony::GlobalSettings::getInstance()->isExist("resident")?
-                                          Peony::GlobalSettings::getInstance()->getValue("resident").toBool():
-                                          false);
+                                      Peony::GlobalSettings::getInstance()->getValue("resident").toBool():
+                                      false);
 
     //get window current directory and selections, then update ohter actions.
     m_edit_widget->updateActions(m_window->getCurrentUri(), m_window->getCurrentSelections());
@@ -157,29 +176,40 @@ OperationMenuEditWidget::OperationMenuEditWidget(MainWindow *window, QWidget *pa
     auto trash = new QToolButton(this);
     m_trash = trash;
     trash->setFixedSize(QSize(40, 40));
-    trash->setIcon(QIcon::fromTheme("user-trash-symbolic"));
+    trash->setIcon(QIcon::fromTheme("ukui-user-trash"));
     trash->setIconSize(QSize(16, 16));
     trash->setAutoRaise(false);
     hbox->addWidget(trash);
 
     vbox->addLayout(hbox);
 
-    connect(m_copy, &QToolButton::clicked, this, [=](){
-        Peony::ClipboardUtils::setClipboardFiles(window->getCurrentSelections(), false);
-        Q_EMIT operationAccepted();
+    connect(m_copy, &QToolButton::clicked, this, [=]() {
+        if (!window->getCurrentSelections().isEmpty()) {
+            if (window->getCurrentSelections().first().startsWith("trash://", Qt::CaseInsensitive)) {
+                return ;
+            }
+
+            Peony::ClipboardUtils::setClipboardFiles(window->getCurrentSelections(), false);
+            Q_EMIT operationAccepted();
+        }
     });
 
-    connect(m_cut, &QToolButton::clicked, this, [=](){
-        Peony::ClipboardUtils::setClipboardFiles(window->getCurrentSelections(), true);
-        Q_EMIT operationAccepted();
+    connect(m_cut, &QToolButton::clicked, this, [=]() {
+        if (!window->getCurrentSelections().isEmpty()) {
+            if (window->getCurrentSelections().first().startsWith("trash://", Qt::CaseInsensitive)) {
+                return ;
+            }
+            Peony::ClipboardUtils::setClipboardFiles(window->getCurrentSelections(), true);
+            Q_EMIT operationAccepted();
+        }
     });
 
-    connect(m_paste, &QToolButton::clicked, this, [=](){
+    connect(m_paste, &QToolButton::clicked, this, [=]() {
         Peony::ClipboardUtils::pasteClipboardFiles(window->getCurrentUri());
         Q_EMIT operationAccepted();
     });
 
-    connect(m_trash, &QToolButton::clicked, this, [=](){
+    connect(m_trash, &QToolButton::clicked, this, [=]() {
         if (window->getCurrentUri() == "trash:///") {
             Peony::FileOperationUtils::executeRemoveActionWithDialog(window->getCurrentSelections());
         } else {
@@ -187,6 +217,19 @@ OperationMenuEditWidget::OperationMenuEditWidget(MainWindow *window, QWidget *pa
         }
         Q_EMIT operationAccepted();
     });
+
+    copy->setProperty("useIconHighlightEffect", true);
+    copy->setProperty("iconHighlightEffectMode", 1);
+    copy->setProperty("fillIconSymbolicColor", true);
+    paste->setProperty("useIconHighlightEffect", true);
+    paste->setProperty("iconHighlightEffectMode", 1);
+    paste->setProperty("fillIconSymbolicColor", true);
+    cut->setProperty("useIconHighlightEffect", true);
+    cut->setProperty("iconHighlightEffectMode", 1);
+    cut->setProperty("fillIconSymbolicColor", true);
+    trash->setProperty("useIconHighlightEffect", true);
+    trash->setProperty("iconHighlightEffectMode", 1);
+    trash->setProperty("fillIconSymbolicColor", true);
 }
 
 void OperationMenuEditWidget::updateActions(const QString &currentDirUri, const QStringList &selections)
