@@ -1,7 +1,7 @@
 /*
  * Peony-Qt
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,6 @@
 #include "preview-page-plugin-iface.h"
 #include "directory-view-factory-manager.h"
 #include "directory-view-plugin-iface.h"
-#include "standard-view-proxy.h"
 #include "directory-view-container.h"
 
 #include "path-edit.h"
@@ -101,13 +100,15 @@
 #include <QDBusMessage>
 #include <QDBusConnectionInterface>
 
+#include <KWindowSystem>
+
 #define KYLIN_USER_GUIDE_PATH "/"
 #define KYLIN_USER_GUIDE_SERVICE QString("com.kylinUserGuide.hotel_%1").arg(getuid())
 #define KYLIN_USER_GUIDE_INTERFACE "com.guide.hotel"
 
 PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicationName) : SingleApplication (argc, argv, applicationName, true)
 {
-    setApplicationVersion("v2.1.0");
+    setApplicationVersion("v3.0.0");
     setApplicationName("peony-qt");
     //setApplicationDisplayName(tr("Peony-Qt"));
 
@@ -126,7 +127,7 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
     QTranslator *t3 = new QTranslator(this);
     t3->load("/usr/share/qt5/translations/qt_"+QLocale::system().name());
     QApplication::installTranslator(t3);
-    //setStyle(Peony::ComplementaryStyle::getStyle());
+    setStyle(Peony::ComplementaryStyle::getStyle());
 
     parser.addOption(quitOption);
     parser.addOption(showItemsOption);
@@ -138,6 +139,15 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
     if (this->isSecondary()) {
         parser.addHelpOption();
         parser.addVersionOption();
+        if (this->arguments().count() == 2 && arguments().last() == ".") {
+            QStringList args;
+            auto dir = g_get_current_dir();
+            args<<"peony"<<dir;
+            g_free(dir);
+            auto message = args.join(' ').toUtf8();
+            sendMessage(message);
+            return;
+        }
         parser.process(arguments());
         auto message = this->arguments().join(' ').toUtf8();
         sendMessage(message);
@@ -159,21 +169,21 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
             setStyle("gtk2");
         }
         QMessageBox::warning(nullptr, tr("Warning"), tr("Peony-Qt can not get the system's icon theme. "
-                                                        "There are 2 reasons might lead to this problem:\n\n"
-                                                        "1. Peony-Qt might be running as root, "
-                                                        "that means you have the higher permission "
-                                                        "and can do some things which normally forbidden. "
-                                                        "But, you should learn that if you were in a "
-                                                        "root, the virtual file system will lose some "
-                                                        "featrue such as you can not use \"My Computer\", "
-                                                        "the theme and icons might also went wrong. So, run "
-                                                        "peony-qt in a root is not recommended.\n\n"
-                                                        "2. You are using a non-qt theme for your system but "
-                                                        "you didn't install the platform theme plugin for qt's "
-                                                        "applications. If you are using gtk-theme, try installing "
-                                                        "the qt5-gtk2-platformtheme package to resolve this problem."));
+                             "There are 2 reasons might lead to this problem:\n\n"
+                             "1. Peony-Qt might be running as root, "
+                             "that means you have the higher permission "
+                             "and can do some things which normally forbidden. "
+                             "But, you should learn that if you were in a "
+                             "root, the virtual file system will lose some "
+                             "featrue such as you can not use \"My Computer\", "
+                             "the theme and icons might also went wrong. So, run "
+                             "peony-qt in a root is not recommended.\n\n"
+                             "2. You are using a non-qt theme for your system but "
+                             "you didn't install the platform theme plugin for qt's "
+                             "applications. If you are using gtk-theme, try installing "
+                             "the qt5-gtk2-platformtheme package to resolve this problem."));
     }
-    Peony::SearchVFSRegister::registSearchVFS();
+    //Peony::SearchVFSRegister::registSearchVFS();
     //QIcon::setThemeName("ukui-icon-theme-one");
     //setAttribute(Qt::AA_UseHighDpiPixmaps);
     //setAttribute(Qt::AA_EnableHighDpiScaling);
@@ -212,10 +222,10 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
                 QMenu *menu = new QMenu(widget);
                 QStringList l;
                 Peony::MenuPluginInterface::Types types = Peony::MenuPluginInterface::Types(Peony::MenuPluginInterface::File|
-                                                                                            Peony::MenuPluginInterface::Volume|
-                                                                                            Peony::MenuPluginInterface::DirectoryBackground|
-                                                                                            Peony::MenuPluginInterface::DesktopBackground|
-                                                                                            Peony::MenuPluginInterface::Other);
+                        Peony::MenuPluginInterface::Volume|
+                        Peony::MenuPluginInterface::DirectoryBackground|
+                        Peony::MenuPluginInterface::DesktopBackground|
+                        Peony::MenuPluginInterface::Other);
                 auto fileActions = iface->menuActions(types, nullptr, l);
                 for (auto action : fileActions) {
                     action->setParent(menu);
@@ -223,7 +233,7 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
                 menu->addActions(fileActions);
 
                 widget->setContextMenuPolicy(Qt::CustomContextMenu);
-                widget->connect(widget, &QWidget::customContextMenuRequested, [menu](const QPoint /*&pos*/){
+                widget->connect(widget, &QWidget::customContextMenuRequested, [menu](const QPoint /*&pos*/) {
                     //menu->exec(pos);
                     menu->exec(QCursor::pos());
                 });
@@ -243,7 +253,7 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
         auto plugin = previewManager->getPlugin(name);
         auto w = plugin->createPreviewPage();
         w->startPreview();
-        QTimer::singleShot(1000, [=](){
+        QTimer::singleShot(1000, [=]() {
             w->cancel();
         });
         auto w1 = dynamic_cast<QWidget*>(w);
@@ -285,11 +295,11 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
         //proxy->setDirectoryUri("file:///home/lanyue");
         proxy->setDirectoryUri("file:///");
         proxy->beginLocationChange();
-        QTimer::singleShot(1000, [=](){
+        QTimer::singleShot(1000, [=]() {
             //proxy->invertSelections();
         });
-        connect(proxy, &Peony::DirectoryViewProxyIface::viewDoubleClicked, [=](const QString &uri){
-            qDebug()<<"double clicked"<<uri;
+        connect(proxy, &Peony::DirectoryViewProxyIface::viewDoubleClicked, [=](const QString &uri) {
+            qDebug()<<"app double clicked"<<uri;
             auto info = Peony::FileInfo::fromUri(uri);
             if (info->isDir() || info->isVolume() || uri.startsWith("network:")) {
                 proxy->setDirectoryUri(uri);
@@ -327,7 +337,7 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
     auto tabPage = new Peony::TabPage(w);
     w->setCentralWidget(tabPage);
 
-    w->connect(e, &QLineEdit::returnPressed, [=](){
+    w->connect(e, &QLineEdit::returnPressed, [=]() {
         tabPage->addPage(e->text());
     });
 
@@ -353,16 +363,16 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
     Peony::PathEdit *edit = new Peony::PathEdit(widget);
     bar->setRootUri("file://" + QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
     bar->connect(bar, &Peony::LocationBar::groupChangedRequest, bar, &Peony::LocationBar::setRootUri);
-    bar->connect(bar, &Peony::LocationBar::blankClicked, [=](){
+    bar->connect(bar, &Peony::LocationBar::blankClicked, [=]() {
         layout->setCurrentWidget(edit);
         edit->setUri(bar->getCurentUri());
     });
     edit->setUri(bar->getCurentUri());
-    edit->connect(edit, &Peony::PathEdit::returnPressed, [=](){
+    edit->connect(edit, &Peony::PathEdit::returnPressed, [=]() {
         bar->setRootUri(edit->text());
         layout->setCurrentWidget(bar);
     });
-    edit->connect(edit, &Peony::PathEdit::editCancelled, [=](){
+    edit->connect(edit, &Peony::PathEdit::editCancelled, [=]() {
         layout->setCurrentWidget(bar);
     });
     layout->addWidget(bar);
@@ -433,11 +443,11 @@ PeonyApplication::PeonyApplication(int &argc, char *argv[], const char *applicat
     auto op = new Peony::FileCountOperation(l);
     QThreadPool::globalInstance()->start(op);
 
-    connect(op, &Peony::FileCountOperation::operationPreparedOne, [=](const QString &uri){
+    connect(op, &Peony::FileCountOperation::operationPreparedOne, [=](const QString &uri) {
         qDebug()<<uri;
     });
 
-    connect(op, &Peony::FileCountOperation::operationFinished, [=](){
+    connect(op, &Peony::FileCountOperation::operationFinished, [=]() {
         qDebug()<<"finished";
         Peony::FileInfoManager::getInstance()->showState();
     });
@@ -470,7 +480,7 @@ void PeonyApplication::parseCmd(quint32 id, QByteArray msg)
 
     parser.process(args);
     if (parser.isSet(quitOption)) {
-        QTimer::singleShot(1, [=](){
+        QTimer::singleShot(1, [=]() {
             qApp->quit();
         });
         return;
@@ -502,13 +512,14 @@ void PeonyApplication::parseCmd(quint32 id, QByteArray msg)
             for (auto parentUri : parentUris) {
                 auto window = new MainWindow(parentUri);
                 //Peony::FMWindow *window = new Peony::FMWindow(parentUri);
-                connect(window, &MainWindow::locationChangeEnd, [=](){
-                    QTimer::singleShot(500, [=]{
+                connect(window, &MainWindow::locationChangeEnd, [=]() {
+                    QTimer::singleShot(500, [=] {
                         window->getCurrentPage()->getView()->setSelections(itemHash.value(parentUri));
                         window->getCurrentPage()->getView()->scrollToSelection(itemHash.value(parentUri).first());
                     });
                 });
                 window->show();
+                KWindowSystem::raiseWindow(window->winId());
             }
         }
 
@@ -521,12 +532,14 @@ void PeonyApplication::parseCmd(quint32 id, QByteArray msg)
                 window->addNewTabs(uris);
             }
             window->show();
+            KWindowSystem::raiseWindow(window->winId());
         }
         if (parser.isSet(showPropertiesOption)) {
             QStringList uris = Peony::FileUtils::toDisplayUris(parser.positionalArguments());
 
             Peony::PropertiesWindow *window = new Peony::PropertiesWindow(uris);
             window->show();
+            KWindowSystem::raiseWindow(window->winId());
         }
     } else {
         if (!parser.positionalArguments().isEmpty()) {
@@ -539,15 +552,17 @@ void PeonyApplication::parseCmd(quint32 id, QByteArray msg)
             }
             window->setAttribute(Qt::WA_DeleteOnClose);
             window->show();
+            KWindowSystem::raiseWindow(window->winId());
         } else {
             auto window = new MainWindow;
             //auto window = new Peony::FMWindow;
             window->setAttribute(Qt::WA_DeleteOnClose);
             window->show();
+            KWindowSystem::raiseWindow(window->winId());
         }
     }
 
-    connect(this, &QApplication::paletteChanged, this, [=](const QPalette &pal){
+    connect(this, &QApplication::paletteChanged, this, [=](const QPalette &pal) {
         for (auto w : allWidgets()) {
             w->setPalette(pal);
             w->update();
@@ -559,11 +574,11 @@ void PeonyApplication::about()
 {
     QMessageBox::about(nullptr,
                        tr("Peony Qt"),
-                       tr("Authour: \n"
+                       tr("Author:\n"
                           "\tYue Lan <lanyue@kylinos.cn>\n"
                           "\tMeihong He <hemeihong@kylinos.cn>\n"
                           "\n"
-                          "Copyright (C): 2019-2020, KYLIN Software Co., Ltd."));
+                          "Copyright (C): 2020, KylinSoft Co., Ltd."));
 }
 
 void PeonyApplication::help()
@@ -596,9 +611,9 @@ void PeonyApplication::showGuide(const QString &appName)
     bool bRet  = false;
 
     QDBusMessage m = QDBusMessage::createMethodCall(KYLIN_USER_GUIDE_SERVICE,
-                                                    KYLIN_USER_GUIDE_PATH,
-                                                    KYLIN_USER_GUIDE_INTERFACE,
-                                                    "showGuide");
+                     KYLIN_USER_GUIDE_PATH,
+                     KYLIN_USER_GUIDE_INTERFACE,
+                     "showGuide");
     m << appName;
 
     QDBusMessage response = QDBusConnection::sessionBus().call(m);
