@@ -78,12 +78,30 @@
 #include <QTimer>
 #include <QDesktopServices>
 
+#include <QProcess>
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+#include <QPainterPath>
+#endif
+
 #include <QDebug>
 
 #include <X11/Xlib.h>
+#include <KWindowEffects>
+
+// NOTE build failed on Archlinux. Can't detect `QGSettings/QGSettings' header
+// fixed by replaced `QGSettings/QGSettings' with `QGSettings'
+#include <QGSettings>
+
+#define FONT_SETTINGS "org.ukui.style"
+
+static MainWindow *last_resize_window = nullptr;
 
 MainWindow::MainWindow(const QString &uri, QWidget *parent) : QMainWindow(parent)
 {
+    setContextMenuPolicy(Qt::CustomContextMenu);
+    installEventFilter(this);
+
     setWindowIcon(QIcon::fromTheme("system-file-manager"));
     setWindowTitle(tr("File Manager"));
 
@@ -99,10 +117,13 @@ MainWindow::MainWindow(const QString &uri, QWidget *parent) : QMainWindow(parent
     //setGraphicsEffect(m_effect);
 
     setAnimated(false);
-    //setAttribute(Qt::WA_DeleteOnClose); //double free, why?
+    setAttribute(Qt::WA_DeleteOnClose);
     setAttribute(Qt::WA_TranslucentBackground);
     //setAttribute(Qt::WA_OpaquePaintEvent);
-    setWindowFlag(Qt::FramelessWindowHint);
+    //fix double window base buttons issue, not effect MinMax button hints
+    auto flags = windowFlags() &~Qt::WindowMinMaxButtonsHint;
+    setWindowFlags(flags |Qt::FramelessWindowHint);
+    //setWindowFlags(windowFlags()|Qt::FramelessWindowHint);
     setContentsMargins(4, 4, 4, 4);
 
     //bind resize handler
@@ -120,25 +141,97 @@ MainWindow::MainWindow(const QString &uri, QWidget *parent) : QMainWindow(parent
     initUI(uri);
 }
 
+MainWindow::~MainWindow()
+{
+    if (last_resize_window == this) {
+        auto settings = Peony::GlobalSettings::getInstance();
+        settings->setValue(DEFAULT_WINDOW_SIZE, this->size());
+        last_resize_window = nullptr;
+    }
+}
+
+bool MainWindow::eventFilter(QObject *watched, QEvent *event)
+{
+    if (event->type() == QEvent::MouseMove) {
+        auto me = static_cast<QMouseEvent *>(event);
+        auto widget = this->childAt(me->pos());
+        if (!widget) {
+            // set default sidebar width flag
+            m_should_save_side_bar_width = true;
+        }
+    }
+
+    if (event->type() == QEvent::MouseButtonRelease) {
+        if (m_should_save_side_bar_width) {
+            // real set default side bar width
+            auto settings = Peony::GlobalSettings::getInstance();
+            settings->setValue(DEFAULT_SIDEBAR_WIDTH, m_side_bar->width());
+        }
+        m_should_save_side_bar_width = false;
+    }
+
+    return false;
+}
+
 QSize MainWindow::sizeHint() const
 {
     auto screenSize = QApplication::primaryScreen()->size();
-    return QSize(screenSize*2/3);
+    QSize defaultSize = (Peony::GlobalSettings::getInstance()->getValue(DEFAULT_WINDOW_SIZE)).toSize();
+    int width = qMin(defaultSize.width(), screenSize.width());
+    int height = qMin(defaultSize.height(), screenSize.height());
+    //return screenSize*2/3;
+    //qreal dpr = qApp->devicePixelRatio();
+    return QSize(width, height);
 }
 
 Peony::FMWindowIface *MainWindow::create(const QString &uri)
 {
-    return new MainWindow(uri);
+    auto window = new MainWindow(uri);
+    if (currentViewSupportZoom())
+        window->setCurrentViewZoomLevel(this->currentViewZoomLevel());
+    return window;
 }
 
 Peony::FMWindowIface *MainWindow::create(const QStringList &uris)
 {
-    if (uris.isEmpty())
-        return new MainWindow;
+    if (uris.isEmpty()) {
+        auto window = new MainWindow;
+        if (currentViewSupportZoom())
+            window->setCurrentViewZoomLevel(this->currentViewZoomLevel());
+        return window;
+    }
     auto uri = uris.first();
     auto l = uris;
     l.removeAt(0);
     auto window = new MainWindow(uri);
+    if (currentViewSupportZoom())
+        window->setCurrentViewZoomLevel(this->currentViewZoomLevel());
+    window->addNewTabs(l);
+    return window;
+}
+
+Peony::FMWindowIface *MainWindow::createWithZoomLevel(const QString &uri, int zoomLevel)
+{
+    auto window = new MainWindow(uri);
+    if (currentViewSupportZoom())
+        window->setCurrentViewZoomLevel(zoomLevel);
+    return window;
+}
+
+Peony::FMWindowIface *MainWindow::createWithZoomLevel(const QStringList &uris, int zoomLevel)
+{
+    if (uris.isEmpty()) {
+        auto window = new MainWindow;
+        if (currentViewSupportZoom())
+            window->setCurrentViewZoomLevel(zoomLevel);
+        return window;
+    }
+    auto uri = uris.first();
+    auto l = uris;
+    l.removeAt(0);
+    auto window = new MainWindow(uri);
+    if (currentViewSupportZoom())
+        window->setCurrentViewZoomLevel(zoomLevel);
     window->addNewTabs(l);
     return window;
 }
@@ -159,6 +252,18 @@ void MainWindow::checkSettings()
     m_show_hidden_file = settings->isExist("show-hidden")? settings->getValue("show-hidden").toBool(): false;
     m_use_default_name_sort_order = settings->isExist("chinese-first")? settings->getValue("chinese-first").toBool(): false;
     m_folder_first = settings->isExist("folder-first")? settings->getValue("folder-first").toBool(): true;
+
+    //font monitor
+    QGSettings *fontSetting = new QGSettings(FONT_SETTINGS, QByteArray(), this);
+    connect(fontSetting, &QGSettings::changed, this, [=](const QString &key){
+        qDebug() << "fontSetting changed:" << key;
+        if (key == "systemFont" || key == "systemFontSize")
+        {
+            QFont font = this->font();
+            for(auto widget : qApp->allWidgets())
+                widget->setFont(font);
+        }
+    });
 }
 
 void MainWindow::setShortCuts()
@@ -173,7 +278,7 @@ void MainWindow::setShortCuts()
     QAction *showHiddenAction = new QAction(this);
     showHiddenAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
     addAction(showHiddenAction);
-    connect(showHiddenAction, &QAction::triggered, this, [=](){
+    connect(showHiddenAction, &QAction::triggered, this, [=]() {
         //qDebug() << "show hidden";
         this->setShowHidden();
     });
@@ -181,21 +286,21 @@ void MainWindow::setShortCuts()
     auto undoAction = new QAction(QIcon::fromTheme("edit-undo-symbolic"), tr("Undo"), this);
     undoAction->setShortcut(QKeySequence::Undo);
     addAction(undoAction);
-    connect(undoAction, &QAction::triggered, [=](){
+    connect(undoAction, &QAction::triggered, [=]() {
         Peony::FileOperationManager::getInstance()->undo();
     });
 
     auto redoAction = new QAction(QIcon::fromTheme("edit-redo-symbolic"), tr("Redo"), this);
     redoAction->setShortcut(QKeySequence::Redo);
     addAction(redoAction);
-    connect(redoAction, &QAction::triggered, [=](){
+    connect(redoAction, &QAction::triggered, [=]() {
         Peony::FileOperationManager::getInstance()->redo();
     });
 
     //add CTRL+D for delete operation
     auto trashAction = new QAction(this);
     trashAction->setShortcuts(QList<QKeySequence>()<<Qt::Key_Delete<<QKeySequence(Qt::CTRL + Qt::Key_D));
-    connect(trashAction, &QAction::triggered, [=](){
+    connect(trashAction, &QAction::triggered, [=]() {
         auto uris = this->getCurrentSelections();
         if (!uris.isEmpty()) {
             bool isTrash = this->getCurrentUri() == "trash:///";
@@ -211,29 +316,30 @@ void MainWindow::setShortCuts()
     auto deleteAction = new QAction(this);
     deleteAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::SHIFT + Qt::Key_Delete));
     addAction(deleteAction);
-    connect(deleteAction, &QAction::triggered, [=](){
+    connect(deleteAction, &QAction::triggered, [=]() {
         auto uris = this->getCurrentSelections();
         Peony::FileOperationUtils::executeRemoveActionWithDialog(uris);
     });
 
     auto searchAction = new QAction(this);
     searchAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::CTRL + Qt::Key_F));
-    connect(searchAction, &QAction::triggered, this, [=](){
-        m_header_bar->startEdit(true);
+    connect(searchAction, &QAction::triggered, this, [=]() {
+        m_is_search = ! m_is_search;
+        m_header_bar->startEdit(m_is_search);
     });
     addAction(searchAction);
 
     //F4 or Alt+D, change to address
     auto locationAction = new QAction(this);
     locationAction->setShortcuts(QList<QKeySequence>()<<Qt::Key_F4<<QKeySequence(Qt::ALT + Qt::Key_D));
-    connect(locationAction, &QAction::triggered, this, [=](){
+    connect(locationAction, &QAction::triggered, this, [=]() {
         m_header_bar->startEdit();
     });
     addAction(locationAction);
 
     auto newWindowAction = new QAction(this);
     newWindowAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_N));
-    connect(newWindowAction, &QAction::triggered, this, [=](){
+    connect(newWindowAction, &QAction::triggered, this, [=]() {
         MainWindow *newWindow = new MainWindow(getCurrentUri());
         newWindow->show();
     });
@@ -241,28 +347,28 @@ void MainWindow::setShortCuts()
 
     auto closeWindowAction = new QAction(this);
     closeWindowAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::ALT + Qt::Key_F4));
-    connect(closeWindowAction, &QAction::triggered, this, [=](){
+    connect(closeWindowAction, &QAction::triggered, this, [=]() {
         this->close();
     });
     addAction(closeWindowAction);
 
     auto aboutAction = new QAction(this);
     aboutAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_F2));
-    connect(aboutAction, &QAction::triggered, this, [=](){
+    connect(aboutAction, &QAction::triggered, this, [=]() {
         PeonyApplication::about();
     });
     addAction(aboutAction);
 
     auto newTabAction = new QAction(this);
     newTabAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_T));
-    connect(newTabAction, &QAction::triggered, this, [=](){
+    connect(newTabAction, &QAction::triggered, this, [=]() {
         this->addNewTabs(QStringList()<<this->getCurrentUri());
     });
     addAction(newTabAction);
 
     auto closeTabAction = new QAction(this);
     closeTabAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_W));
-    connect(closeTabAction, &QAction::triggered, this, [=](){
+    connect(closeTabAction, &QAction::triggered, this, [=]() {
         if (m_tab->count() <= 1) {
             this->close();
         } else {
@@ -273,7 +379,7 @@ void MainWindow::setShortCuts()
 
     auto nextTabAction = new QAction(this);
     nextTabAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Tab));
-    connect(nextTabAction, &QAction::triggered, this, [=](){
+    connect(nextTabAction, &QAction::triggered, this, [=]() {
         int currentIndex = m_tab->currentIndex();
         if (currentIndex + 1 < m_tab->count()) {
             m_tab->setCurrentIndex(currentIndex + 1);
@@ -285,7 +391,7 @@ void MainWindow::setShortCuts()
 
     auto previousTabAction = new QAction(this);
     previousTabAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Tab));
-    connect(previousTabAction, &QAction::triggered, this, [=](){
+    connect(previousTabAction, &QAction::triggered, this, [=]() {
         int currentIndex = m_tab->currentIndex();
         if (currentIndex > 0) {
             m_tab->setCurrentIndex(currentIndex - 1);
@@ -304,7 +410,7 @@ void MainWindow::setShortCuts()
     auto propertiesWindowAction = new QAction(this);
     propertiesWindowAction->setShortcuts(QList<QKeySequence>()<<QKeySequence(Qt::ALT + Qt::Key_Return)
                                          <<QKeySequence(Qt::ALT + Qt::Key_Enter));
-    connect(propertiesWindowAction, &QAction::triggered, this, [=](){
+    connect(propertiesWindowAction, &QAction::triggered, this, [=]() {
         //Fixed issue:when use this shortcut without any selections, this will crash
         if (getCurrentSelections().count() > 0)
         {
@@ -316,14 +422,14 @@ void MainWindow::setShortCuts()
 
     auto helpAction = new QAction(this);
     helpAction->setShortcut(QKeySequence(Qt::Key_F1));
-    connect(helpAction, &QAction::triggered, this, [=](){
+    connect(helpAction, &QAction::triggered, this, [=]() {
         PeonyApplication::help();
     });
     addAction(helpAction);
 
     auto maxAction = new QAction(this);
     maxAction->setShortcut(QKeySequence(Qt::Key_F11));
-    connect(maxAction, &QAction::triggered, this, [=](){
+    connect(maxAction, &QAction::triggered, this, [=]() {
         if (!this->isFullScreen()) {
             this->showFullScreen();
         } else {
@@ -334,7 +440,7 @@ void MainWindow::setShortCuts()
 
     auto previewPageAction = new QAction(this);
     previewPageAction->setShortcuts(QList<QKeySequence>()<<Qt::Key_F3<<QKeySequence(Qt::ALT + Qt::Key_P));
-    connect(previewPageAction, &QAction::triggered, this, [=](){
+    connect(previewPageAction, &QAction::triggered, this, [=]() {
         auto triggered = m_tab->getTriggeredPreviewPage();
         if (triggered)
         {
@@ -351,9 +457,75 @@ void MainWindow::setShortCuts()
     });
     addAction(previewPageAction);
 
+    auto refreshWindowAction = new QAction(this);
+    refreshWindowAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
+    connect(refreshWindowAction, &QAction::triggered, this, [=]() {
+        this->refresh();
+    });
+    addAction(refreshWindowAction);
+
+    auto listToIconViewAction = new QAction(this);
+    listToIconViewAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_1));
+    connect(listToIconViewAction, &QAction::triggered, this, [=]() {
+        this->beginSwitchView(QString("Icon View"));
+    });
+    addAction(listToIconViewAction);
+
+    auto iconToListViewAction = new QAction(this);
+    iconToListViewAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_2));
+    connect(iconToListViewAction, &QAction::triggered, this, [=]() {
+        this->beginSwitchView(QString("List View"));
+    });
+    addAction(iconToListViewAction);
+
+    auto reverseSelectAction = new QAction(this);
+    reverseSelectAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_L));
+    connect(reverseSelectAction, &QAction::triggered, this, [=]() {
+        this->getCurrentPage()->getView()->invertSelections();
+    });
+    addAction(reverseSelectAction);
+
+    auto remodelViewAction = new QAction(this);
+    remodelViewAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_0));
+    connect(remodelViewAction, &QAction::triggered, this, [=]() {
+        this->getCurrentPage()->setZoomLevelRequest(25);
+    });
+    addAction(remodelViewAction);
+
+    auto enlargViewAction = new QAction(this);
+    enlargViewAction->setShortcut(QKeySequence::ZoomIn);
+    connect(enlargViewAction, &QAction::triggered, this, [=]() {
+        int defaultZoomLevel = this->currentViewZoomLevel();
+        if(defaultZoomLevel <= 95){ defaultZoomLevel+=5; }
+        for (int i = 0; i < 5; i++) {
+            this->getCurrentPage()->setZoomLevelRequest(defaultZoomLevel);
+        }
+
+    });
+    addAction(enlargViewAction);
+
+    auto shrinkViewAction = new QAction(this);
+    shrinkViewAction->setShortcut(QKeySequence::ZoomOut);
+    connect(shrinkViewAction, &QAction::triggered, this, [=]() {
+        int defaultZoomLevel = this->currentViewZoomLevel();
+        if(defaultZoomLevel > 6){ defaultZoomLevel-=5; }
+        this->getCurrentPage()->setZoomLevelRequest(defaultZoomLevel);
+    });
+    addAction(shrinkViewAction);
+
+    auto quitAllAction = new QAction(this);
+    quitAllAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q));
+    connect(quitAllAction, &QAction::triggered, this, [=]() {
+        QProcess p(0);
+        p.start("peony", QStringList()<<"-q");
+        p.waitForStarted();
+        p.waitForFinished();
+    });
+    addAction(quitAllAction);
+
     auto refreshAction = new QAction(this);
     refreshAction->setShortcut(Qt::Key_F5);
-    connect(refreshAction, &QAction::triggered, this, [=](){
+    connect(refreshAction, &QAction::triggered, this, [=]() {
         this->refresh();
     });
     addAction(refreshAction);
@@ -361,15 +533,18 @@ void MainWindow::setShortCuts()
     //file operations
     auto *copyAction = new QAction(this);
     copyAction->setShortcut(QKeySequence::Copy);
-    connect(copyAction, &QAction::triggered, [=](){
+    connect(copyAction, &QAction::triggered, [=]() {
         if (!this->getCurrentSelections().isEmpty())
+            if (this->getCurrentSelections().first().startsWith("trash://", Qt::CaseInsensitive)) {
+                return ;
+            }
             Peony::ClipboardUtils::setClipboardFiles(this->getCurrentSelections(), false);
     });
     addAction(copyAction);
 
     auto *pasteAction = new QAction(this);
     pasteAction->setShortcut(QKeySequence::Paste);
-    connect(pasteAction, &QAction::triggered, [=](){
+    connect(pasteAction, &QAction::triggered, [=]() {
         if (Peony::ClipboardUtils::isClipboardHasFiles()) {
             //FIXME: how about duplicated copy?
             //FIXME: how to deal with a failed move?
@@ -380,8 +555,11 @@ void MainWindow::setShortCuts()
 
     auto *cutAction = new QAction(this);
     cutAction->setShortcut(QKeySequence::Cut);
-    connect(cutAction, &QAction::triggered, [=](){
+    connect(cutAction, &QAction::triggered, [=]() {
         if (!this->getCurrentSelections().isEmpty()) {
+            if (this->getCurrentSelections().first().startsWith("trash://", Qt::CaseInsensitive)) {
+                return ;
+            }
             Peony::ClipboardUtils::setClipboardFiles(this->getCurrentSelections(), true);
         }
     });
@@ -391,20 +569,21 @@ void MainWindow::setShortCuts()
 void MainWindow::updateTabPageTitle()
 {
     m_tab->updateTabPageTitle();
+    auto show = Peony::FileUtils::getFileDisplayName(getCurrentUri());
+    QString title = show + "-" + tr("File Manager");
+    //qDebug() << "updateTabPageTitle:" <<title;
+    setWindowTitle(title);
 }
 
 void MainWindow::createFolderOperation()
 {
     Peony::CreateTemplateOperation op(getCurrentUri(), Peony::CreateTemplateOperation::EmptyFolder, tr("New Folder"));
-    Peony::FileOperationErrorDialog dlg;
-    connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialog::handleError);
+    Peony::FileOperationErrorDialogConflict dlg;
+    connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialogConflict::handle);
     op.run();
     auto targetUri = op.target();
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-        QTimer::singleShot(500, this, [=](){
-#else
-        QTimer::singleShot(500, [=](){
-#endif
+
+    QTimer::singleShot(500, this, [=]() {
         this->getCurrentPage()->getView()->scrollToSelection(targetUri);
         this->editUri(targetUri);
     });
@@ -475,6 +654,33 @@ int MainWindow::getCurrentSortColumn()
     return m_tab->getSortType();
 }
 
+int MainWindow::currentViewZoomLevel()
+{
+    if (getCurrentPage()) {
+        if (auto view = getCurrentPage()->getView()) {
+            return view->currentZoomLevel();
+        }
+    }
+
+    int defaultZoomLevel = Peony::GlobalSettings::getInstance()->getValue(DEFAULT_VIEW_ZOOM_LEVEL).toInt();
+
+    if (defaultZoomLevel >= 0 && defaultZoomLevel <= 100) {
+        return defaultZoomLevel;
+    }
+
+    return m_tab->m_status_bar->m_slider->value();
+}
+
+bool MainWindow::currentViewSupportZoom()
+{
+    if (getCurrentPage()) {
+        if (auto view = getCurrentPage()->getView()) {
+            return view->supportZoom();
+        }
+    }
+    return m_tab->m_status_bar->m_slider->isEnabled();
+}
+
 void MainWindow::maximizeOrRestore()
 {
     if (!this->isMaximized()) {
@@ -494,7 +700,7 @@ void MainWindow::updateHeaderBar()
 {
     m_header_bar->setLocation(getCurrentUri());
     m_header_bar->updateIcons();
-    m_status_bar->update();
+    //m_status_bar->update();
 }
 
 void MainWindow::goToUri(const QString &uri, bool addHistory, bool force)
@@ -537,12 +743,17 @@ void MainWindow::beginSwitchView(const QString &viewId)
 //    int sortType = getCurrentSortColumn();
 //    Qt::SortOrder sortOrder = getCurrentSortOrder();
     m_tab->switchViewType(viewId);
+    // save zoom level
+    Peony::GlobalSettings::getInstance()->setValue(DEFAULT_VIEW_ZOOM_LEVEL, currentViewZoomLevel());
     m_tab->setCurrentSelections(selection);
+    m_tab->m_status_bar->m_slider->setEnabled(m_tab->currentPage()->getView()->supportZoom());
 }
 
 void MainWindow::refresh()
 {
-    goToUri(getCurrentUri(), false, true);
+    locationChangeStart();
+    m_tab->refresh();
+    //goToUri(getCurrentUri(), false, true);
 }
 
 void MainWindow::advanceSearch()
@@ -603,6 +814,9 @@ void MainWindow::forceStopLoading()
 void MainWindow::setCurrentSelectionUris(const QStringList &uris)
 {
     m_tab->setCurrentSelections(uris);
+    if (uris.isEmpty())
+        return;
+    getCurrentPage()->getView()->scrollToSelection(uris.first());
 }
 
 void MainWindow::setCurrentSortOrder(Qt::SortOrder order)
@@ -625,12 +839,23 @@ void MainWindow::editUris(const QStringList &uris)
     m_tab->editUris(uris);
 }
 
+void MainWindow::setCurrentViewZoomLevel(int zoomLevel)
+{
+    if (currentViewSupportZoom())
+        m_tab->m_status_bar->m_slider->setValue(zoomLevel);
+}
+
 void MainWindow::resizeEvent(QResizeEvent *e)
 {
+    QMainWindow::resizeEvent(e);
     m_header_bar->updateMaximizeState();
     validBorder();
     update();
-    QMainWindow::resizeEvent(e);
+
+    if (m_resize_handler->isButtonDown()) {
+        // set save window size flag
+        last_resize_window = this;
+    }
 }
 
 /*!
@@ -644,10 +869,39 @@ void MainWindow::paintEvent(QPaintEvent *e)
     QColor color = this->palette().window().color();
     QColor colorBase = this->palette().base().color();
 
+    int R1 = color.red();
+    int G1 = color.green();
+    int B1 = color.blue();
+    qreal a1 = 0.3;
+
+    int R2 = colorBase.red();
+    int G2 = colorBase.green();
+    int B2 = colorBase.blue();
+    qreal a2 = 1;
+
+    qreal a = 1 - (1 - a1)*(1 - a2);
+
+    qreal R = (a1*R1 + (1 - a1)*a2*R2) / a;
+    qreal G = (a1*G1 + (1 - a1)*a2*G2) / a;
+    qreal B = (a1*B1 + (1 - a1)*a2*B2) / a;
+
+    colorBase.setRed(R);
+    colorBase.setGreen(G);
+    colorBase.setBlue(B);
+
     auto sidebarOpacity = Peony::GlobalSettings::getInstance()->getValue(SIDEBAR_BG_OPACITY).toInt();
 
     colorBase.setAlphaF(sidebarOpacity/100.0);
 
+    if (qApp->property("blurEnable").isValid()) {
+        bool blurEnable = qApp->property("blurEnable").toBool();
+        if (!blurEnable) {
+            colorBase.setAlphaF(1);
+        }
+    } else {
+        colorBase.setAlphaF(1);
+    }
+
     QPainterPath sidebarPath;
     sidebarPath.setFillRule(Qt::FillRule::WindingFill);
     auto adjustedRect = sideBarRect().adjusted(0, 1, 0, 0);
@@ -669,8 +923,10 @@ void MainWindow::mousePressEvent(QMouseEvent *e)
 {
     //qDebug()<<"mouse pressed"<<e;
     QMainWindow::mousePressEvent(e);
-    if (e->button() == Qt::LeftButton && !e->isAccepted())
+    if (e->button() == Qt::LeftButton && !e->isAccepted()) {
         m_is_draging = true;
+        m_offset = mapFromGlobal(QCursor::pos());
+    }
 }
 
 void MainWindow::mouseMoveEvent(QMouseEvent *e)
@@ -682,30 +938,57 @@ void MainWindow::mouseMoveEvent(QMouseEvent *e)
     QMainWindow::mouseMoveEvent(e);
     if (!m_is_draging)
         return;
-    Display *display = QX11Info::display();
-    Atom netMoveResize = XInternAtom(display, "_NET_WM_MOVERESIZE", False);
-    XEvent xEvent;
-    const auto pos = QCursor::pos();
 
-    memset(&xEvent, 0, sizeof(XEvent));
-    xEvent.xclient.type = ClientMessage;
-    xEvent.xclient.message_type = netMoveResize;
-    xEvent.xclient.display = display;
-    xEvent.xclient.window = this->winId();
-    xEvent.xclient.format = 32;
-    xEvent.xclient.data.l[0] = pos.x();
-    xEvent.xclient.data.l[1] = pos.y();
-    xEvent.xclient.data.l[2] = 8;
-    xEvent.xclient.data.l[3] = Button1;
-    xEvent.xclient.data.l[4] = 0;
+    if (QX11Info::isPlatformX11()) {
+        Display *display = QX11Info::display();
+        Atom netMoveResize = XInternAtom(display, "_NET_WM_MOVERESIZE", False);
+        XEvent xEvent;
+        const auto pos = QCursor::pos();
 
-    XUngrabPointer(display, CurrentTime);
-    XSendEvent(display, QX11Info::appRootWindow(QX11Info::appScreen()),
-               False, SubstructureNotifyMask | SubstructureRedirectMask,
-               &xEvent);
-    XFlush(display);
+        memset(&xEvent, 0, sizeof(XEvent));
+        xEvent.xclient.type = ClientMessage;
+        xEvent.xclient.message_type = netMoveResize;
+        xEvent.xclient.display = display;
+        xEvent.xclient.window = this->winId();
+        xEvent.xclient.format = 32;
+        xEvent.xclient.data.l[0] = pos.x();
+        xEvent.xclient.data.l[1] = pos.y();
+        xEvent.xclient.data.l[2] = 8;
+        xEvent.xclient.data.l[3] = Button1;
+        xEvent.xclient.data.l[4] = 0;
 
-    m_is_draging = false;
+        XUngrabPointer(display, CurrentTime);
+        XSendEvent(display, QX11Info::appRootWindow(QX11Info::appScreen()),
+                   False, SubstructureNotifyMask | SubstructureRedirectMask,
+                   &xEvent);
+        //XFlush(display);
+
+        XEvent xevent;
+        memset(&xevent, 0, sizeof(XEvent));
+
+        xevent.type = ButtonRelease;
+        xevent.xbutton.button = Button1;
+        xevent.xbutton.window = this->winId();
+        xevent.xbutton.x = e->pos().x();
+        xevent.xbutton.y = e->pos().y();
+        xevent.xbutton.x_root = pos.x();
+        xevent.xbutton.y_root = pos.y();
+        xevent.xbutton.display = display;
+
+        XSendEvent(display, this->effectiveWinId(), False, ButtonReleaseMask, &xevent);
+        XFlush(display);
+
+        if (e->source() == Qt::MouseEventSynthesizedByQt) {
+            if (!this->mouseGrabber()) {
+                this->grabMouse();
+                this->releaseMouse();
+            }
+        }
+
+        m_is_draging = false;
+    } else {
+        this->move(QCursor::pos() - m_offset);
+    }
 }
 
 void MainWindow::mouseReleaseEvent(QMouseEvent *e)
@@ -726,6 +1009,7 @@ void MainWindow::validBorder()
         setContentsMargins(0, 0, 0, 0);
         m_effect->setPadding(0);
         setProperty("blurRegion", QVariant());
+        KWindowEffects::enableBlurBehind(this->winId(), true);
     } else {
         setContentsMargins(4, 4, 4, 4);
         m_effect->setPadding(4);
@@ -734,12 +1018,14 @@ void MainWindow::validBorder()
         rect.adjust(4, 4, -4, -4);
         path.addRoundedRect(rect, 6, 6);
         setProperty("blurRegion", QRegion(path.toFillPolygon().toPolygon()));
+        //use KWindowEffects
+        KWindowEffects::enableBlurBehind(this->winId(), true, QRegion(path.toFillPolygon().toPolygon()));
     }
 }
 
 void MainWindow::initUI(const QString &uri)
 {
-    connect(this, &MainWindow::locationChangeStart, this, [=](){
+    connect(this, &MainWindow::locationChangeStart, this, [=]() {
         m_side_bar->blockSignals(true);
         m_header_bar->blockSignals(true);
         QCursor c;
@@ -747,10 +1033,10 @@ void MainWindow::initUI(const QString &uri)
         this->setCursor(c);
         m_tab->setCursor(c);
         m_side_bar->setCursor(c);
-        m_status_bar->update();
+        //m_status_bar->update();
     });
 
-    connect(this, &MainWindow::locationChangeEnd, this, [=](){
+    connect(this, &MainWindow::locationChangeEnd, this, [=]() {
         m_side_bar->blockSignals(false);
         m_header_bar->blockSignals(false);
         QCursor c;
@@ -759,7 +1045,7 @@ void MainWindow::initUI(const QString &uri)
         m_tab->setCursor(c);
         m_side_bar->setCursor(c);
         updateHeaderBar();
-        m_status_bar->update();
+        //m_status_bar->update();
     });
 
     //HeaderBar
@@ -772,6 +1058,14 @@ void MainWindow::initUI(const QString &uri)
 
     connect(m_header_bar, &HeaderBar::updateLocationRequest, this, &MainWindow::goToUri);
     connect(m_header_bar, &HeaderBar::viewTypeChangeRequest, this, &MainWindow::beginSwitchView);
+    connect(m_header_bar, &HeaderBar::updateZoomLevelHintRequest, this, [=](int zoomLevelHint) {
+        if (zoomLevelHint >= 0) {
+            m_tab->m_status_bar->m_slider->setEnabled(true);
+            m_tab->m_status_bar->m_slider->setValue(zoomLevelHint);
+        } else {
+            m_tab->m_status_bar->m_slider->setEnabled(false);
+        }
+    });
 
     //SideBar
     QDockWidget *sidebarContainer = new QDockWidget(this);
@@ -829,51 +1123,73 @@ void MainWindow::initUI(const QString &uri)
     sidebarContainer->setWidget(splitter);
     addDockWidget(Qt::LeftDockWidgetArea, sidebarContainer);
 
-    m_status_bar = new Peony::StatusBar(this, this);
-    setStatusBar(m_status_bar);
+//    m_status_bar = new Peony::StatusBar(this, this);
+//    setStatusBar(m_status_bar);
 
     auto views = new TabWidget;
     m_tab = views;
     if (uri.isNull()) {
         auto home = "file://" + QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
-        m_tab->addPage(home);
+        m_tab->addPage(home, true);
     } else {
         m_tab->addPage(uri, true);
+        m_header_bar->setLocation(uri);
     }
+    QTimer::singleShot(1, this, [=]() {
+        // FIXME:
+        // it is strange that if we set size hint by qsettings,
+        // the tab bar will "shrink" until we update geometry
+        // after a while (may resize window or move splitter).
+        // we should find out the reason and remove this dirty
+        // code.
+        m_tab->updateTabBarGeometry();
+        m_tab->updateStatusBarGeometry();
+    });
 
-    connect(views->tabBar(), &QTabBar::tabBarDoubleClicked, this, [=](int index){
+    connect(views->tabBar(), &QTabBar::tabBarDoubleClicked, this, [=](int index) {
         if (index == -1)
             maximizeOrRestore();
     });
     connect(views, &TabWidget::closeWindowRequest, this, &QWidget::close);
+    connect(m_header_bar, &HeaderBar::updateSearchRequest, m_tab, &TabWidget::updateSearchBar);
 
     X11WindowManager *tabBarHandler = X11WindowManager::getInstance();
     tabBarHandler->registerWidget(views->tabBar());
 
     setCentralWidget(views);
 
+    // check slider zoom level
+    if (currentViewSupportZoom())
+        setCurrentViewZoomLevel(currentViewZoomLevel());
+
     //bind signals
+    connect(m_tab, &TabWidget::searchRecursiveChanged, headerBar, &HeaderBar::updateSearchRecursive);
+    connect(m_tab, &TabWidget::closeSearch, headerBar, &HeaderBar::closeSearch);
     connect(m_tab, &TabWidget::clearTrash, this, &MainWindow::cleanTrash);
     connect(m_tab, &TabWidget::recoverFromTrash, this, &MainWindow::recoverFromTrash);
     connect(m_tab, &TabWidget::updateWindowLocationRequest, this, &MainWindow::goToUri);
     connect(m_tab, &TabWidget::activePageLocationChanged, this, &MainWindow::locationChangeEnd);
     connect(m_tab, &TabWidget::activePageViewTypeChanged, this, &MainWindow::updateHeaderBar);
     connect(m_tab, &TabWidget::activePageChanged, this, &MainWindow::updateHeaderBar);
-    connect(m_tab, &TabWidget::menuRequest, this, [=](){
+    connect(m_tab, &TabWidget::activePageChanged, this, [=](){
+        // check slider zoom level
+        setCurrentViewZoomLevel(currentViewZoomLevel());
+    });
+    connect(m_tab, &TabWidget::menuRequest, this, [=]() {
         Peony::DirectoryViewMenu menu(this);
         menu.exec(QCursor::pos());
         auto urisToEdit = menu.urisToEdit();
-        if (!urisToEdit.isEmpty()) {
-#if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-            QTimer::singleShot(100, this, [=](){
-#else
-            QTimer::singleShot(100, [=](){
-#endif
+        if (!urisToEdit.isEmpty())
+        {
+            QTimer::singleShot(100, this, [=]() {
                 this->getCurrentPage()->getView()->scrollToSelection(urisToEdit.first());
                 this->editUri(urisToEdit.first());
             });
         }
     });
+//    connect(m_tab, &TabWidget::currentSelectionChanged, this, [=](){
+//        m_status_bar->update();
+//    });
 }
 
 void MainWindow::cleanTrash()
